@@ -29,8 +29,10 @@
 #include "winreg.h"
 #include "winternl.h"
 #include "winerror.h"
+#include "wincred.h"
 
 #include "wine/library.h"
+#include "wine/unicode.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(advapi);
@@ -53,24 +55,20 @@ GetUserNameA( LPSTR lpszName, LPDWORD lpSize )
 {
     WCHAR *buffer;
     BOOL ret;
-    DWORD sizeW = *lpSize * 2;
+    DWORD sizeW = *lpSize;
 
     if (!(buffer = HeapAlloc( GetProcessHeap(), 0, sizeW * sizeof(WCHAR) )))
     {
         SetLastError( ERROR_NOT_ENOUGH_MEMORY );
         return FALSE;
     }
+
     ret = GetUserNameW( buffer, &sizeW );
     if (ret)
-    {
-        if (!(*lpSize = WideCharToMultiByte( CP_ACP, 0, buffer, -1, lpszName, *lpSize, NULL, NULL )))
-        {
-            *lpSize = WideCharToMultiByte( CP_ACP, 0, buffer, -1, NULL, 0, NULL, NULL );
-            SetLastError( ERROR_MORE_DATA );
-            ret = FALSE;
-        }
-    }
-    else *lpSize = sizeW * 2;
+        *lpSize = WideCharToMultiByte( CP_ACP, 0, buffer, -1, lpszName, *lpSize, NULL, NULL );
+    else
+        *lpSize = sizeW;
+
     HeapFree( GetProcessHeap(), 0, buffer );
     return ret;
 }
@@ -85,10 +83,11 @@ GetUserNameW( LPWSTR lpszName, LPDWORD lpSize )
 {
     const char *name = wine_get_user_name();
     DWORD i, len = MultiByteToWideChar( CP_UNIXCP, 0, name, -1, NULL, 0 );
+    LPWSTR backslash;
 
     if (len > *lpSize)
     {
-        SetLastError(ERROR_MORE_DATA);
+        SetLastError( ERROR_INSUFFICIENT_BUFFER );
         *lpSize = len;
         return FALSE;
     }
@@ -98,9 +97,22 @@ GetUserNameW( LPWSTR lpszName, LPDWORD lpSize )
 
     /* Word uses the user name to create named mutexes and file mappings,
      * and backslashes in the name cause the creation to fail.
+     * Also, Windows doesn't return the domain name in the user name even when
+     * joined to a domain. A Unix box joined to a domain using winbindd will
+     * contain the domain name in the username. So we need to cut this off.
+     * FIXME: Only replaces forward and backslashes for now, should get the
+     * winbind separator char from winbindd and replace that.
      */
     for (i = 0; lpszName[i]; i++)
-        if (lpszName[i] == '\\' || lpszName[i] == '/') lpszName[i] = '_';
+        if (lpszName[i] == '/') lpszName[i] = '\\';
+
+    backslash = strrchrW(lpszName, '\\');
+    if (backslash == NULL)
+        return TRUE;
+
+    len = lstrlenW(backslash);
+    memmove(lpszName, backslash + 1, len * sizeof(WCHAR));
+    *lpSize = len;
     return TRUE;
 }
 
@@ -286,7 +298,8 @@ DWORD WINAPI CommandLineFromMsiDescriptor( WCHAR *szDescriptor,
     hmsi = LoadLibraryW( szMsi );
     if (!hmsi)
         return r;
-    mpcfd = (void*) GetProcAddress( hmsi, "MsiProvideComponentFromDescriptorW" );
+    mpcfd = (fnMsiProvideComponentFromDescriptor)GetProcAddress( hmsi,
+                                                                 "MsiProvideComponentFromDescriptorW" );
     if (mpcfd)
         r = mpcfd( szDescriptor, szCommandLine, pcchCommandLine, NULL );
     FreeLibrary( hmsi );
