@@ -294,6 +294,7 @@ static BOOL elf_map_file(struct elf_map_file_data* emfd, struct image_file_map*
         filename = NULL;
         break;
     default: assert(0);
+        return FALSE;
     }
 
     elf_reset_file_map(fmap);
@@ -767,28 +768,6 @@ static int elf_new_wine_thunks(struct module* module, const struct hash_table* h
                  */
                 module->sortlist_valid = TRUE;
             }
-            else if (strcmp(ste->ht_elt.name, symt->hash_elt.name))
-            {
-                ULONG64 xaddr = 0, xsize = 0;
-                DWORD   kind = -1;
-
-                symt_get_address(&symt->symt, &xaddr);
-                symt_get_info(module, &symt->symt, TI_GET_LENGTH,   &xsize);
-                symt_get_info(module, &symt->symt, TI_GET_DATAKIND, &kind);
-
-                /* If none of symbols has a correct size, we consider they are both markers
-                 * Hence, we can silence this warning
-                 * Also, we check that we don't have two symbols, one local, the other 
-                 * global which is legal
-                 */
-                if ((xsize || ste->symp->st_size) &&
-                    (kind == (ELF32_ST_BIND(ste->symp->st_info) == STB_LOCAL) ? DataIsFileStatic : DataIsGlobal))
-                    FIXME("Duplicate in %s: %s<%08lx-%08x> %s<%s-%s>\n",
-                          debugstr_w(module->module.ModuleName),
-                          ste->ht_elt.name, addr, (unsigned int)ste->symp->st_size,
-                          symt->hash_elt.name,
-                          wine_dbgstr_longlong(xaddr), wine_dbgstr_longlong(xsize));
-            }
         }
     }
     /* see comment above */
@@ -1012,7 +991,6 @@ static BOOL elf_load_debug_info_from_map(struct module* module,
                     WARN("Couldn't correctly read stabs\n");
                 ret = ret || lret;
             }
-            else lret = FALSE;
             image_unmap_section(&stab_sect);
             image_unmap_section(&stabstr_sect);
         }
