@@ -233,7 +233,7 @@ static void _dump_EnumDevices_dwFlags(DWORD dwFlags) {
     TRACE("\n");
 }
 
-static void _dump_diactionformatA(LPDIACTIONFORMATA lpdiActionFormat) {
+void _dump_diactionformatA(LPDIACTIONFORMATA lpdiActionFormat) {
     unsigned int i;
 
     FIXME("diaf.dwSize = %d\n", lpdiActionFormat->dwSize);
@@ -317,38 +317,6 @@ void _copy_diactionformatWtoA(LPDIACTIONFORMATA to, LPDIACTIONFORMATW from)
     }
 }
 
-/* _diactionformat_priority
- *
- *  Given a DIACTIONFORMAT structure and a DI genre, returns the enumeration
- *  priority. Joysticks should pass the game genre, and mouse or keyboard their
- *  respective DI*_MASK
- */
-static DWORD _diactionformat_priorityA(LPDIACTIONFORMATA lpdiaf, DWORD genre)
-{
-    int i;
-    DWORD priorityFlags = 0;
-
-    /* If there's at least one action for the device it's priority 1 */
-    for(i=0; i < lpdiaf->dwActionSize; i++)
-        if ((lpdiaf->rgoAction[i].dwSemantic & genre) == genre)
-            priorityFlags |= DIEDBS_MAPPEDPRI1;
-
-    return priorityFlags;
-}
-
-static DWORD _diactionformat_priorityW(LPDIACTIONFORMATW lpdiaf, DWORD genre)
-{
-    int i;
-    DWORD priorityFlags = 0;
-
-    /* If there's at least one action for the device it's priority 1 */
-    for(i=0; i < lpdiaf->dwActionSize; i++)
-        if ((lpdiaf->rgoAction[i].dwSemantic & genre) == genre)
-            priorityFlags |= DIEDBS_MAPPEDPRI1;
-
-    return priorityFlags;
-}
-
 /******************************************************************************
  *	IDirectInputA_EnumDevices
  */
@@ -874,14 +842,12 @@ static HRESULT WINAPI IDirectInput8AImpl_EnumDevicesBySemantics(
       LPVOID pvRef, DWORD dwFlags
 )
 {
-    static REFGUID guids[2] = { &GUID_SysKeyboard, &GUID_SysMouse };
-    static const DWORD actionMasks[] = { DIKEYBOARD_MASK, DIMOUSE_MASK };
     IDirectInputImpl *This = impl_from_IDirectInput8A( iface );
     DIDEVICEINSTANCEA didevi;
     LPDIRECTINPUTDEVICE8A lpdid;
-    DWORD callbackFlags;
-    int i, j;
-
+    BOOL ret;
+    DWORD callbackFlags = 0;
+    int i;
 
     FIXME("(this=%p,%s,%p,%p,%p,%04x): semi-stub\n", This, ptszUserName, lpdiActionFormat,
           lpCallback, pvRef, dwFlags);
@@ -898,43 +864,29 @@ static HRESULT WINAPI IDirectInput8AImpl_EnumDevicesBySemantics(
 
     didevi.dwSize = sizeof(didevi);
 
-    /* Enumerate all the joysticks */
-    for (i = 0; i < NB_DINPUT_DEVICES; i++)
-    {
-        BOOL enumSuccess;
+    /* enum the keyboard first */
+    IDirectInput_CreateDevice(iface, &GUID_SysKeyboard, &lpdid, NULL);
+    IDirectInputDevice_GetDeviceInfo(lpdid, &didevi);
+    /* if there's any DIKEYBOARD action, keyboard is priority 1*/
+    for(i=0; i < lpdiActionFormat->dwActionSize; i++)
+        if ((lpdiActionFormat->rgoAction[i].dwSemantic & DIKEYBOARD_MASK) == DIKEYBOARD_MASK)
+            callbackFlags |= DIEDBS_MAPPEDPRI1;
 
-        if (!dinput_devices[i]->enum_deviceA) continue;
+    ret = lpCallback(&didevi, lpdid, callbackFlags, 1, pvRef);
 
-        for (j = 0, enumSuccess = -1; enumSuccess != 0; j++)
-        {
-            TRACE(" - checking device %u ('%s')\n", i, dinput_devices[i]->name);
+    if (ret == DIENUM_STOP) return DI_OK;
 
-            callbackFlags = _diactionformat_priorityA(lpdiActionFormat, lpdiActionFormat->dwGenre);
-            /* Default behavior is to enumerate attached game controllers */
-            enumSuccess = dinput_devices[i]->enum_deviceA(DI8DEVCLASS_GAMECTRL, DIEDFL_ATTACHEDONLY | dwFlags, &didevi, This->dwVersion, j);
-            if (enumSuccess)
-            {
-                IDirectInput_CreateDevice(iface, &didevi.guidInstance, &lpdid, NULL);
+    callbackFlags = 0;
 
-                if (lpCallback(&didevi, lpdid, callbackFlags, 0, pvRef) == DIENUM_STOP)
-                    return DI_OK;
-            }
-        }
-    }
+    /* and then the mouse */
+    IDirectInput_CreateDevice(iface, &GUID_SysMouse, &lpdid, NULL);
+    IDirectInputDevice_GetDeviceInfo(lpdid, &didevi);
+    /* same as above, but for the mouse */
+    for(i=0; i < lpdiActionFormat->dwActionSize; i++)
+        if ((lpdiActionFormat->rgoAction[i].dwSemantic & DIMOUSE_MASK) == DIMOUSE_MASK)
+            callbackFlags |= DIEDBS_MAPPEDPRI1;
 
-    if (dwFlags & DIEDBSFL_FORCEFEEDBACK) return DI_OK;
-
-    /* Enumerate keyboard and mouse */
-    for(i=0; i < sizeof(guids)/sizeof(guids[0]); i++)
-    {
-        callbackFlags = _diactionformat_priorityA(lpdiActionFormat, actionMasks[i]);
-
-        IDirectInput_CreateDevice(iface, guids[i], &lpdid, NULL);
-        IDirectInputDevice_GetDeviceInfo(lpdid, &didevi);
-
-        if (lpCallback(&didevi, lpdid, callbackFlags, sizeof(guids)/sizeof(guids[0]) - (i+1), pvRef) == DIENUM_STOP)
-            return DI_OK;
-    }
+    lpCallback(&didevi, lpdid, callbackFlags, 0, pvRef);
 
     return DI_OK;
 }
@@ -945,119 +897,67 @@ static HRESULT WINAPI IDirectInput8WImpl_EnumDevicesBySemantics(
       LPVOID pvRef, DWORD dwFlags
 )
 {
-    static REFGUID guids[2] = { &GUID_SysKeyboard, &GUID_SysMouse };
-    static const DWORD actionMasks[] = { DIKEYBOARD_MASK, DIMOUSE_MASK };
     IDirectInputImpl *This = impl_from_IDirectInput8W(iface);
     DIDEVICEINSTANCEW didevi;
     LPDIRECTINPUTDEVICE8W lpdid;
-    DWORD callbackFlags;
-    int i, j;
+    BOOL ret;
+    DWORD callbackFlags = 0;
+    int i;
 
     FIXME("(this=%p,%s,%p,%p,%p,%04x): semi-stub\n", This, debugstr_w(ptszUserName), lpdiActionFormat,
           lpCallback, pvRef, dwFlags);
 
     didevi.dwSize = sizeof(didevi);
 
-    /* Enumerate all the joysticks */
-    for (i = 0; i < NB_DINPUT_DEVICES; i++)
-    {
-        BOOL enumSuccess;
+    /* enum the keyboard first */
+    IDirectInput_CreateDevice(iface, &GUID_SysKeyboard, &lpdid, NULL);
+    IDirectInputDevice_GetDeviceInfo(lpdid, &didevi);
+    /* if there's any DIKEYBOARD action, keyboard is priority 1*/
+    for(i=0; i < lpdiActionFormat->dwActionSize; i++)
+        if ((lpdiActionFormat->rgoAction[i].dwSemantic & DIKEYBOARD_MASK) == DIKEYBOARD_MASK)
+            callbackFlags |= DIEDBS_MAPPEDPRI1;
 
-        if (!dinput_devices[i]->enum_deviceW) continue;
+    ret = lpCallback(&didevi, lpdid, callbackFlags, 1, pvRef);
 
-        for (j = 0, enumSuccess = -1; enumSuccess != 0; j++)
-        {
-            TRACE(" - checking device %u ('%s')\n", i, dinput_devices[i]->name);
+    if (ret == DIENUM_STOP) return DI_OK;
 
-            callbackFlags = _diactionformat_priorityW(lpdiActionFormat, lpdiActionFormat->dwGenre);
-            /* Default behavior is to enumerate attached game controllers */
-            enumSuccess = dinput_devices[i]->enum_deviceW(DI8DEVCLASS_GAMECTRL, DIEDFL_ATTACHEDONLY | dwFlags, &didevi, This->dwVersion, j);
-            if (enumSuccess)
-            {
-                IDirectInput_CreateDevice(iface, &didevi.guidInstance, &lpdid, NULL);
+    callbackFlags = 0;
 
-                if (lpCallback(&didevi, lpdid, callbackFlags, 0, pvRef) == DIENUM_STOP)
-                    return DI_OK;
-            }
-        }
-    }
+    /* and then the mouse */
+    IDirectInput_CreateDevice(iface, &GUID_SysMouse, &lpdid, NULL);
+    IDirectInputDevice_GetDeviceInfo(lpdid, &didevi);
+    /* same as above, but for the mouse */
+    for(i=0; i < lpdiActionFormat->dwActionSize; i++)
+        if ((lpdiActionFormat->rgoAction[i].dwSemantic & DIMOUSE_MASK) == DIMOUSE_MASK)
+            callbackFlags |= DIEDBS_MAPPEDPRI1;
 
-    if (dwFlags & DIEDBSFL_FORCEFEEDBACK) return DI_OK;
-
-    /* Enumerate keyboard and mouse */
-    for(i=0; i < sizeof(guids)/sizeof(guids[0]); i++)
-    {
-        callbackFlags = _diactionformat_priorityW(lpdiActionFormat, actionMasks[i]);
-
-        IDirectInput_CreateDevice(iface, guids[i], &lpdid, NULL);
-        IDirectInputDevice_GetDeviceInfo(lpdid, &didevi);
-
-        if (lpCallback(&didevi, lpdid, callbackFlags, sizeof(guids)/sizeof(guids[0]) - (i+1), pvRef) == DIENUM_STOP)
-            return DI_OK;
-    }
+    lpCallback(&didevi, lpdid, callbackFlags, 0, pvRef);
 
     return DI_OK;
 }
 
-static HRESULT WINAPI IDirectInput8WImpl_ConfigureDevices(
-      LPDIRECTINPUT8W iface, LPDICONFIGUREDEVICESCALLBACK lpdiCallback,
-      LPDICONFIGUREDEVICESPARAMSW lpdiCDParams, DWORD dwFlags, LPVOID pvRefData
-)
-{
-    IDirectInputImpl *This = impl_from_IDirectInput8W(iface);
-
-    FIXME("(this=%p,%p,%p,%04x,%p): stub\n", This, lpdiCallback, lpdiCDParams, dwFlags, pvRefData);
-
-    /* Call helper function in config.c to do the real work */
-    return _configure_devices(iface, lpdiCallback, lpdiCDParams, dwFlags, pvRefData);
-}
-
 static HRESULT WINAPI IDirectInput8AImpl_ConfigureDevices(
       LPDIRECTINPUT8A iface, LPDICONFIGUREDEVICESCALLBACK lpdiCallback,
       LPDICONFIGUREDEVICESPARAMSA lpdiCDParams, DWORD dwFlags, LPVOID pvRefData
 )
 {
-    IDirectInputImpl *This = impl_from_IDirectInput8A(iface);
-    DIACTIONFORMATW diafW;
-    DICONFIGUREDEVICESPARAMSW diCDParamsW;
-    HRESULT hr;
-    int i;
+      IDirectInputImpl *This = impl_from_IDirectInput8A( iface );
 
-     FIXME("(this=%p,%p,%p,%04x,%p): stub\n", This, lpdiCallback, lpdiCDParams, dwFlags, pvRefData);
+      FIXME("(this=%p,%p,%p,%04x,%p): stub\n", This, lpdiCallback, lpdiCDParams,
+            dwFlags, pvRefData);
+      return 0;
+}
 
-    /* Copy parameters */
-    diCDParamsW.dwSize = sizeof(DICONFIGUREDEVICESPARAMSW);
-    diCDParamsW.dwcFormats = lpdiCDParams->dwcFormats;
-    diCDParamsW.lprgFormats = &diafW;
-    diCDParamsW.hwnd = lpdiCDParams->hwnd;
+static HRESULT WINAPI IDirectInput8WImpl_ConfigureDevices(
+      LPDIRECTINPUT8W iface, LPDICONFIGUREDEVICESCALLBACK lpdiCallback,
+      LPDICONFIGUREDEVICESPARAMSW lpdiCDParams, DWORD dwFlags, LPVOID pvRefData
+)
+{
+      IDirectInputImpl *This = impl_from_IDirectInput8W( iface );
 
-    diafW.rgoAction = HeapAlloc(GetProcessHeap(), 0, sizeof(DIACTIONW)*lpdiCDParams->lprgFormats->dwNumActions);
-    _copy_diactionformatAtoW(&diafW, lpdiCDParams->lprgFormats);
-
-    /* Copy action names */
-    for (i=0; i < diafW.dwNumActions; i++)
-    {
-        const char* from = lpdiCDParams->lprgFormats->rgoAction[i].u.lptszActionName;
-        int len = MultiByteToWideChar(CP_ACP, 0, from , -1, NULL , 0);
-        WCHAR *to = HeapAlloc(GetProcessHeap(), 0, sizeof(WCHAR)*len);
-
-        MultiByteToWideChar(CP_ACP, 0, from , -1, to , len);
-        diafW.rgoAction[i].u.lptszActionName = to;
-    }
-
-    hr = IDirectInput8WImpl_ConfigureDevices(&This->IDirectInput8W_iface, lpdiCallback, &diCDParamsW, dwFlags, pvRefData);
-
-    /* Copy back configuration */
-    if (SUCCEEDED(hr))
-        _copy_diactionformatWtoA(lpdiCDParams->lprgFormats, &diafW);
-
-    /* Free memory */
-    for (i=0; i < diafW.dwNumActions; i++)
-        HeapFree(GetProcessHeap(), 0, (void*) diafW.rgoAction[i].u.lptszActionName);
-
-    HeapFree(GetProcessHeap(), 0, diafW.rgoAction);
-
-    return hr;
+      FIXME("(this=%p,%p,%p,%04x,%p): stub\n", This, lpdiCallback, lpdiCDParams,
+            dwFlags, pvRefData);
+      return 0;
 }
 
 static const IDirectInput7AVtbl ddi7avt = {
@@ -1211,7 +1111,7 @@ HRESULT WINAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID *ppv)
  */
 HRESULT WINAPI DllRegisterServer(void)
 {
-    return __wine_register_resources( DINPUT_instance );
+    return __wine_register_resources( DINPUT_instance, NULL );
 }
 
 /***********************************************************************
@@ -1219,7 +1119,7 @@ HRESULT WINAPI DllRegisterServer(void)
  */
 HRESULT WINAPI DllUnregisterServer(void)
 {
-    return __wine_unregister_resources( DINPUT_instance );
+    return __wine_unregister_resources( DINPUT_instance, NULL );
 }
 
 /******************************************************************************
