@@ -37,6 +37,8 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(dsound);
 
+static HRESULT SecondaryBufferImpl_Destroy(SecondaryBufferImpl *pdsb);
+
 /*******************************************************************************
  *		IDirectSoundNotify
  */
@@ -182,36 +184,22 @@ static HRESULT IDirectSoundNotifyImpl_Destroy(
  *		IDirectSoundBuffer
  */
 
-static inline IDirectSoundBufferImpl *impl_from_IDirectSoundBuffer8(IDirectSoundBuffer8 *iface)
-{
-    return CONTAINING_RECORD(iface, IDirectSoundBufferImpl, IDirectSoundBuffer8_iface);
+static HRESULT WINAPI IDirectSoundBufferImpl_SetFormat(
+	LPDIRECTSOUNDBUFFER8 iface,LPCWAVEFORMATEX wfex
+) {
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
+
+	TRACE("(%p,%p)\n",This,wfex);
+	/* This method is not available on secondary buffers */
+	WARN("invalid call\n");
+	return DSERR_INVALIDCALL;
 }
 
-static inline BOOL is_primary_buffer(IDirectSoundBufferImpl *This)
-{
-    return This->dsbd.dwFlags & DSBCAPS_PRIMARYBUFFER ? TRUE : FALSE;
-}
-
-static HRESULT WINAPI IDirectSoundBufferImpl_SetFormat(IDirectSoundBuffer8 *iface,
-        LPCWAVEFORMATEX wfex)
-{
-    IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
-
-    TRACE("(%p,%p)\n", iface, wfex);
-
-    if (is_primary_buffer(This))
-        return primarybuffer_SetFormat(This->device, wfex);
-    else {
-        WARN("not available for secondary buffers.\n");
-        return DSERR_INVALIDCALL;
-    }
-}
-
-static HRESULT WINAPI IDirectSoundBufferImpl_SetVolume(IDirectSoundBuffer8 *iface, LONG vol)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
+static HRESULT WINAPI IDirectSoundBufferImpl_SetVolume(
+	LPDIRECTSOUNDBUFFER8 iface,LONG vol
+) {
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	LONG oldVol;
-
 	HRESULT hres = DS_OK;
 
 	TRACE("(%p,%d)\n",This,vol);
@@ -256,10 +244,10 @@ static HRESULT WINAPI IDirectSoundBufferImpl_SetVolume(IDirectSoundBuffer8 *ifac
 	return hres;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_GetVolume(IDirectSoundBuffer8 *iface, LONG *vol)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
-
+static HRESULT WINAPI IDirectSoundBufferImpl_GetVolume(
+	LPDIRECTSOUNDBUFFER8 iface,LPLONG vol
+) {
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	TRACE("(%p,%p)\n",This,vol);
 
 	if (!(This->dsbd.dwFlags & DSBCAPS_CTRLVOLUME)) {
@@ -277,18 +265,14 @@ static HRESULT WINAPI IDirectSoundBufferImpl_GetVolume(IDirectSoundBuffer8 *ifac
 	return DS_OK;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_SetFrequency(IDirectSoundBuffer8 *iface, DWORD freq)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
+static HRESULT WINAPI IDirectSoundBufferImpl_SetFrequency(
+	LPDIRECTSOUNDBUFFER8 iface,DWORD freq
+) {
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	DWORD oldFreq;
 
 	TRACE("(%p,%d)\n",This,freq);
 
-        if (is_primary_buffer(This)) {
-                WARN("not available for primary buffers.\n");
-                return DSERR_CONTROLUNAVAIL;
-        }
-
 	if (!(This->dsbd.dwFlags & DSBCAPS_CTRLFREQUENCY)) {
 		WARN("control unavailable\n");
 		return DSERR_CONTROLUNAVAIL;
@@ -320,12 +304,11 @@ static HRESULT WINAPI IDirectSoundBufferImpl_SetFrequency(IDirectSoundBuffer8 *i
 	return DS_OK;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_Play(IDirectSoundBuffer8 *iface, DWORD reserved1,
-        DWORD reserved2, DWORD flags)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
+static HRESULT WINAPI IDirectSoundBufferImpl_Play(
+	LPDIRECTSOUNDBUFFER8 iface,DWORD reserved1,DWORD reserved2,DWORD flags
+) {
 	HRESULT hres = DS_OK;
-
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	TRACE("(%p,%08x,%08x,%08x)\n",This,reserved1,reserved2,flags);
 
 	/* **** */
@@ -351,11 +334,10 @@ static HRESULT WINAPI IDirectSoundBufferImpl_Play(IDirectSoundBuffer8 *iface, DW
 	return hres;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_Stop(IDirectSoundBuffer8 *iface)
+static HRESULT WINAPI IDirectSoundBufferImpl_Stop(LPDIRECTSOUNDBUFFER8 iface)
 {
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
 	HRESULT hres = DS_OK;
-
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	TRACE("(%p)\n",This);
 
 	/* **** */
@@ -382,41 +364,50 @@ static HRESULT WINAPI IDirectSoundBufferImpl_Stop(IDirectSoundBuffer8 *iface)
 	return hres;
 }
 
-static ULONG WINAPI IDirectSoundBufferImpl_AddRef(IDirectSoundBuffer8 *iface)
+static ULONG WINAPI IDirectSoundBufferImpl_AddRef(LPDIRECTSOUNDBUFFER8 iface)
 {
-    IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
-    ULONG ref = InterlockedIncrement(&This->ref);
-
+    IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
+    ULONG ref = InterlockedIncrement(&(This->ref));
     TRACE("(%p) ref was %d\n", This, ref - 1);
-
-    if(ref == 1)
-        InterlockedIncrement(&This->numIfaces);
-
     return ref;
 }
 
-static ULONG WINAPI IDirectSoundBufferImpl_Release(IDirectSoundBuffer8 *iface)
+static ULONG WINAPI IDirectSoundBufferImpl_Release(LPDIRECTSOUNDBUFFER8 iface)
 {
-    IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
-    ULONG ref = InterlockedDecrement(&This->ref);
-
+    IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
+    ULONG ref = InterlockedDecrement(&(This->ref));
     TRACE("(%p) ref was %d\n", This, ref + 1);
 
-    if (!ref && !InterlockedDecrement(&This->numIfaces)) {
-        if (is_primary_buffer(This))
-            primarybuffer_destroy(This);
-        else
-            secondarybuffer_destroy(This);
+    if (!ref) {
+	DirectSoundDevice_RemoveBuffer(This->device, This);
+	RtlDeleteResource(&This->lock);
+
+	if (This->hwbuf)
+		IDsDriverBuffer_Release(This->hwbuf);
+	if (!This->hwbuf || (This->device->drvdesc.dwFlags & DSDDESC_USESYSTEMMEMORY)) {
+		This->buffer->ref--;
+		list_remove(&This->entry);
+		if (This->buffer->ref==0) {
+			HeapFree(GetProcessHeap(),0,This->buffer->memory);
+			HeapFree(GetProcessHeap(),0,This->buffer);
+		}
+	}
+
+	HeapFree(GetProcessHeap(), 0, This->tmp_buffer);
+	HeapFree(GetProcessHeap(), 0, This->notifies);
+	HeapFree(GetProcessHeap(), 0, This->pwfx);
+	HeapFree(GetProcessHeap(), 0, This);
+
+	TRACE("(%p) released\n", This);
     }
     return ref;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_GetCurrentPosition(IDirectSoundBuffer8 *iface,
-        DWORD *playpos, DWORD *writepos)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
+static HRESULT WINAPI IDirectSoundBufferImpl_GetCurrentPosition(
+	LPDIRECTSOUNDBUFFER8 iface,LPDWORD playpos,LPDWORD writepos
+) {
 	HRESULT	hres;
-
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	TRACE("(%p,%p,%p)\n",This,playpos,writepos);
 
 	RtlAcquireResourceShared(&This->lock, TRUE);
@@ -453,10 +444,10 @@ static HRESULT WINAPI IDirectSoundBufferImpl_GetCurrentPosition(IDirectSoundBuff
 	return DS_OK;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_GetStatus(IDirectSoundBuffer8 *iface, DWORD *status)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
-
+static HRESULT WINAPI IDirectSoundBufferImpl_GetStatus(
+	LPDIRECTSOUNDBUFFER8 iface,LPDWORD status
+) {
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	TRACE("(%p,%p), thread is %04x\n",This,status,GetCurrentThreadId());
 
 	if (status == NULL) {
@@ -478,12 +469,14 @@ static HRESULT WINAPI IDirectSoundBufferImpl_GetStatus(IDirectSoundBuffer8 *ifac
 }
 
 
-static HRESULT WINAPI IDirectSoundBufferImpl_GetFormat(IDirectSoundBuffer8 *iface,
-        LPWAVEFORMATEX lpwf, DWORD wfsize, DWORD *wfwritten)
+static HRESULT WINAPI IDirectSoundBufferImpl_GetFormat(
+    LPDIRECTSOUNDBUFFER8 iface,
+    LPWAVEFORMATEX lpwf,
+    DWORD wfsize,
+    LPDWORD wfwritten)
 {
-    IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
     DWORD size;
-
+    IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
     TRACE("(%p,%p,%d,%p)\n",This,lpwf,wfsize,wfwritten);
 
     size = sizeof(WAVEFORMATEX) + This->pwfx->cbSize;
@@ -512,15 +505,23 @@ static HRESULT WINAPI IDirectSoundBufferImpl_GetFormat(IDirectSoundBuffer8 *ifac
     return DS_OK;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_Lock(IDirectSoundBuffer8 *iface, DWORD writecursor,
-        DWORD writebytes, void **lplpaudioptr1, DWORD *audiobytes1, void **lplpaudioptr2,
-        DWORD *audiobytes2, DWORD flags)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
+static HRESULT WINAPI IDirectSoundBufferImpl_Lock(
+	LPDIRECTSOUNDBUFFER8 iface,DWORD writecursor,DWORD writebytes,LPVOID *lplpaudioptr1,LPDWORD audiobytes1,LPVOID *lplpaudioptr2,LPDWORD audiobytes2,DWORD flags
+) {
 	HRESULT hres = DS_OK;
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 
-        TRACE("(%p,%d,%d,%p,%p,%p,%p,0x%08x) at %d\n", This, writecursor, writebytes, lplpaudioptr1,
-                audiobytes1, lplpaudioptr2, audiobytes2, flags, GetTickCount());
+	TRACE("(%p,%d,%d,%p,%p,%p,%p,0x%08x) at %d\n",
+		This,
+		writecursor,
+		writebytes,
+		lplpaudioptr1,
+		audiobytes1,
+		lplpaudioptr2,
+		audiobytes2,
+		flags,
+		GetTickCount()
+	);
 
         if (!audiobytes1)
             return DSERR_INVALIDPARAM;
@@ -600,13 +601,12 @@ static HRESULT WINAPI IDirectSoundBufferImpl_Lock(IDirectSoundBuffer8 *iface, DW
 	return DS_OK;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_SetCurrentPosition(IDirectSoundBuffer8 *iface,
-        DWORD newpos)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
+static HRESULT WINAPI IDirectSoundBufferImpl_SetCurrentPosition(
+	LPDIRECTSOUNDBUFFER8 iface,DWORD newpos
+) {
 	HRESULT hres = DS_OK;
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	DWORD oldpos;
-
 	TRACE("(%p,%d)\n",This,newpos);
 
 	/* **** */
@@ -620,7 +620,7 @@ static HRESULT WINAPI IDirectSoundBufferImpl_SetCurrentPosition(IDirectSoundBuff
 	This->sec_mixpos = newpos;
 
 	/* at this point, do not attempt to reset buffers, mess with primary mix position,
-           or anything like that to reduce latency. The data already prebuffered cannot be changed */
+           or anything like that to reduce latancy. The data already prebuffered cannot be changed */
 
 	/* position HW buffer if applicable, else just start mixing from new location instead */
 	if (This->hwbuf) {
@@ -638,10 +638,11 @@ static HRESULT WINAPI IDirectSoundBufferImpl_SetCurrentPosition(IDirectSoundBuff
 	return hres;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_SetPan(IDirectSoundBuffer8 *iface, LONG pan)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
+static HRESULT WINAPI IDirectSoundBufferImpl_SetPan(
+	LPDIRECTSOUNDBUFFER8 iface,LONG pan
+) {
 	HRESULT hres = DS_OK;
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 
 	TRACE("(%p,%d)\n",This,pan);
 
@@ -677,10 +678,10 @@ static HRESULT WINAPI IDirectSoundBufferImpl_SetPan(IDirectSoundBuffer8 *iface,
 	return hres;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_GetPan(IDirectSoundBuffer8 *iface, LONG *pan)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
-
+static HRESULT WINAPI IDirectSoundBufferImpl_GetPan(
+	LPDIRECTSOUNDBUFFER8 iface,LPLONG pan
+) {
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	TRACE("(%p,%p)\n",This,pan);
 
 	if (!(This->dsbd.dwFlags & DSBCAPS_CTRLPAN)) {
@@ -698,10 +699,10 @@ static HRESULT WINAPI IDirectSoundBufferImpl_GetPan(IDirectSoundBuffer8 *iface,
 	return DS_OK;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_Unlock(IDirectSoundBuffer8 *iface, void *p1, DWORD x1,
-        void *p2, DWORD x2)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface), *iter;
+static HRESULT WINAPI IDirectSoundBufferImpl_Unlock(
+	LPDIRECTSOUNDBUFFER8 iface,LPVOID p1,DWORD x1,LPVOID p2,DWORD x2
+) {
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface, *iter;
 	HRESULT hres = DS_OK;
 
 	TRACE("(%p,%p,%d,%p,%d)\n", This,p1,x1,p2,x2);
@@ -744,18 +745,18 @@ static HRESULT WINAPI IDirectSoundBufferImpl_Unlock(IDirectSoundBuffer8 *iface,
 	return hres;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_Restore(IDirectSoundBuffer8 *iface)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
-
+static HRESULT WINAPI IDirectSoundBufferImpl_Restore(
+	LPDIRECTSOUNDBUFFER8 iface
+) {
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	FIXME("(%p):stub\n",This);
 	return DS_OK;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_GetFrequency(IDirectSoundBuffer8 *iface, DWORD *freq)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
-
+static HRESULT WINAPI IDirectSoundBufferImpl_GetFrequency(
+	LPDIRECTSOUNDBUFFER8 iface,LPDWORD freq
+) {
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	TRACE("(%p,%p)\n",This,freq);
 
 	if (freq == NULL) {
@@ -769,10 +770,10 @@ static HRESULT WINAPI IDirectSoundBufferImpl_GetFrequency(IDirectSoundBuffer8 *i
 	return DS_OK;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_SetFX(IDirectSoundBuffer8 *iface, DWORD dwEffectsCount,
-        LPDSEFFECTDESC pDSFXDesc, DWORD *pdwResultCodes)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
+static HRESULT WINAPI IDirectSoundBufferImpl_SetFX(
+	LPDIRECTSOUNDBUFFER8 iface,DWORD dwEffectsCount,LPDSEFFECTDESC pDSFXDesc,LPDWORD pdwResultCodes
+) {
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	DWORD u;
 
 	FIXME("(%p,%u,%p,%p): stub\n",This,dwEffectsCount,pDSFXDesc,pdwResultCodes);
@@ -784,10 +785,10 @@ static HRESULT WINAPI IDirectSoundBufferImpl_SetFX(IDirectSoundBuffer8 *iface, D
 	return DSERR_CONTROLUNAVAIL;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_AcquireResources(IDirectSoundBuffer8 *iface,
-        DWORD dwFlags, DWORD dwEffectsCount, DWORD *pdwResultCodes)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
+static HRESULT WINAPI IDirectSoundBufferImpl_AcquireResources(
+	LPDIRECTSOUNDBUFFER8 iface,DWORD dwFlags,DWORD dwEffectsCount,LPDWORD pdwResultCodes
+) {
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	DWORD u;
 
 	FIXME("(%p,%08u,%u,%p): stub, faking success\n",This,dwFlags,dwEffectsCount,pdwResultCodes);
@@ -799,10 +800,10 @@ static HRESULT WINAPI IDirectSoundBufferImpl_AcquireResources(IDirectSoundBuffer
 	return DS_OK;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_GetObjectInPath(IDirectSoundBuffer8 *iface,
-        REFGUID rguidObject, DWORD dwIndex, REFGUID rguidInterface, void **ppObject)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
+static HRESULT WINAPI IDirectSoundBufferImpl_GetObjectInPath(
+	LPDIRECTSOUNDBUFFER8 iface,REFGUID rguidObject,DWORD dwIndex,REFGUID rguidInterface,LPVOID* ppObject
+) {
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 
 	FIXME("(%p,%s,%u,%s,%p): stub\n",This,debugstr_guid(rguidObject),dwIndex,debugstr_guid(rguidInterface),ppObject);
 
@@ -810,19 +811,18 @@ static HRESULT WINAPI IDirectSoundBufferImpl_GetObjectInPath(IDirectSoundBuffer8
 	return DSERR_CONTROLUNAVAIL;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_Initialize(IDirectSoundBuffer8 *iface,
-        IDirectSound *dsound, LPCDSBUFFERDESC dbsd)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
-
+static HRESULT WINAPI IDirectSoundBufferImpl_Initialize(
+	LPDIRECTSOUNDBUFFER8 iface,LPDIRECTSOUND dsound,LPCDSBUFFERDESC dbsd
+) {
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	WARN("(%p) already initialized\n", This);
 	return DSERR_ALREADYINITIALIZED;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_GetCaps(IDirectSoundBuffer8 *iface, LPDSBCAPS caps)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
-
+static HRESULT WINAPI IDirectSoundBufferImpl_GetCaps(
+	LPDIRECTSOUNDBUFFER8 iface,LPDSBCAPS caps
+) {
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
   	TRACE("(%p)->(%p)\n",This,caps);
 
 	if (caps == NULL) {
@@ -848,10 +848,10 @@ static HRESULT WINAPI IDirectSoundBufferImpl_GetCaps(IDirectSoundBuffer8 *iface,
 	return DS_OK;
 }
 
-static HRESULT WINAPI IDirectSoundBufferImpl_QueryInterface(IDirectSoundBuffer8 *iface, REFIID riid,
-        void **ppobj)
-{
-        IDirectSoundBufferImpl *This = impl_from_IDirectSoundBuffer8(iface);
+static HRESULT WINAPI IDirectSoundBufferImpl_QueryInterface(
+	LPDIRECTSOUNDBUFFER8 iface,REFIID riid,LPVOID *ppobj
+) {
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 
 	TRACE("(%p,%s,%p)\n",This,debugstr_guid(riid),ppobj);
 
@@ -865,9 +865,15 @@ static HRESULT WINAPI IDirectSoundBufferImpl_QueryInterface(IDirectSoundBuffer8
 	if ( IsEqualGUID(riid, &IID_IUnknown) ||
 	     IsEqualGUID(riid, &IID_IDirectSoundBuffer) ||
 	     IsEqualGUID(riid, &IID_IDirectSoundBuffer8) ) {
-                IDirectSoundBuffer8_AddRef(iface);
-                *ppobj = iface;
-                return S_OK;
+		if (!This->secondary)
+			SecondaryBufferImpl_Create(This, &(This->secondary));
+		if (This->secondary) {
+			IDirectSoundBuffer8_AddRef((LPDIRECTSOUNDBUFFER8)This->secondary);
+			*ppobj = This->secondary;
+			return S_OK;
+		}
+		WARN("IID_IDirectSoundBuffer\n");
+		return E_NOINTERFACE;
 	}
 
 	if ( IsEqualGUID( &IID_IDirectSoundNotify, riid ) ) {
@@ -972,10 +978,10 @@ HRESULT IDirectSoundBufferImpl_Create(
 
 	TRACE("Created buffer at %p\n", dsb);
 
-        dsb->ref = 1;
-        dsb->numIfaces = 1;
+	dsb->ref = 0;
+	dsb->secondary = 0;
 	dsb->device = device;
-        dsb->IDirectSoundBuffer8_iface.lpVtbl = &dsbvt;
+	dsb->lpVtbl = &dsbvt;
 	dsb->iks = NULL;
 
 	/* size depends on version */
@@ -1126,30 +1132,6 @@ HRESULT IDirectSoundBufferImpl_Create(
 	return err;
 }
 
-void secondarybuffer_destroy(IDirectSoundBufferImpl *This)
-{
-    DirectSoundDevice_RemoveBuffer(This->device, This);
-    RtlDeleteResource(&This->lock);
-
-    if (This->hwbuf)
-        IDsDriverBuffer_Release(This->hwbuf);
-    if (!This->hwbuf || (This->device->drvdesc.dwFlags & DSDDESC_USESYSTEMMEMORY)) {
-        This->buffer->ref--;
-        list_remove(&This->entry);
-        if (This->buffer->ref == 0) {
-            HeapFree(GetProcessHeap(), 0, This->buffer->memory);
-            HeapFree(GetProcessHeap(), 0, This->buffer);
-        }
-    }
-
-    HeapFree(GetProcessHeap(), 0, This->tmp_buffer);
-    HeapFree(GetProcessHeap(), 0, This->notifies);
-    HeapFree(GetProcessHeap(), 0, This->pwfx);
-    HeapFree(GetProcessHeap(), 0, This);
-
-    TRACE("(%p) released\n", This);
-}
-
 HRESULT IDirectSoundBufferImpl_Destroy(
     IDirectSoundBufferImpl *pdsb)
 {
@@ -1157,7 +1139,7 @@ HRESULT IDirectSoundBufferImpl_Destroy(
 
     /* This keeps the *_Destroy functions from possibly deleting
      * this object until it is ready to be deleted */
-    InterlockedIncrement(&pdsb->numIfaces);
+    IDirectSoundBufferImpl_AddRef((LPDIRECTSOUNDBUFFER8)pdsb);
 
     if (pdsb->iks) {
         WARN("iks not NULL\n");
@@ -1177,7 +1159,13 @@ HRESULT IDirectSoundBufferImpl_Destroy(
         pdsb->notify = NULL;
     }
 
-    secondarybuffer_destroy(pdsb);
+    if (pdsb->secondary) {
+        WARN("dsb not NULL\n");
+        SecondaryBufferImpl_Destroy(pdsb->secondary);
+        pdsb->secondary = NULL;
+    }
+
+    while (IDirectSoundBuffer8_Release((LPDIRECTSOUNDBUFFER8)pdsb) > 0);
 
     return S_OK;
 }
@@ -1189,7 +1177,7 @@ HRESULT IDirectSoundBufferImpl_Duplicate(
 {
     IDirectSoundBufferImpl *dsb;
     HRESULT hres = DS_OK;
-    TRACE("(%p,%p,%p)\n", device, ppdsb, pdsb);
+    TRACE("(%p,%p,%p)\n", device, pdsb, pdsb);
 
     dsb = HeapAlloc(GetProcessHeap(),0,sizeof(*dsb));
     if (dsb == NULL) {
@@ -1222,16 +1210,13 @@ HRESULT IDirectSoundBufferImpl_Duplicate(
 
     dsb->buffer->ref++;
     list_add_head(&dsb->buffer->buffers, &dsb->entry);
-    dsb->ref = 1;
-    dsb->numIfaces = 1;
+    dsb->ref = 0;
     dsb->state = STATE_STOPPED;
     dsb->buf_mixpos = dsb->sec_mixpos = 0;
-    dsb->notify = NULL;
-    dsb->notifies = NULL;
-    dsb->nrofnotifies = 0;
     dsb->device = device;
     dsb->ds3db = NULL;
     dsb->iks = NULL; /* FIXME? */
+    dsb->secondary = NULL;
     dsb->tmp_buffer = NULL;
     DSOUND_RecalcFormat(dsb);
     DSOUND_MixToTemporary(dsb, 0, dsb->buflen, FALSE);
@@ -1254,6 +1239,308 @@ HRESULT IDirectSoundBufferImpl_Duplicate(
     return hres;
 }
 
+/*******************************************************************************
+ *		SecondaryBuffer
+ */
+
+static HRESULT WINAPI SecondaryBufferImpl_QueryInterface(
+	LPDIRECTSOUNDBUFFER8 iface,REFIID riid,LPVOID *ppobj)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%s,%p)\n",This,debugstr_guid(riid),ppobj);
+
+	return IDirectSoundBufferImpl_QueryInterface((LPDIRECTSOUNDBUFFER8)This->dsb,riid,ppobj);
+}
+
+static ULONG WINAPI SecondaryBufferImpl_AddRef(LPDIRECTSOUNDBUFFER8 iface)
+{
+    SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+    ULONG ref = InterlockedIncrement(&(This->ref));
+    TRACE("(%p) ref was %d\n", This, ref - 1);
+    return ref;
+}
+
+static ULONG WINAPI SecondaryBufferImpl_Release(LPDIRECTSOUNDBUFFER8 iface)
+{
+    SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+    ULONG ref;
+    TRACE("(%p)\n", This);
+    ref = InterlockedDecrement(&(This->ref));
+    TRACE("ref was %d\n", ref + 1);
+
+    if (!ref) {
+        This->dsb->secondary = NULL;
+        IDirectSoundBuffer_Release((LPDIRECTSOUNDBUFFER8)This->dsb);
+        HeapFree(GetProcessHeap(), 0, This);
+        TRACE("(%p) released\n", This);
+    }
+    return ref;
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_GetCaps(
+	LPDIRECTSOUNDBUFFER8 iface,LPDSBCAPS caps)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+  	TRACE("(%p)->(%p)\n",This,caps);
+
+	return IDirectSoundBufferImpl_GetCaps((LPDIRECTSOUNDBUFFER8)This->dsb,caps);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_GetCurrentPosition(
+	LPDIRECTSOUNDBUFFER8 iface,LPDWORD playpos,LPDWORD writepos)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%p,%p)\n",This,playpos,writepos);
+
+	return IDirectSoundBufferImpl_GetCurrentPosition((LPDIRECTSOUNDBUFFER8)This->dsb,playpos,writepos);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_GetFormat(
+	LPDIRECTSOUNDBUFFER8 iface,LPWAVEFORMATEX lpwf,DWORD wfsize,LPDWORD wfwritten)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%p,%d,%p)\n",This,lpwf,wfsize,wfwritten);
+
+	return IDirectSoundBufferImpl_GetFormat((LPDIRECTSOUNDBUFFER8)This->dsb,lpwf,wfsize,wfwritten);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_GetVolume(
+	LPDIRECTSOUNDBUFFER8 iface,LPLONG vol)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%p)\n",This,vol);
+
+	return IDirectSoundBufferImpl_GetVolume((LPDIRECTSOUNDBUFFER8)This->dsb,vol);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_GetPan(
+	LPDIRECTSOUNDBUFFER8 iface,LPLONG pan)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%p)\n",This,pan);
+
+	return IDirectSoundBufferImpl_GetPan((LPDIRECTSOUNDBUFFER8)This->dsb,pan);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_GetFrequency(
+	LPDIRECTSOUNDBUFFER8 iface,LPDWORD freq)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%p)\n",This,freq);
+
+	return IDirectSoundBufferImpl_GetFrequency((LPDIRECTSOUNDBUFFER8)This->dsb,freq);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_GetStatus(
+	LPDIRECTSOUNDBUFFER8 iface,LPDWORD status)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%p)\n",This,status);
+
+	return IDirectSoundBufferImpl_GetStatus((LPDIRECTSOUNDBUFFER8)This->dsb,status);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_Initialize(
+	LPDIRECTSOUNDBUFFER8 iface,LPDIRECTSOUND dsound,LPCDSBUFFERDESC dbsd)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%p,%p)\n",This,dsound,dbsd);
+
+	return IDirectSoundBufferImpl_Initialize((LPDIRECTSOUNDBUFFER8)This->dsb,dsound,dbsd);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_Lock(
+    LPDIRECTSOUNDBUFFER8 iface,
+    DWORD writecursor,
+    DWORD writebytes,
+    LPVOID *lplpaudioptr1,
+    LPDWORD audiobytes1,
+    LPVOID *lplpaudioptr2,
+    LPDWORD audiobytes2,
+    DWORD dwFlags)
+{
+    SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+    TRACE("(%p,%d,%d,%p,%p,%p,%p,0x%08x)\n",
+        This,writecursor,writebytes,lplpaudioptr1,audiobytes1,lplpaudioptr2,audiobytes2,dwFlags);
+
+    return IDirectSoundBufferImpl_Lock((LPDIRECTSOUNDBUFFER8)This->dsb,
+        writecursor,writebytes,lplpaudioptr1,audiobytes1,lplpaudioptr2,audiobytes2,dwFlags);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_Play(
+	LPDIRECTSOUNDBUFFER8 iface,DWORD reserved1,DWORD reserved2,DWORD flags)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%08x,%08x,%08x)\n",This,reserved1,reserved2,flags);
+
+	return IDirectSoundBufferImpl_Play((LPDIRECTSOUNDBUFFER8)This->dsb,reserved1,reserved2,flags);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_SetCurrentPosition(
+	LPDIRECTSOUNDBUFFER8 iface,DWORD newpos)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%d)\n",This,newpos);
+
+	return IDirectSoundBufferImpl_SetCurrentPosition((LPDIRECTSOUNDBUFFER8)This->dsb,newpos);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_SetFormat(
+	LPDIRECTSOUNDBUFFER8 iface,LPCWAVEFORMATEX wfex)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%p)\n",This,wfex);
+
+	return IDirectSoundBufferImpl_SetFormat((LPDIRECTSOUNDBUFFER8)This->dsb,wfex);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_SetVolume(
+	LPDIRECTSOUNDBUFFER8 iface,LONG vol)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%d)\n",This,vol);
+
+	return IDirectSoundBufferImpl_SetVolume((LPDIRECTSOUNDBUFFER8)This->dsb,vol);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_SetPan(
+	LPDIRECTSOUNDBUFFER8 iface,LONG pan)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%d)\n",This,pan);
+
+	return IDirectSoundBufferImpl_SetPan((LPDIRECTSOUNDBUFFER8)This->dsb,pan);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_SetFrequency(
+	LPDIRECTSOUNDBUFFER8 iface,DWORD freq)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%d)\n",This,freq);
+
+	return IDirectSoundBufferImpl_SetFrequency((LPDIRECTSOUNDBUFFER8)This->dsb,freq);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_Stop(LPDIRECTSOUNDBUFFER8 iface)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p)\n",This);
+
+	return IDirectSoundBufferImpl_Stop((LPDIRECTSOUNDBUFFER8)This->dsb);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_Unlock(
+    LPDIRECTSOUNDBUFFER8 iface,
+    LPVOID lpvAudioPtr1,
+    DWORD dwAudioBytes1,
+    LPVOID lpvAudioPtr2,
+    DWORD dwAudioBytes2)
+{
+    SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+    TRACE("(%p,%p,%d,%p,%d)\n",
+        This, lpvAudioPtr1, dwAudioBytes1, lpvAudioPtr2, dwAudioBytes2);
+
+    return IDirectSoundBufferImpl_Unlock((LPDIRECTSOUNDBUFFER8)This->dsb,
+        lpvAudioPtr1,dwAudioBytes1,lpvAudioPtr2,dwAudioBytes2);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_Restore(
+	LPDIRECTSOUNDBUFFER8 iface)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p)\n",This);
+
+	return IDirectSoundBufferImpl_Restore((LPDIRECTSOUNDBUFFER8)This->dsb);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_SetFX(
+	LPDIRECTSOUNDBUFFER8 iface,DWORD dwEffectsCount,LPDSEFFECTDESC pDSFXDesc,LPDWORD pdwResultCodes)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%u,%p,%p)\n",This,dwEffectsCount,pDSFXDesc,pdwResultCodes);
+
+	return IDirectSoundBufferImpl_SetFX((LPDIRECTSOUNDBUFFER8)This->dsb,dwEffectsCount,pDSFXDesc,pdwResultCodes);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_AcquireResources(
+	LPDIRECTSOUNDBUFFER8 iface,DWORD dwFlags,DWORD dwEffectsCount,LPDWORD pdwResultCodes)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%08u,%u,%p)\n",This,dwFlags,dwEffectsCount,pdwResultCodes);
+
+	return IDirectSoundBufferImpl_AcquireResources((LPDIRECTSOUNDBUFFER8)This->dsb,dwFlags,dwEffectsCount,pdwResultCodes);
+}
+
+static HRESULT WINAPI SecondaryBufferImpl_GetObjectInPath(
+	LPDIRECTSOUNDBUFFER8 iface,REFGUID rguidObject,DWORD dwIndex,REFGUID rguidInterface,LPVOID* ppObject)
+{
+	SecondaryBufferImpl *This = (SecondaryBufferImpl *)iface;
+	TRACE("(%p,%s,%u,%s,%p)\n",This,debugstr_guid(rguidObject),dwIndex,debugstr_guid(rguidInterface),ppObject);
+
+	return IDirectSoundBufferImpl_GetObjectInPath((LPDIRECTSOUNDBUFFER8)This->dsb,rguidObject,dwIndex,rguidInterface,ppObject);
+}
+
+static const IDirectSoundBuffer8Vtbl sbvt =
+{
+	SecondaryBufferImpl_QueryInterface,
+	SecondaryBufferImpl_AddRef,
+	SecondaryBufferImpl_Release,
+	SecondaryBufferImpl_GetCaps,
+	SecondaryBufferImpl_GetCurrentPosition,
+	SecondaryBufferImpl_GetFormat,
+	SecondaryBufferImpl_GetVolume,
+	SecondaryBufferImpl_GetPan,
+	SecondaryBufferImpl_GetFrequency,
+	SecondaryBufferImpl_GetStatus,
+	SecondaryBufferImpl_Initialize,
+	SecondaryBufferImpl_Lock,
+	SecondaryBufferImpl_Play,
+	SecondaryBufferImpl_SetCurrentPosition,
+	SecondaryBufferImpl_SetFormat,
+	SecondaryBufferImpl_SetVolume,
+	SecondaryBufferImpl_SetPan,
+	SecondaryBufferImpl_SetFrequency,
+	SecondaryBufferImpl_Stop,
+	SecondaryBufferImpl_Unlock,
+	SecondaryBufferImpl_Restore,
+	SecondaryBufferImpl_SetFX,
+	SecondaryBufferImpl_AcquireResources,
+	SecondaryBufferImpl_GetObjectInPath
+};
+
+HRESULT SecondaryBufferImpl_Create(
+	IDirectSoundBufferImpl *dsb,
+	SecondaryBufferImpl **psb)
+{
+	SecondaryBufferImpl *sb;
+	TRACE("(%p,%p)\n",dsb,psb);
+
+	sb = HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,sizeof(*sb));
+
+	if (sb == 0) {
+		WARN("out of memory\n");
+		*psb = NULL;
+		return DSERR_OUTOFMEMORY;
+	}
+	sb->ref = 0;
+	sb->dsb = dsb;
+	sb->lpVtbl = &sbvt;
+
+	IDirectSoundBuffer8_AddRef((LPDIRECTSOUNDBUFFER8)dsb);
+	*psb = sb;
+	return S_OK;
+}
+
+static HRESULT SecondaryBufferImpl_Destroy(
+    SecondaryBufferImpl *pdsb)
+{
+    TRACE("(%p)\n",pdsb);
+
+    while (SecondaryBufferImpl_Release((LPDIRECTSOUNDBUFFER8)pdsb) > 0);
+
+    return S_OK;
+}
+
 /*******************************************************************************
  *              IKsBufferPropertySet
  */
