@@ -62,7 +62,9 @@ static HGLOBAL dib_copy(const BITMAPINFO *info, UINT coloruse)
     if (info->bmiHeader.biCompression != BI_RGB && info->bmiHeader.biCompression != BI_BITFIELDS)
         size = info->bmiHeader.biSizeImage;
     else
-        size = get_dib_image_size(info);
+        size = DIB_GetDIBImageBytes(info->bmiHeader.biWidth,
+                                    info->bmiHeader.biHeight,
+                                    info->bmiHeader.biBitCount);
     size += bitmap_info_size( info, coloruse );
 
     if (!(hmem = GlobalAlloc( GMEM_MOVEABLE, size )))
@@ -329,9 +331,9 @@ HBRUSH WINAPI CreateSolidBrush( COLORREF color )
  * Set the brush origin for a device context.
  *
  * PARAMS
- *  hdc    [I] Device context to set the brush origin for
+ *  hdc    [I] Device context to set the brush origin for 
  *  x      [I] New x origin
- *  y      [I] New y origin
+ *  y      [I] Ney y origin
  *  oldorg [O] If non NULL, destination for previously set brush origin.
  *
  * RETURNS
@@ -392,7 +394,7 @@ static HGDIOBJ BRUSH_SelectObject( HGDIOBJ handle, HDC hdc )
             PHYSDEV pattern_dev = physdev;
             /* FIXME: This will go away once the dib driver implements
                pattern brushes */
-            if(pattern_dev == dc->dibdrv)
+            if(pattern_dev == &dc->dibdrv.dev)
                 pattern_dev = GET_NEXT_PHYSDEV( physdev, pSelectBrush );
 
             BITMAP_SetOwnerDC( (HBITMAP)brush->logbrush.lbHatch, pattern_dev );
