@@ -57,6 +57,13 @@ static inline BYTE *get_pixel_ptr_4(const dib_info *dib, int x, int y)
     return (BYTE*)dib->bits + y * dib->stride + x / 2;
 }
 
+static inline BYTE *get_pixel_ptr_1(const dib_info *dib, int x, int y)
+{
+    return (BYTE*)dib->bits + y * dib->stride + x / 8;
+}
+
+static const BYTE pixel_masks_1[8] = {0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};
+
 static inline void do_rop_32(DWORD *ptr, DWORD and, DWORD xor)
 {
     *ptr = (*ptr & and) ^ xor;
@@ -221,6 +228,60 @@ static void solid_rects_4(const dib_info *dib, int num, const RECT *rc, DWORD an
     }
 }
 
+static void solid_rects_1(const dib_info *dib, int num, const RECT *rc, DWORD and, DWORD xor)
+{
+    BYTE *ptr, *start;
+    int x, y, i;
+    BYTE byte_and = (and & 1) ? 0xff : 0;
+    BYTE byte_xor = (xor & 1) ? 0xff : 0;
+    BYTE start_and, start_xor, end_and, end_xor, mask;
+    static const BYTE masks[8] = {0xff, 0x7f, 0x3f, 0x1f, 0x0f, 0x07, 0x03, 0x01};
+
+    for(i = 0; i < num; i++, rc++)
+    {
+        if(rc->left >= rc->right) continue;
+
+        start = get_pixel_ptr_1(dib, rc->left, rc->top);
+
+        if((rc->left & ~7) == (rc->right & ~7)) /* Special case for lines that start and end in the same byte */
+        {
+            mask = masks[rc->left & 7] & ~masks[rc->right & 7];
+
+            start_and = byte_and | ~mask;
+            start_xor = byte_xor & mask;
+            for(y = rc->top; y < rc->bottom; y++, start += dib->stride)
+            {
+                do_rop_8(start, start_and, start_xor);
+            }
+        }
+        else
+        {
+            mask = masks[rc->left & 7];
+            start_and = byte_and | ~mask;
+            start_xor = byte_xor & mask;
+
+            mask = masks[rc->right & 7];
+            /* This is inverted wrt to start mask, so end_and/xor assignments reflect this */
+            end_and = byte_and | mask;
+            end_xor = byte_xor & ~mask;
+
+            for(y = rc->top; y < rc->bottom; y++, start += dib->stride)
+            {
+                ptr = start;
+
+                if(rc->left & 7)
+                    do_rop_8(ptr++, start_and, start_xor);
+
+                for(x = (rc->left + 7) & ~7; x < (rc->right & ~7); x += 8)
+                    do_rop_8(ptr++, byte_and, byte_xor);
+
+                if(rc->right & 7)
+                    do_rop_8(ptr, end_and, end_xor);
+            }
+        }
+    }
+}
+
 static void solid_rects_null(const dib_info *dib, int num, const RECT *rc, DWORD and, DWORD xor)
 {
     return;
@@ -516,6 +577,70 @@ static void pattern_rects_4(const dib_info *dib, int num, const RECT *rc, const
     }
 }
 
+static void pattern_rects_1(const dib_info *dib, int num, const RECT *rc, const POINT *origin,
+                            const dib_info *brush, void *and_bits, void *xor_bits)
+{
+    BYTE *ptr, *start, *start_and, *and_ptr, *start_xor, *xor_ptr;
+    int x, y, i;
+    POINT offset;
+
+    for(i = 0; i < num; i++, rc++)
+    {
+        offset = calc_brush_offset(rc, brush, origin);
+
+        start = get_pixel_ptr_1(dib, rc->left, rc->top);
+        start_and = (BYTE*)and_bits + offset.y * brush->stride;
+        start_xor = (BYTE*)xor_bits + offset.y * brush->stride;
+
+        for(y = rc->top; y < rc->bottom; y++, start += dib->stride)
+        {
+            INT brush_x = offset.x;
+            BYTE byte_and, byte_xor;
+
+            and_ptr = start_and + brush_x / 8;
+            xor_ptr = start_xor + brush_x / 8;
+
+            for(x = rc->left, ptr = start; x < rc->right; x++)
+            {
+                byte_and = (*and_ptr & pixel_masks_1[brush_x % 8]) ? 0xff : 0;
+                byte_and |= ~pixel_masks_1[x % 8];
+                byte_xor = (*xor_ptr & pixel_masks_1[brush_x % 8]) ? 0xff : 0;
+                byte_xor &= pixel_masks_1[x % 8];
+
+                do_rop_8(ptr, byte_and, byte_xor);
+
+                if((x & 7) == 7) ptr++;
+
+                if((brush_x & 7) == 7)
+                {
+                    and_ptr++;
+                    xor_ptr++;
+                }
+
+                if(++brush_x == brush->width)
+                {
+                    brush_x = 0;
+                    and_ptr = start_and;
+                    xor_ptr = start_xor;
+                }
+            }
+
+            offset.y++;
+            if(offset.y == brush->height)
+            {
+                start_and = and_bits;
+                start_xor = xor_bits;
+                offset.y = 0;
+            }
+            else
+            {
+                start_and += brush->stride;
+                start_xor += brush->stride;
+            }
+        }
+    }
+}
+
 static void pattern_rects_null(const dib_info *dib, int num, const RECT *rc, const POINT *origin,
                                const dib_info *brush, void *and_bits, void *xor_bits)
 {
@@ -722,6 +847,27 @@ static BOOL convert_to_8888(dib_info *dst, const dib_info *src, const RECT *src_
         break;
     }
 
+    case 1:
+    {
+        BYTE *src_start = get_pixel_ptr_1(src, src_rect->left, src_rect->top), *src_pixel;
+        for(y = src_rect->top; y < src_rect->bottom; y++)
+        {
+            dst_pixel = dst_start;
+            src_pixel = src_start;
+            for(x = src_rect->left; x < src_rect->right; x++)
+            {
+                RGBQUAD rgb;
+                src_val = (*src_pixel & pixel_masks_1[x % 8]) ? 1 : 0;
+                if((x % 8) == 7) src_pixel++;
+                rgb = src->color_table[src_val];
+                *dst_pixel++ = rgb.rgbRed << 16 | rgb.rgbGreen << 8 | rgb.rgbBlue;
+            }
+            dst_start += dst->stride / 4;
+            src_start += src->stride;
+        }
+        break;
+    }
+
     default:
         FIXME("Unsupported conversion: %d -> 8888\n", src->bit_count);
         return FALSE;
@@ -868,6 +1014,29 @@ static BOOL convert_to_32(dib_info *dst, const dib_info *src, const RECT *src_re
         break;
     }
 
+    case 1:
+    {
+        BYTE *src_start = get_pixel_ptr_1(src, src_rect->left, src_rect->top), *src_pixel;
+        for(y = src_rect->top; y < src_rect->bottom; y++)
+        {
+            dst_pixel = dst_start;
+            src_pixel = src_start;
+            for(x = src_rect->left; x < src_rect->right; x++)
+            {
+                RGBQUAD rgb;
+                src_val = (*src_pixel & pixel_masks_1[x % 8]) ? 1 : 0;
+                if((x % 8) == 7) src_pixel++;
+                rgb = src->color_table[src_val];
+                *dst_pixel++ = put_field(rgb.rgbRed,   dst->red_shift,   dst->red_len) |
+                               put_field(rgb.rgbGreen, dst->green_shift, dst->green_len) |
+                               put_field(rgb.rgbBlue,  dst->blue_shift,  dst->blue_len);
+            }
+            dst_start += dst->stride / 4;
+            src_start += src->stride;
+        }
+        break;
+    }
+
     default:
         FIXME("Unsupported conversion: %d -> 32\n", src->bit_count);
         return FALSE;
@@ -1007,6 +1176,29 @@ static BOOL convert_to_24(dib_info *dst, const dib_info *src, const RECT *src_re
         break;
     }
 
+    case 1:
+    {
+        BYTE *src_start = get_pixel_ptr_1(src, src_rect->left, src_rect->top), *src_pixel;
+        for(y = src_rect->top; y < src_rect->bottom; y++)
+        {
+            dst_pixel = dst_start;
+            src_pixel = src_start;
+            for(x = src_rect->left; x < src_rect->right; x++)
+            {
+                RGBQUAD rgb;
+                src_val = (*src_pixel & pixel_masks_1[x % 8]) ? 1 : 0;
+                if((x % 8) == 7) src_pixel++;
+                rgb = src->color_table[src_val];
+                *dst_pixel++ = rgb.rgbBlue;
+                *dst_pixel++ = rgb.rgbGreen;
+                *dst_pixel++ = rgb.rgbRed;
+            }
+            dst_start += dst->stride;
+            src_start += src->stride;
+        }
+        break;
+    }
+
     default:
         FIXME("Unsupported conversion: %d -> 24\n", src->bit_count);
         return FALSE;
@@ -1151,6 +1343,29 @@ static BOOL convert_to_555(dib_info *dst, const dib_info *src, const RECT *src_r
         break;
     }
 
+    case 1:
+    {
+        BYTE *src_start = get_pixel_ptr_1(src, src_rect->left, src_rect->top), *src_pixel;
+        for(y = src_rect->top; y < src_rect->bottom; y++)
+        {
+            dst_pixel = dst_start;
+            src_pixel = src_start;
+            for(x = src_rect->left; x < src_rect->right; x++)
+            {
+                RGBQUAD rgb;
+                src_val = (*src_pixel & pixel_masks_1[x % 8]) ? 1 : 0;
+                if((x % 8) == 7) src_pixel++;
+                rgb = src->color_table[src_val];
+                *dst_pixel++ = ((rgb.rgbRed   << 7) & 0x7c00) |
+                               ((rgb.rgbGreen << 2) & 0x03e0) |
+                               ((rgb.rgbBlue  >> 3) & 0x001f);
+            }
+            dst_start += dst->stride / 2;
+            src_start += src->stride;
+        }
+        break;
+    }
+
     default:
         FIXME("Unsupported conversion: %d -> 555\n", src->bit_count);
         return FALSE;
@@ -1298,6 +1513,29 @@ static BOOL convert_to_16(dib_info *dst, const dib_info *src, const RECT *src_re
         break;
     }
 
+    case 1:
+    {
+        BYTE *src_start = get_pixel_ptr_1(src, src_rect->left, src_rect->top), *src_pixel;
+        for(y = src_rect->top; y < src_rect->bottom; y++)
+        {
+            dst_pixel = dst_start;
+            src_pixel = src_start;
+            for(x = src_rect->left; x < src_rect->right; x++)
+            {
+                RGBQUAD rgb;
+                src_val = (*src_pixel & pixel_masks_1[x % 8]) ? 1 : 0;
+                if((x % 8) == 7) src_pixel++;
+                rgb = src->color_table[src_val];
+                *dst_pixel++ = put_field(rgb.rgbRed,   dst->red_shift,   dst->red_len) |
+                               put_field(rgb.rgbGreen, dst->green_shift, dst->green_len) |
+                               put_field(rgb.rgbBlue,  dst->blue_shift,  dst->blue_len);
+            }
+            dst_start += dst->stride / 2;
+            src_start += src->stride;
+        }
+        break;
+    }
+
     default:
         FIXME("Unsupported conversion: %d -> 16\n", src->bit_count);
         return FALSE;
@@ -1467,6 +1705,27 @@ static BOOL convert_to_8(dib_info *dst, const dib_info *src, const RECT *src_rec
         break;
     }
 
+    case 1:
+    {
+        BYTE *src_start = get_pixel_ptr_1(src, src_rect->left, src_rect->top), *src_pixel;
+        for(y = src_rect->top; y < src_rect->bottom; y++)
+        {
+            dst_pixel = dst_start;
+            src_pixel = src_start;
+            for(x = src_rect->left; x < src_rect->right; x++)
+            {
+                RGBQUAD rgb;
+                src_val = (*src_pixel & pixel_masks_1[x % 8]) ? 1 : 0;
+                if((x % 8) == 7) src_pixel++;
+                rgb = src->color_table[src_val];
+                *dst_pixel++ = colorref_to_pixel_colortable(dst, RGB(rgb.rgbRed, rgb.rgbGreen, rgb.rgbBlue));
+            }
+            dst_start += dst->stride;
+            src_start += src->stride;
+        }
+        break;
+    }
+
     default:
         FIXME("Unsupported conversion: %d -> 8\n", src->bit_count);
         return FALSE;
@@ -1665,6 +1924,34 @@ static BOOL convert_to_4(dib_info *dst, const dib_info *src, const RECT *src_rec
         break;
     }
 
+    case 1:
+    {
+        BYTE *src_start = get_pixel_ptr_1(src, src_rect->left, src_rect->top), *src_pixel;
+        for(y = src_rect->top; y < src_rect->bottom; y++)
+        {
+            dst_pixel = dst_start;
+            src_pixel = src_start;
+            for(x = src_rect->left; x < src_rect->right; x++)
+            {
+                RGBQUAD rgb;
+                src_val = (*src_pixel & pixel_masks_1[x % 8]) ? 1 : 0;
+                if((x % 8) == 7) src_pixel++;
+                rgb = src->color_table[src_val];
+                dst_val = colorref_to_pixel_colortable(dst, RGB(rgb.rgbRed, rgb.rgbGreen, rgb.rgbBlue));
+                if((x - src_rect->left) & 1)
+                {
+                    *dst_pixel = (dst_val & 0x0f) | (*dst_pixel & 0xf0);
+                    dst_pixel++;
+                }
+                else
+                    *dst_pixel = (dst_val << 4) & 0xf0;
+            }
+            dst_start += dst->stride;
+            src_start += src->stride;
+        }
+        break;
+    }
+
     default:
         FIXME("Unsupported conversion: %d -> 4\n", src->bit_count);
         return FALSE;
@@ -1674,17 +1961,474 @@ static BOOL convert_to_4(dib_info *dst, const dib_info *src, const RECT *src_rec
     return TRUE;
 }
 
+static BOOL convert_to_1(dib_info *dst, const dib_info *src, const RECT *src_rect)
+{
+    BYTE *dst_start = dst->bits, *dst_pixel, dst_val;
+    INT x, y;
+    DWORD src_val;
+    int bit_pos;
+
+    /* FIXME: Brushes should be dithered. */
+
+    switch(src->bit_count)
+    {
+    case 32:
+    {
+        DWORD *src_start = get_pixel_ptr_32(src, src_rect->left, src_rect->top), *src_pixel;
+
+        if(src->funcs == &funcs_8888)
+        {
+            for(y = src_rect->top; y < src_rect->bottom; y++)
+            {
+                dst_pixel = dst_start;
+                src_pixel = src_start;
+                for(x = src_rect->left, bit_pos = 0; x < src_rect->right; x++)
+                {
+                    src_val = *src_pixel++;
+                    dst_val = colorref_to_pixel_colortable(dst, ((src_val >> 16) & 0x0000ff) |
+                                                                ( src_val        & 0x00ff00) |
+                                                                ((src_val << 16) & 0xff0000) ) ? 0xff : 0;
+
+                    if(bit_pos == 0) *dst_pixel = 0;
+                    *dst_pixel = (*dst_pixel & ~pixel_masks_1[bit_pos]) | (dst_val & pixel_masks_1[bit_pos]);
+
+                    if(++bit_pos == 8)
+                    {
+                        dst_pixel++;
+                        bit_pos = 0;
+                    }
+                }
+                dst_start += dst->stride;
+                src_start += src->stride / 4;
+            }
+        }
+        else
+        {
+            FIXME("Unsupported conversion: 32 -> 1\n");
+            return FALSE;
+        }
+        break;
+    }
+
+    case 24:
+    {
+        BYTE *src_start = get_pixel_ptr_24(src, src_rect->left, src_rect->top), *src_pixel;
+
+        for(y = src_rect->top; y < src_rect->bottom; y++)
+        {
+            dst_pixel = dst_start;
+            src_pixel = src_start;
+            for(x = src_rect->left, bit_pos = 0; x < src_rect->right; x++)
+            {
+                RGBQUAD rgb;
+                rgb.rgbBlue  = *src_pixel++;
+                rgb.rgbGreen = *src_pixel++;
+                rgb.rgbRed   = *src_pixel++;
+
+                dst_val = colorref_to_pixel_colortable(dst, ( rgb.rgbRed         & 0x0000ff) |
+                                                            ((rgb.rgbGreen << 8) & 0x00ff00) |
+                                                            ((rgb.rgbBlue << 16) & 0xff0000)) ? 0xff : 0;
+
+                if(bit_pos == 0) *dst_pixel = 0;
+                *dst_pixel = (*dst_pixel & ~pixel_masks_1[bit_pos]) | (dst_val & pixel_masks_1[bit_pos]);
+
+                if(++bit_pos == 8)
+                {
+                    dst_pixel++;
+                    bit_pos = 0;
+                }
+            }
+            dst_start += dst->stride;
+            src_start += src->stride;
+        }
+        break;
+    }
+
+    case 16:
+    {
+        WORD *src_start = get_pixel_ptr_16(src, src_rect->left, src_rect->top), *src_pixel;
+        if(src->funcs == &funcs_555)
+        {
+            for(y = src_rect->top; y < src_rect->bottom; y++)
+            {
+                dst_pixel = dst_start;
+                src_pixel = src_start;
+                for(x = src_rect->left, bit_pos = 0; x < src_rect->right; x++)
+                {
+                    src_val = *src_pixel++;
+                    dst_val = colorref_to_pixel_colortable(dst, ((src_val >>  7) & 0x0000f8) | ((src_val >> 12) & 0x000007) |
+                                                                ((src_val <<  6) & 0x00f800) | ((src_val <<  1) & 0x000700) |
+                                                                ((src_val << 19) & 0xf80000) | ((src_val << 14) & 0x070000) ) ? 0xff : 0;
+
+                    if(bit_pos == 0) *dst_pixel = 0;
+                    *dst_pixel = (*dst_pixel & ~pixel_masks_1[bit_pos]) | (dst_val & pixel_masks_1[bit_pos]);
+
+                    if(++bit_pos == 8)
+                    {
+                        dst_pixel++;
+                        bit_pos = 0;
+                    }
+                }
+                dst_start += dst->stride;
+                src_start += src->stride / 2;
+            }
+        }
+        else
+        {
+            FIXME("Unsupported conversion: 16 -> 1\n");
+            return FALSE;
+        }
+        break;
+    }
+
+    case 8:
+    {
+        BYTE *src_start = get_pixel_ptr_8(src, src_rect->left, src_rect->top), *src_pixel;
+
+        for(y = src_rect->top; y < src_rect->bottom; y++)
+        {
+            dst_pixel = dst_start;
+            src_pixel = src_start;
+            for(x = src_rect->left, bit_pos = 0; x < src_rect->right; x++)
+            {
+                RGBQUAD rgb;
+                src_val = *src_pixel++;
+                if(src_val >= src->color_table_size) src_val = src->color_table_size - 1;
+                rgb = src->color_table[src_val];
+                dst_val = colorref_to_pixel_colortable(dst, RGB(rgb.rgbRed, rgb.rgbGreen, rgb.rgbBlue)) ? 0xff : 0;
+
+                if(bit_pos == 0) *dst_pixel = 0;
+                *dst_pixel = (*dst_pixel & ~pixel_masks_1[bit_pos]) | (dst_val & pixel_masks_1[bit_pos]);
+
+                if(++bit_pos == 8)
+                {
+                    dst_pixel++;
+                    bit_pos = 0;
+                }
+            }
+            dst_start += dst->stride;
+            src_start += src->stride;
+        }
+        break;
+    }
+
+    case 4:
+    {
+        BYTE *src_start = get_pixel_ptr_4(src, src_rect->left, src_rect->top), *src_pixel;
+
+        for(y = src_rect->top; y < src_rect->bottom; y++)
+        {
+            dst_pixel = dst_start;
+            src_pixel = src_start;
+            for(x = src_rect->left, bit_pos = 0; x < src_rect->right; x++)
+            {
+                RGBQUAD rgb;
+                if(x & 1)
+                    src_val = *src_pixel++ & 0xf;
+                else
+                    src_val = (*src_pixel >> 4) & 0xf;
+                if(src_val >= src->color_table_size) src_val = src->color_table_size - 1;
+                rgb = src->color_table[src_val];
+                dst_val = colorref_to_pixel_colortable(dst, RGB(rgb.rgbRed, rgb.rgbGreen, rgb.rgbBlue)) ? 0xff : 0;
+
+                if(bit_pos == 0) *dst_pixel = 0;
+                *dst_pixel = (*dst_pixel & ~pixel_masks_1[bit_pos]) | (dst_val & pixel_masks_1[bit_pos]);
+
+                if(++bit_pos == 8)
+                {
+                    dst_pixel++;
+                    bit_pos = 0;
+                }
+            }
+            dst_start += dst->stride;
+            src_start += src->stride;
+        }
+        break;
+    }
+
+    /* Note that while MSDN states that a 1 bpp dib brush -> mono dc
+       uses text/bkgnd colours instead of the dib's colour table, this
+       doesn't appear to be the case for a dc backed by a
+       dibsection. */
+
+    case 1:
+    {
+        BYTE *src_start = get_pixel_ptr_1(src, src_rect->left, src_rect->top), *src_pixel;
+        for(y = src_rect->top; y < src_rect->bottom; y++)
+        {
+            dst_pixel = dst_start;
+            src_pixel = src_start;
+            for(x = src_rect->left, bit_pos = 0; x < src_rect->right; x++)
+            {
+                RGBQUAD rgb;
+                src_val = (*src_pixel & pixel_masks_1[x % 8]) ? 1 : 0;
+                if((x % 8) == 7) src_pixel++;
+                rgb = src->color_table[src_val];
+                dst_val = colorref_to_pixel_colortable(dst, RGB(rgb.rgbRed, rgb.rgbGreen, rgb.rgbBlue)) ? 0xff : 0;
+
+                if(bit_pos == 0) *dst_pixel = 0;
+                *dst_pixel = (*dst_pixel & ~pixel_masks_1[bit_pos]) | (dst_val & pixel_masks_1[bit_pos]);
+
+                if(++bit_pos == 8)
+                {
+                    dst_pixel++;
+                    bit_pos = 0;
+                }
+            }
+            dst_start += dst->stride;
+            src_start += src->stride;
+        }
+        break;
+    }
+
+    default:
+        FIXME("Unsupported conversion: %d -> 1\n", src->bit_count);
+        return FALSE;
+
+    }
+    return TRUE;
+}
+
 static BOOL convert_to_null(dib_info *dst, const dib_info *src, const RECT *src_rect)
 {
+    return FALSE;
+}
+
+static BOOL create_rop_masks_32(const dib_info *dib, const dib_info *hatch, const rop_mask *fg, const rop_mask *bg, rop_mask_bits *bits)
+{
+    BYTE *hatch_start = hatch->bits, *hatch_ptr;
+    DWORD mask_start = 0, mask_offset;
+    DWORD *and_bits = bits->and, *xor_bits = bits->xor;
+    int x, y;
+
+    for(y = 0; y < hatch->height; y++)
+    {
+        hatch_ptr = hatch_start;
+        mask_offset = mask_start;
+        for(x = 0; x < hatch->width; x++)
+        {
+            if(*hatch_ptr & pixel_masks_1[x % 8])
+            {
+                and_bits[mask_offset] = fg->and;
+                xor_bits[mask_offset] = fg->xor;
+            }
+            else
+            {
+                and_bits[mask_offset] = bg->and;
+                xor_bits[mask_offset] = bg->xor;
+            }
+            if(x % 8 == 7) hatch_ptr++;
+            mask_offset++;
+        }
+        hatch_start += hatch->stride;
+        mask_start += dib->stride / 4;
+    }
+
     return TRUE;
 }
 
+static BOOL create_rop_masks_24(const dib_info *dib, const dib_info *hatch, const rop_mask *fg, const rop_mask *bg, rop_mask_bits *bits)
+{
+    BYTE *hatch_start = hatch->bits, *hatch_ptr;
+    DWORD mask_start = 0, mask_offset;
+    BYTE *and_bits = bits->and, *xor_bits = bits->xor;
+    int x, y;
+
+    for(y = 0; y < hatch->height; y++)
+    {
+        hatch_ptr = hatch_start;
+        mask_offset = mask_start;
+        for(x = 0; x < hatch->width; x++)
+        {
+            if(*hatch_ptr & pixel_masks_1[x % 8])
+            {
+                and_bits[mask_offset]   =  fg->and        & 0xff;
+                xor_bits[mask_offset++] =  fg->xor        & 0xff;
+                and_bits[mask_offset]   = (fg->and >>  8) & 0xff;
+                xor_bits[mask_offset++] = (fg->xor >>  8) & 0xff;
+                and_bits[mask_offset]   = (fg->and >> 16) & 0xff;
+                xor_bits[mask_offset++] = (fg->xor >> 16) & 0xff;
+            }
+            else
+            {
+                and_bits[mask_offset]   =  bg->and        & 0xff;
+                xor_bits[mask_offset++] =  bg->xor        & 0xff;
+                and_bits[mask_offset]   = (bg->and >>  8) & 0xff;
+                xor_bits[mask_offset++] = (bg->xor >>  8) & 0xff;
+                and_bits[mask_offset]   = (bg->and >> 16) & 0xff;
+                xor_bits[mask_offset++] = (bg->xor >> 16) & 0xff;
+            }
+            if(x % 8 == 7) hatch_ptr++;
+        }
+        hatch_start += hatch->stride;
+        mask_start += dib->stride;
+    }
+
+    return TRUE;
+}
+
+static BOOL create_rop_masks_16(const dib_info *dib, const dib_info *hatch, const rop_mask *fg, const rop_mask *bg, rop_mask_bits *bits)
+{
+    BYTE *hatch_start = hatch->bits, *hatch_ptr;
+    DWORD mask_start = 0, mask_offset;
+    WORD *and_bits = bits->and, *xor_bits = bits->xor;
+    int x, y;
+
+    for(y = 0; y < hatch->height; y++)
+    {
+        hatch_ptr = hatch_start;
+        mask_offset = mask_start;
+        for(x = 0; x < hatch->width; x++)
+        {
+            if(*hatch_ptr & pixel_masks_1[x % 8])
+            {
+                and_bits[mask_offset] = fg->and;
+                xor_bits[mask_offset] = fg->xor;
+            }
+            else
+            {
+                and_bits[mask_offset] = bg->and;
+                xor_bits[mask_offset] = bg->xor;
+            }
+            if(x % 8 == 7) hatch_ptr++;
+            mask_offset++;
+        }
+        hatch_start += hatch->stride;
+        mask_start += dib->stride / 2;
+    }
+
+    return TRUE;
+}
+
+static BOOL create_rop_masks_8(const dib_info *dib, const dib_info *hatch, const rop_mask *fg, const rop_mask *bg, rop_mask_bits *bits)
+{
+    BYTE *hatch_start = hatch->bits, *hatch_ptr;
+    DWORD mask_start = 0, mask_offset;
+    BYTE *and_bits = bits->and, *xor_bits = bits->xor;
+    int x, y;
+
+    for(y = 0; y < hatch->height; y++)
+    {
+        hatch_ptr = hatch_start;
+        mask_offset = mask_start;
+        for(x = 0; x < hatch->width; x++)
+        {
+            if(*hatch_ptr & pixel_masks_1[x % 8])
+            {
+                and_bits[mask_offset] = fg->and;
+                xor_bits[mask_offset] = fg->xor;
+            }
+            else
+            {
+                and_bits[mask_offset] = bg->and;
+                xor_bits[mask_offset] = bg->xor;
+            }
+            if(x % 8 == 7) hatch_ptr++;
+            mask_offset++;
+        }
+        hatch_start += hatch->stride;
+        mask_start += dib->stride;
+    }
+
+    return TRUE;
+}
+
+static BOOL create_rop_masks_4(const dib_info *dib, const dib_info *hatch, const rop_mask *fg, const rop_mask *bg, rop_mask_bits *bits)
+{
+    BYTE *hatch_start = hatch->bits, *hatch_ptr;
+    DWORD mask_start = 0, mask_offset;
+    BYTE *and_bits = bits->and, *xor_bits = bits->xor;
+    const rop_mask *rop_mask;
+    int x, y;
+
+    for(y = 0; y < hatch->height; y++)
+    {
+        hatch_ptr = hatch_start;
+        mask_offset = mask_start;
+        for(x = 0; x < hatch->width; x++)
+        {
+            if(*hatch_ptr & pixel_masks_1[x % 8])
+                rop_mask = fg;
+            else
+                rop_mask = bg;
+
+            if(x & 1)
+            {
+                and_bits[mask_offset] = (rop_mask->and & 0x0f) | (and_bits[mask_offset] & 0xf0);
+                xor_bits[mask_offset] = (rop_mask->xor & 0x0f) | (xor_bits[mask_offset] & 0xf0);
+                mask_offset++;
+            }
+            else
+            {
+                and_bits[mask_offset] = (rop_mask->and << 4) & 0xf0;
+                xor_bits[mask_offset] = (rop_mask->xor << 4) & 0xf0;
+            }
+
+            if(x % 8 == 7) hatch_ptr++;
+        }
+        hatch_start += hatch->stride;
+        mask_start += dib->stride;
+    }
+
+    return TRUE;
+}
+
+static BOOL create_rop_masks_1(const dib_info *dib, const dib_info *hatch, const rop_mask *fg, const rop_mask *bg, rop_mask_bits *bits)
+{
+    BYTE *hatch_start = hatch->bits, *hatch_ptr;
+    DWORD mask_start = 0, mask_offset;
+    BYTE *and_bits = bits->and, *xor_bits = bits->xor;
+    rop_mask rop_mask;
+    int x, y, bit_pos;
+
+    for(y = 0; y < hatch->height; y++)
+    {
+        hatch_ptr = hatch_start;
+        mask_offset = mask_start;
+        for(x = 0, bit_pos = 0; x < hatch->width; x++)
+        {
+            if(*hatch_ptr & pixel_masks_1[x % 8])
+            {
+                rop_mask.and = (fg->and & 1) ? 0xff : 0;
+                rop_mask.xor = (fg->xor & 1) ? 0xff : 0;
+            }
+            else
+            {
+                rop_mask.and = (bg->and & 1) ? 0xff : 0;
+                rop_mask.xor = (bg->xor & 1) ? 0xff : 0;
+            }
+
+            if(bit_pos == 0) and_bits[mask_offset] = xor_bits[mask_offset] = 0;
+
+            and_bits[mask_offset] = (rop_mask.and & pixel_masks_1[bit_pos]) | (and_bits[mask_offset] & ~pixel_masks_1[bit_pos]);
+            xor_bits[mask_offset] = (rop_mask.xor & pixel_masks_1[bit_pos]) | (xor_bits[mask_offset] & ~pixel_masks_1[bit_pos]);
+
+            if(++bit_pos == 8)
+            {
+                mask_offset++;
+                hatch_ptr++;
+                bit_pos = 0;
+            }
+        }
+        hatch_start += hatch->stride;
+        mask_start += dib->stride;
+    }
+
+    return TRUE;
+}
+
+static BOOL create_rop_masks_null(const dib_info *dib, const dib_info *hatch, const rop_mask *fg, const rop_mask *bg, rop_mask_bits *bits)
+{
+    return FALSE;
+}
+
 const primitive_funcs funcs_8888 =
 {
     solid_rects_32,
     pattern_rects_32,
     colorref_to_pixel_888,
-    convert_to_8888
+    convert_to_8888,
+    create_rop_masks_32
 };
 
 const primitive_funcs funcs_32 =
@@ -1692,7 +2436,8 @@ const primitive_funcs funcs_32 =
     solid_rects_32,
     pattern_rects_32,
     colorref_to_pixel_masks,
-    convert_to_32
+    convert_to_32,
+    create_rop_masks_32
 };
 
 const primitive_funcs funcs_24 =
@@ -1700,7 +2445,8 @@ const primitive_funcs funcs_24 =
     solid_rects_24,
     pattern_rects_24,
     colorref_to_pixel_888,
-    convert_to_24
+    convert_to_24,
+    create_rop_masks_24
 };
 
 const primitive_funcs funcs_555 =
@@ -1708,7 +2454,8 @@ const primitive_funcs funcs_555 =
     solid_rects_16,
     pattern_rects_16,
     colorref_to_pixel_555,
-    convert_to_555
+    convert_to_555,
+    create_rop_masks_16
 };
 
 const primitive_funcs funcs_16 =
@@ -1716,7 +2463,8 @@ const primitive_funcs funcs_16 =
     solid_rects_16,
     pattern_rects_16,
     colorref_to_pixel_masks,
-    convert_to_16
+    convert_to_16,
+    create_rop_masks_16
 };
 
 const primitive_funcs funcs_8 =
@@ -1724,7 +2472,8 @@ const primitive_funcs funcs_8 =
     solid_rects_8,
     pattern_rects_8,
     colorref_to_pixel_colortable,
-    convert_to_8
+    convert_to_8,
+    create_rop_masks_8
 };
 
 const primitive_funcs funcs_4 =
@@ -1732,7 +2481,17 @@ const primitive_funcs funcs_4 =
     solid_rects_4,
     pattern_rects_4,
     colorref_to_pixel_colortable,
-    convert_to_4
+    convert_to_4,
+    create_rop_masks_4
+};
+
+const primitive_funcs funcs_1 =
+{
+    solid_rects_1,
+    pattern_rects_1,
+    colorref_to_pixel_colortable,
+    convert_to_1,
+    create_rop_masks_1
 };
 
 const primitive_funcs funcs_null =
@@ -1740,5 +2499,6 @@ const primitive_funcs funcs_null =
     solid_rects_null,
     pattern_rects_null,
     colorref_to_pixel_null,
-    convert_to_null
+    convert_to_null,
+    create_rop_masks_null
 };
