@@ -26,6 +26,7 @@
 #include "windef.h"
 #include "winbase.h"
 #include "wingdi.h"
+#include "wine/gdi_driver.h"
 
 /* Metafile defines */
 #define META_EOF 0x0000
@@ -72,13 +73,6 @@ typedef struct tagGDIOBJHDR
 
 /* Device functions for the Wine driver interface */
 
-typedef struct gdi_physdev
-{
-    const struct tagDC_FUNCS *funcs;
-    struct gdi_physdev       *next;
-    HDC                       hdc;
-} *PHYSDEV;
-
 typedef struct
 {
     int bit_count, width, height;
@@ -109,6 +103,18 @@ typedef struct
     BOOL mark;
 } dash_pos;
 
+typedef struct
+{
+    DWORD and;
+    DWORD xor;
+} rop_mask;
+
+typedef struct
+{
+    void *and;
+    void *xor;
+} rop_mask_bits;
+
 typedef struct dibdrv_physdev
 {
     struct gdi_physdev dev;
@@ -126,6 +132,7 @@ typedef struct dibdrv_physdev
 
     /* brush */
     UINT brush_style;
+    UINT brush_hatch;
     INT brush_rop;   /* PatBlt, for example, can override the DC's rop2 */
     COLORREF brush_colorref;
     DWORD brush_color, brush_and, brush_xor;
@@ -141,7 +148,7 @@ typedef struct dibdrv_physdev
 #define DEFER_PEN        2
 #define DEFER_BRUSH      4
 
-typedef struct tagDC_FUNCS
+typedef struct gdi_dc_funcs
 {
     INT      (CDECL *pAbortDoc)(PHYSDEV);
     BOOL     (CDECL *pAbortPath)(PHYSDEV);
