@@ -547,7 +547,7 @@ static HRESULT WINAPI MMDevice_Activate(IMMDevice *iface, REFIID riid, DWORD cls
     else if (IsEqualIID(riid, &IID_IAudioSessionManager)
              || IsEqualIID(riid, &IID_IAudioSessionManager2))
     {
-        hr = drvs.pGetAudioSessionManager(This->flow, (IAudioSessionManager2**)ppv);
+        hr = drvs.pGetAudioSessionManager(iface, (IAudioSessionManager2**)ppv);
     }
     else if (IsEqualIID(riid, &IID_IBaseFilter))
     {
@@ -824,6 +824,9 @@ HRESULT MMDevEnum_Create(REFIID riid, void **ppv)
 {
     MMDevEnumImpl *This = MMDevEnumerator;
 
+    if(!drvs.pGetAudioEndpoint)
+        return AUDCLNT_E_SERVICE_NOT_RUNNING;
+
     if (!This)
     {
         This = HeapAlloc(GetProcessHeap(), 0, sizeof(*This));
