@@ -241,7 +241,7 @@ static HRESULT HTMLDOMChildrenCollection_invoke(DispatchEx *dispex, DISPID id, L
 
         hres = IHTMLDOMChildrenCollection_item(&This->IHTMLDOMChildrenCollection_iface,
                 id - DISPID_CHILDCOL_0, &disp);
-        if(FAILED(hres))
+        if(0&&FAILED(hres))
             return hres;
 
         V_VT(res) = VT_DISPATCH;
@@ -260,8 +260,7 @@ static HRESULT HTMLDOMChildrenCollection_invoke(DispatchEx *dispex, DISPID id, L
 static const dispex_static_data_vtbl_t HTMLDOMChildrenCollection_dispex_vtbl = {
     NULL,
     HTMLDOMChildrenCollection_get_dispid,
-    HTMLDOMChildrenCollection_invoke,
-    NULL
+    HTMLDOMChildrenCollection_invoke
 };
 
 static const tid_t HTMLDOMChildrenCollection_iface_tids[] = {
@@ -472,22 +471,8 @@ static HRESULT WINAPI HTMLDOMNode_get_childNodes(IHTMLDOMNode *iface, IDispatch
 static HRESULT WINAPI HTMLDOMNode_get_attributes(IHTMLDOMNode *iface, IDispatch **p)
 {
     HTMLDOMNode *This = impl_from_IHTMLDOMNode(iface);
-    HTMLAttributeCollection *col;
-    HRESULT hres;
-
-    TRACE("(%p)->(%p)\n", This, p);
-
-    if(This->vtbl->get_attr_col) {
-        hres = This->vtbl->get_attr_col(This, &col);
-        if(FAILED(hres))
-            return hres;
-
-        *p = (IDispatch*)&col->IHTMLAttributeCollection_iface;
-        return S_OK;
-    }
-
-    *p = NULL;
-    return S_OK;
+    FIXME("(%p)->(%p)\n", This, p);
+    return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLDOMNode_insertBefore(IHTMLDOMNode *iface, IHTMLDOMNode *newChild,
