@@ -19,6 +19,7 @@
 #include "config.h"
 
 #include <stdarg.h>
+#include <assert.h>
 
 #define COBJMACROS
 
@@ -63,13 +64,29 @@ struct  nsWineURI {
     NSContainer *container;
     windowref_t *window_ref;
     nsChannelBSC *channel_bsc;
-    BSTR wine_url;
     IUri *uri;
     IUriBuilder *uri_builder;
     BOOL is_doc_uri;
 };
 
-static nsresult create_uri(nsIURI*,HTMLWindow*,NSContainer*,nsWineURI**);
+static BOOL ensure_uri(nsWineURI *This)
+{
+    HRESULT hres;
+
+    assert(This->uri || This->uri_builder);
+
+    if(!This->uri) {
+        hres = IUriBuilder_CreateUriSimple(This->uri_builder, 0, 0, &This->uri);
+        if(FAILED(hres)) {
+            WARN("CreateUriSimple failed: %08x\n", hres);
+            return FALSE;
+        }
+    }
+
+    return TRUE;
+}
+
+static nsresult create_nsuri(IUri*,nsIURI*,HTMLWindow*,NSContainer*,nsWineURI**);
 
 static const char *debugstr_nsacstr(const nsACString *nsstr)
 {
@@ -100,7 +117,7 @@ HRESULT nsuri_to_url(LPCWSTR nsuri, BOOL ret_empty, BSTR *ret)
     return S_OK;
 }
 
-static BOOL exec_shldocvw_67(HTMLDocumentObj *doc, LPCWSTR url)
+static BOOL exec_shldocvw_67(HTMLDocumentObj *doc, BSTR url)
 {
     IOleCommandTarget *cmdtrg = NULL;
     HRESULT hres;
@@ -110,13 +127,12 @@ static BOOL exec_shldocvw_67(HTMLDocumentObj *doc, LPCWSTR url)
         VARIANT varUrl, varRes;
 
         V_VT(&varUrl) = VT_BSTR;
-        V_BSTR(&varUrl) = SysAllocString(url);
+        V_BSTR(&varUrl) = url;
         V_VT(&varRes) = VT_BOOL;
 
         hres = IOleCommandTarget_Exec(cmdtrg, &CGID_ShellDocView, 67, 0, &varUrl, &varRes);
 
         IOleCommandTarget_Release(cmdtrg);
-        SysFreeString(V_BSTR(&varUrl));
 
         if(SUCCEEDED(hres) && !V_BOOL(&varRes)) {
             TRACE("got VARIANT_FALSE, do not load\n");
@@ -127,11 +143,11 @@ static BOOL exec_shldocvw_67(HTMLDocumentObj *doc, LPCWSTR url)
     return TRUE;
 }
 
-static BOOL before_async_open(nsChannel *channel, NSContainer *container)
+static nsresult before_async_open(nsChannel *channel, NSContainer *container, BOOL *cancel)
 {
     HTMLDocumentObj *doc = container->doc;
+    BSTR display_uri;
     DWORD hlnf = 0;
-    BOOL cancel;
     HRESULT hres;
 
     if(!doc) {
@@ -143,14 +159,31 @@ static BOOL before_async_open(nsChannel *channel, NSContainer *container)
         doc = container_iter->doc;
     }
 
-    if(!doc->client)
-        return TRUE;
+    if(!doc->client) {
+        *cancel = TRUE;
+        return NS_OK;
+    }
 
-    if(!hlnf && !exec_shldocvw_67(doc, channel->uri->wine_url))
-        return FALSE;
+    hres = IUri_GetDisplayUri(channel->uri->uri, &display_uri);
+    if(FAILED(hres))
+        return NS_ERROR_FAILURE;
 
-    hres = hlink_frame_navigate(&doc->basedoc, channel->uri->wine_url, channel, hlnf, &cancel);
-    return FAILED(hres) || cancel;
+    if(!hlnf) {
+        BOOL b;
+
+        b = !exec_shldocvw_67(doc, display_uri);
+        if(b) {
+            SysFreeString(display_uri);
+            *cancel = FALSE;
+            return NS_OK;
+        }
+    }
+
+    hres = hlink_frame_navigate(&doc->basedoc, display_uri, channel, hlnf, cancel);
+    SysFreeString(display_uri);
+    if(FAILED(hres))
+        *cancel = TRUE;
+    return NS_OK;
 }
 
 HRESULT load_nsuri(HTMLWindow *window, nsWineURI *uri, nsChannelBSC *channelbsc, DWORD flags)
@@ -189,26 +222,6 @@ HRESULT load_nsuri(HTMLWindow *window, nsWineURI *uri, nsChannelBSC *channelbsc,
     return S_OK;
 }
 
-static HRESULT set_wine_url(nsWineURI *This, LPCWSTR url)
-{
-    TRACE("(%p)->(%s)\n", This, debugstr_w(url));
-
-    if(url) {
-        BSTR new_url;
-
-        new_url = SysAllocString(url);
-        if(!new_url)
-            return E_OUTOFMEMORY;
-        SysFreeString(This->wine_url);
-        This->wine_url = new_url;
-    }else {
-        heap_free(This->wine_url);
-        This->wine_url = NULL;
-    }
-
-    return S_OK;
-}
-
 static void set_uri_nscontainer(nsWineURI *This, NSContainer *nscontainer)
 {
     if(This->container) {
@@ -886,7 +899,7 @@ static nsresult async_open(nsChannel *This, HTMLWindow *window, BOOL is_doc_chan
     IMoniker *mon = NULL;
     HRESULT hres;
 
-    hres = CreateURLMoniker(NULL, This->uri->wine_url, &mon);
+    hres = CreateURLMonikerEx2(NULL, This->uri->uri, &mon, 0);
     if(FAILED(hres)) {
         WARN("CreateURLMoniker failed: %08x\n", hres);
         return NS_ERROR_UNEXPECTED;
@@ -922,10 +935,13 @@ static nsresult NSAPI nsChannel_AsyncOpen(nsIHttpChannel *iface, nsIStreamListen
 {
     nsChannel *This = impl_from_nsIHttpChannel(iface);
     HTMLWindow *window = NULL;
-    BOOL open = TRUE;
+    BOOL cancel = FALSE;
     nsresult nsres = NS_OK;
 
-    TRACE("(%p)->(%p %p) opening %s\n", This, aListener, aContext, debugstr_w(This->uri->wine_url));
+    TRACE("(%p)->(%p %p)\n", This, aListener, aContext);
+
+    if(!ensure_uri(This->uri))
+        return NS_ERROR_FAILURE;
 
     if(This->uri->is_doc_uri) {
         window = get_channel_window(This);
@@ -938,7 +954,9 @@ static nsresult NSAPI nsChannel_AsyncOpen(nsIHttpChannel *iface, nsIStreamListen
             if(This->uri->container->doc)
                 FIXME("nscontainer->doc = %p\n", This->uri->container->doc);
 
-            b = before_async_open(This, This->uri->container);
+            nsres = before_async_open(This, This->uri->container, &b);
+            if(NS_FAILED(nsres))
+                return nsres;
             if(b)
                 FIXME("Navigation not cancelled\n");
             return NS_ERROR_UNEXPECTED;
@@ -974,17 +992,17 @@ static nsresult NSAPI nsChannel_AsyncOpen(nsIHttpChannel *iface, nsIStreamListen
                 This->content_type = heap_strdupWtoA(window->doc_obj->mime);
             }
 
-            open = FALSE;
+            cancel = TRUE;
         }else {
-            open = !before_async_open(This, window->doc_obj->nscontainer);
-            if(!open) {
+            nsres = before_async_open(This, window->doc_obj->nscontainer, &cancel);
+            if(NS_SUCCEEDED(nsres)  && cancel) {
                 TRACE("canceled\n");
                 nsres = NS_ERROR_UNEXPECTED;
             }
         }
     }
 
-    if(open)
+    if(!cancel)
         nsres = async_open(This, window, This->uri->is_doc_uri, aListener, aContext);
 
     if(NS_SUCCEEDED(nsres) && This->load_group) {
@@ -1500,31 +1518,6 @@ static const nsIHttpChannelInternalVtbl nsHttpChannelInternalVtbl = {
     nsHttpChannelInternal_SetChannelIsForDownload
 };
 
-static BOOL ensure_uri(nsWineURI *This)
-{
-    HRESULT hres;
-
-    if(This->uri)
-        return TRUE;
-
-    if(This->uri_builder) {
-        hres = IUriBuilder_CreateUriSimple(This->uri_builder, 0, 0, &This->uri);
-        if(FAILED(hres)) {
-            WARN("CreateUriSimple failed: %08x\n", hres);
-            return FALSE;
-        }
-
-        return TRUE;
-    }
-
-    hres = CreateUri(This->wine_url, 0, 0, &This->uri);
-    if(FAILED(hres)) {
-        WARN("CreateUri failed: %08x\n", hres);
-        return FALSE;
-    }
-
-    return TRUE;
-}
 
 static void invalidate_uri(nsWineURI *This)
 {
@@ -1553,23 +1546,6 @@ static BOOL ensure_uri_builder(nsWineURI *This)
     return TRUE;
 }
 
-/* Temporary helper */
-static void sync_wine_url(nsWineURI *This)
-{
-    BSTR new_url;
-    HRESULT hres;
-
-    if(!ensure_uri(This))
-        return;
-
-    hres = IUri_GetDisplayUri(This->uri, &new_url);
-    if(FAILED(hres))
-        return;
-
-    SysFreeString(This->wine_url);
-    This->wine_url = new_url;
-}
-
 static nsresult get_uri_string(nsWineURI *This, Uri_PROPERTY prop, nsACString *ret)
 {
     char *vala;
@@ -1658,7 +1634,6 @@ static nsrefcnt NSAPI nsURI_Release(nsIURL *iface)
             nsIURI_Release(This->nsuri);
         if(This->uri)
             IUri_Release(This->uri);
-        SysFreeString(This->wine_url);
         heap_free(This);
     }
 
@@ -1703,7 +1678,6 @@ static nsresult NSAPI nsURI_SetSpec(nsIURL *iface, const nsACString *aSpec)
     }
 
     This->uri = uri;
-    sync_wine_url(This);
     return NS_OK;
 }
 
@@ -1767,7 +1741,6 @@ static nsresult NSAPI nsURI_SetScheme(nsIURL *iface, const nsACString *aScheme)
     if(FAILED(hres))
         return NS_ERROR_UNEXPECTED;
 
-    sync_wine_url(This);
     return NS_OK;
 }
 
@@ -1829,7 +1802,6 @@ static nsresult NSAPI nsURI_SetUsername(nsIURL *iface, const nsACString *aUserna
     if(FAILED(hres))
         return NS_ERROR_UNEXPECTED;
 
-    sync_wine_url(This);
     return NS_OK;
 }
 
@@ -1864,7 +1836,6 @@ static nsresult NSAPI nsURI_SetPassword(nsIURL *iface, const nsACString *aPasswo
     if(FAILED(hres))
         return NS_ERROR_UNEXPECTED;
 
-    sync_wine_url(This);
     return NS_OK;
 }
 
@@ -1943,7 +1914,6 @@ static nsresult NSAPI nsURI_SetHost(nsIURL *iface, const nsACString *aHost)
     if(FAILED(hres))
         return NS_ERROR_UNEXPECTED;
 
-    sync_wine_url(This);
     return NS_OK;
 }
 
@@ -1955,9 +1925,6 @@ static nsresult NSAPI nsURI_GetPort(nsIURL *iface, PRInt32 *aPort)
 
     TRACE("(%p)->(%p)\n", This, aPort);
 
-    if(This->nsuri)
-        return nsIURI_GetPort(This->nsuri, aPort);
-
     if(!ensure_uri(This))
         return NS_ERROR_UNEXPECTED;
 
@@ -1974,16 +1941,15 @@ static nsresult NSAPI nsURI_GetPort(nsIURL *iface, PRInt32 *aPort)
 static nsresult NSAPI nsURI_SetPort(nsIURL *iface, PRInt32 aPort)
 {
     nsWineURI *This = impl_from_nsIURL(iface);
+    HRESULT hres;
 
     TRACE("(%p)->(%d)\n", This, aPort);
 
-    if(This->nsuri) {
-        invalidate_uri(This);
-        return nsIURI_SetPort(This->nsuri, aPort);
-    }
+    if(!ensure_uri_builder(This))
+        return NS_ERROR_UNEXPECTED;
 
-    FIXME("default action not implemented\n");
-    return NS_ERROR_NOT_IMPLEMENTED;
+    hres = IUriBuilder_SetPort(This->uri_builder, aPort != -1, aPort);
+    return SUCCEEDED(hres) ? NS_OK : NS_ERROR_FAILURE;
 }
 
 static nsresult NSAPI nsURI_GetPath(nsIURL *iface, nsACString *aPath)
@@ -2017,7 +1983,6 @@ static nsresult NSAPI nsURI_SetPath(nsIURL *iface, const nsACString *aPath)
     if(FAILED(hres))
         return NS_ERROR_UNEXPECTED;
 
-    sync_wine_url(This);
     return NS_OK;
 }
 
@@ -2090,16 +2055,12 @@ static nsresult NSAPI nsURI_Clone(nsIURL *iface, nsIURI **_retval)
         }
     }
 
-    nsres = create_uri(nsuri, This->window_ref ? This->window_ref->window : NULL, This->container, &wine_uri);
+    nsres = create_nsuri(This->uri, nsuri, This->window_ref ? This->window_ref->window : NULL, This->container, &wine_uri);
     if(NS_FAILED(nsres)) {
-        WARN("create_uri failed: %08x\n", nsres);
+        WARN("create_nsuri failed: %08x\n", nsres);
         return nsres;
     }
 
-    wine_uri->uri = This->uri;
-    IUri_AddRef(wine_uri->uri);
-    sync_wine_url(wine_uri);
-
     *_retval = (nsIURI*)&wine_uri->nsIURL_iface;
     return NS_OK;
 }
@@ -2255,7 +2216,6 @@ static nsresult NSAPI nsURL_SetQuery(nsIURL *iface, const nsACString *aQuery)
     if(FAILED(hres))
         return NS_ERROR_UNEXPECTED;
 
-    sync_wine_url(This);
     return NS_OK;
 }
 
@@ -2307,7 +2267,6 @@ static nsresult NSAPI nsURL_SetRef(nsIURL *iface, const nsACString *aRef)
     if(FAILED(hres))
         return NS_ERROR_UNEXPECTED;
 
-    sync_wine_url(This);
     return NS_OK;
 }
 
@@ -2494,7 +2453,7 @@ static const nsIURLVtbl nsURLVtbl = {
     nsURL_GetRelativeSpec
 };
 
-static nsresult create_uri(nsIURI *nsuri, HTMLWindow *window, NSContainer *container, nsWineURI **_retval)
+static nsresult create_nsuri(IUri *iuri, nsIURI *nsuri, HTMLWindow *window, NSContainer *container, nsWineURI **_retval)
 {
     nsWineURI *ret = heap_alloc_zero(sizeof(nsWineURI));
 
@@ -2505,6 +2464,9 @@ static nsresult create_uri(nsIURI *nsuri, HTMLWindow *window, NSContainer *conta
     set_uri_nscontainer(ret, container);
     set_uri_window(ret, window);
 
+    IUri_AddRef(iuri);
+    ret->uri = iuri;
+
     if(nsuri)
         nsIURI_QueryInterface(nsuri, &IID_nsIURL, (void**)&ret->nsurl);
 
@@ -2516,13 +2478,19 @@ static nsresult create_uri(nsIURI *nsuri, HTMLWindow *window, NSContainer *conta
 HRESULT create_doc_uri(HTMLWindow *window, WCHAR *url, nsWineURI **ret)
 {
     nsWineURI *uri;
+    IUri *iuri;
     nsresult nsres;
+    HRESULT hres;
 
-    nsres = create_uri(NULL, window, window->doc_obj->nscontainer, &uri);
+    hres = CreateUri(url, 0, 0, &iuri);
+    if(FAILED(hres))
+        return hres;
+
+    nsres = create_nsuri(iuri, NULL, window, window->doc_obj->nscontainer, &uri);
+    IUri_Release(iuri);
     if(NS_FAILED(nsres))
         return E_FAIL;
 
-    set_wine_url(uri, url);
     uri->is_doc_uri = TRUE;
 
     *ret = uri;
@@ -2565,22 +2533,23 @@ HRESULT create_redirect_nschannel(const WCHAR *url, nsChannel *orig_channel, nsC
     HTMLWindow *window = NULL;
     nsChannel *channel;
     nsWineURI *uri;
+    IUri *iuri;
     nsresult nsres;
     HRESULT hres;
 
+    hres = CreateUri(url, 0, 0, &iuri);
+    if(FAILED(hres))
+        return hres;
+
     if(orig_channel->uri->window_ref)
         window = orig_channel->uri->window_ref->window;
-    nsres = create_uri(NULL, window, NULL, &uri);
+    nsres = create_nsuri(iuri, NULL, window, NULL, &uri);
+    IUri_Release(iuri);
     if(NS_FAILED(nsres))
         return E_FAIL;
 
-    hres = CreateUri(url, 0, 0, &uri->uri);
-    if(SUCCEEDED(hres))
-        nsres = create_nschannel(uri, &channel);
-    sync_wine_url(uri);
+    nsres = create_nschannel(uri, &channel);
     nsIURL_Release(&uri->nsIURL_iface);
-    if(FAILED(hres))
-        return hres;
     if(NS_FAILED(nsres))
         return E_FAIL;
 
@@ -2897,16 +2866,14 @@ static nsresult NSAPI nsIOService_NewURI(nsIIOService *iface, const nsACString *
         return nsres;
     }
 
-    nsres = create_uri(uri, window, NULL, &wine_uri);
+    nsres = create_nsuri(urlmon_uri, uri, window, NULL, &wine_uri);
+    IUri_Release(urlmon_uri);
     if(base_wine_uri)
         nsIURI_Release(&base_wine_uri->nsIURL_iface);
     if(NS_FAILED(nsres))
         return nsres;
 
-    wine_uri->uri = urlmon_uri;
-
-    sync_wine_url(wine_uri);
-    *_retval = (nsIURI*)wine_uri;
+    *_retval = (nsIURI*)&wine_uri->nsIURL_iface;
     return nsres;
 }
 
