@@ -549,17 +549,23 @@ static void set_target_path( MSIPACKAGE *package, MSIFOLDER *folder, const WCHAR
 {
     FolderList *fl;
     MSIFOLDER *child;
+    WCHAR *target_path;
 
-    msi_free( folder->ResolvedTarget );
-    folder->ResolvedTarget = strdupW( path );
-    msi_clean_path( folder->ResolvedTarget );
-    msi_set_property( package->db, folder->Directory, folder->ResolvedTarget );
-
-    LIST_FOR_EACH_ENTRY( fl, &folder->children, FolderList, entry )
+    if (!(target_path = strdupW( path ))) return;
+    msi_clean_path( target_path );
+    if (strcmpW( target_path, folder->ResolvedTarget ))
     {
-        child = fl->folder;
-        msi_resolve_target_folder( package, child->Directory, FALSE );
+        msi_free( folder->ResolvedTarget );
+        folder->ResolvedTarget = target_path;
+        msi_set_property( package->db, folder->Directory, folder->ResolvedTarget );
+
+        LIST_FOR_EACH_ENTRY( fl, &folder->children, FolderList, entry )
+        {
+            child = fl->folder;
+            msi_resolve_target_folder( package, child->Directory, FALSE );
+        }
     }
+    else msi_free( target_path );
 }
 
 UINT MSI_SetTargetPathW( MSIPACKAGE *package, LPCWSTR szFolder, LPCWSTR szFolderPath )
@@ -726,7 +732,10 @@ BOOL WINAPI MsiGetMode(MSIHANDLE hInstall, MSIRUNMODE iRunMode)
     {
     case MSIRUNMODE_ADMIN:
         FIXME("no support for administrative installs\n");
-        r = FALSE;
+        break;
+
+    case MSIRUNMODE_ADVERTISE:
+        FIXME("no support for advertised installs\n");
         break;
 
     case MSIRUNMODE_WINDOWS9X:
@@ -756,6 +765,10 @@ BOOL WINAPI MsiGetMode(MSIHANDLE hInstall, MSIRUNMODE iRunMode)
         r = msi_get_property_int( package->db, szInstalled, 0 ) != 0;
         break;
 
+    case MSIRUNMODE_ROLLBACKENABLED:
+        r = msi_get_property_int( package->db, szRollbackDisabled, 0 ) == 0;
+        break;
+
     case MSIRUNMODE_REBOOTATEND:
         r = package->need_reboot;
         break;
@@ -1014,6 +1027,70 @@ UINT WINAPI MsiSetFeatureStateW(MSIHANDLE hInstall, LPCWSTR szFeature,
     return rc;
 }
 
+/***********************************************************************
+* MsiSetFeatureAttributesA   (MSI.@)
+*/
+UINT WINAPI MsiSetFeatureAttributesA( MSIHANDLE handle, LPCSTR feature, DWORD attrs )
+{
+    UINT r;
+    WCHAR *featureW = NULL;
+
+    TRACE("%u, %s, 0x%08x\n", handle, debugstr_a(feature), attrs);
+
+    if (feature && !(featureW = strdupAtoW( feature ))) return ERROR_OUTOFMEMORY;
+
+    r = MsiSetFeatureAttributesW( handle, featureW, attrs );
+    msi_free( featureW );
+    return r;
+}
+
+static DWORD unmap_feature_attributes( DWORD attrs )
+{
+    DWORD ret = 0;
+
+    if (attrs & INSTALLFEATUREATTRIBUTE_FAVORLOCAL)             ret = msidbFeatureAttributesFavorLocal;
+    if (attrs & INSTALLFEATUREATTRIBUTE_FAVORSOURCE)            ret |= msidbFeatureAttributesFavorSource;
+    if (attrs & INSTALLFEATUREATTRIBUTE_FOLLOWPARENT)           ret |= msidbFeatureAttributesFollowParent;
+    if (attrs & INSTALLFEATUREATTRIBUTE_FAVORADVERTISE)         ret |= msidbFeatureAttributesFavorAdvertise;
+    if (attrs & INSTALLFEATUREATTRIBUTE_DISALLOWADVERTISE)      ret |= msidbFeatureAttributesDisallowAdvertise;
+    if (attrs & INSTALLFEATUREATTRIBUTE_NOUNSUPPORTEDADVERTISE) ret |= msidbFeatureAttributesNoUnsupportedAdvertise;
+    return ret;
+}
+
+/***********************************************************************
+* MsiSetFeatureAttributesW   (MSI.@)
+*/
+UINT WINAPI MsiSetFeatureAttributesW( MSIHANDLE handle, LPCWSTR name, DWORD attrs )
+{
+    MSIPACKAGE *package;
+    MSIFEATURE *feature;
+    WCHAR *costing;
+
+    TRACE("%u, %s, 0x%08x\n", handle, debugstr_w(name), attrs);
+
+    if (!name || !name[0]) return ERROR_UNKNOWN_FEATURE;
+
+    if (!(package = msihandle2msiinfo( handle, MSIHANDLETYPE_PACKAGE )))
+        return ERROR_INVALID_HANDLE;
+
+    costing = msi_dup_property( package->db, szCostingComplete );
+    if (!costing || !strcmpW( costing, szOne ))
+    {
+        msi_free( costing );
+        msiobj_release( &package->hdr );
+        return ERROR_FUNCTION_FAILED;
+    }
+    msi_free( costing );
+    if (!(feature = msi_get_loaded_feature( package, name )))
+    {
+        msiobj_release( &package->hdr );
+        return ERROR_UNKNOWN_FEATURE;
+    }
+    feature->Attributes = unmap_feature_attributes( attrs );
+    msiobj_release( &package->hdr );
+    return ERROR_SUCCESS;
+}
+
 /***********************************************************************
 * MsiGetFeatureStateA   (MSI.@)
 */
@@ -1023,12 +1100,10 @@ UINT WINAPI MsiGetFeatureStateA(MSIHANDLE hInstall, LPCSTR szFeature,
     LPWSTR szwFeature = NULL;
     UINT rc;
     
-    szwFeature = strdupAtoW(szFeature);
-
-    rc = MsiGetFeatureStateW(hInstall,szwFeature,piInstalled, piAction);
+    if (szFeature && !(szwFeature = strdupAtoW(szFeature))) return ERROR_OUTOFMEMORY;
 
+    rc = MsiGetFeatureStateW(hInstall, szwFeature, piInstalled, piAction);
     msi_free( szwFeature);
-
     return rc;
 }
 
@@ -1241,6 +1316,126 @@ UINT WINAPI MsiGetFeatureCostW(MSIHANDLE hInstall, LPCWSTR szFeature,
     return ret;
 }
 
+/***********************************************************************
+* MsiGetFeatureInfoA   (MSI.@)
+*/
+UINT WINAPI MsiGetFeatureInfoA( MSIHANDLE handle, LPCSTR feature, LPDWORD attrs,
+                                LPSTR title, LPDWORD title_len, LPSTR help, LPDWORD help_len )
+{
+    UINT r;
+    WCHAR *titleW = NULL, *helpW = NULL, *featureW = NULL;
+
+    TRACE("%u, %s, %p, %p, %p, %p, %p\n", handle, debugstr_a(feature), attrs, title,
+          title_len, help, help_len);
+
+    if (feature && !(featureW = strdupAtoW( feature ))) return ERROR_OUTOFMEMORY;
+
+    if (title && title_len && !(titleW = msi_alloc( *title_len * sizeof(WCHAR) )))
+    {
+        msi_free( featureW );
+        return ERROR_OUTOFMEMORY;
+    }
+    if (help && help_len && !(helpW = msi_alloc( *help_len * sizeof(WCHAR) )))
+    {
+        msi_free( featureW );
+        msi_free( titleW );
+        return ERROR_OUTOFMEMORY;
+    }
+    r = MsiGetFeatureInfoW( handle, featureW, attrs, titleW, title_len, helpW, help_len );
+    if (r == ERROR_SUCCESS)
+    {
+        if (titleW) WideCharToMultiByte( CP_ACP, 0, titleW, -1, title, *title_len + 1, NULL, NULL );
+        if (helpW) WideCharToMultiByte( CP_ACP, 0, helpW, -1, help, *help_len + 1, NULL, NULL );
+    }
+    msi_free( titleW );
+    msi_free( helpW );
+    msi_free( featureW );
+    return r;
+}
+
+static DWORD map_feature_attributes( DWORD attrs )
+{
+    DWORD ret = 0;
+
+    if (attrs == msidbFeatureAttributesFavorLocal)            ret |= INSTALLFEATUREATTRIBUTE_FAVORLOCAL;
+    if (attrs & msidbFeatureAttributesFavorSource)            ret |= INSTALLFEATUREATTRIBUTE_FAVORSOURCE;
+    if (attrs & msidbFeatureAttributesFollowParent)           ret |= INSTALLFEATUREATTRIBUTE_FOLLOWPARENT;
+    if (attrs & msidbFeatureAttributesFavorAdvertise)         ret |= INSTALLFEATUREATTRIBUTE_FAVORADVERTISE;
+    if (attrs & msidbFeatureAttributesDisallowAdvertise)      ret |= INSTALLFEATUREATTRIBUTE_DISALLOWADVERTISE;
+    if (attrs & msidbFeatureAttributesNoUnsupportedAdvertise) ret |= INSTALLFEATUREATTRIBUTE_NOUNSUPPORTEDADVERTISE;
+    return ret;
+}
+
+static UINT MSI_GetFeatureInfo( MSIPACKAGE *package, LPCWSTR name, LPDWORD attrs,
+                                LPWSTR title, LPDWORD title_len, LPWSTR help, LPDWORD help_len )
+{
+    UINT r = ERROR_SUCCESS;
+    MSIFEATURE *feature = msi_get_loaded_feature( package, name );
+    int len;
+
+    if (!feature) return ERROR_UNKNOWN_FEATURE;
+    if (attrs) *attrs = map_feature_attributes( feature->Attributes );
+    if (title_len)
+    {
+        if (feature->Title) len = strlenW( feature->Title );
+        else len = 0;
+        if (*title_len <= len)
+        {
+            *title_len = len;
+            if (title) r = ERROR_MORE_DATA;
+        }
+        else if (title)
+        {
+            if (feature->Title) strcpyW( title, feature->Title );
+            else *title = 0;
+            *title_len = len;
+        }
+    }
+    if (help_len)
+    {
+        if (feature->Description) len = strlenW( feature->Description );
+        else len = 0;
+        if (*help_len <= len)
+        {
+            *help_len = len;
+            if (help) r = ERROR_MORE_DATA;
+        }
+        else if (help)
+        {
+            if (feature->Description) strcpyW( help, feature->Description );
+            else *help = 0;
+            *help_len = len;
+        }
+    }
+    return r;
+}
+
+/***********************************************************************
+* MsiGetFeatureInfoW   (MSI.@)
+*/
+UINT WINAPI MsiGetFeatureInfoW( MSIHANDLE handle, LPCWSTR feature, LPDWORD attrs,
+                                LPWSTR title, LPDWORD title_len, LPWSTR help, LPDWORD help_len )
+{
+    UINT r;
+    MSIPACKAGE *package;
+
+    TRACE("%u, %s, %p, %p, %p, %p, %p\n", handle, debugstr_w(feature), attrs, title,
+          title_len, help, help_len);
+
+    if (!feature) return ERROR_INVALID_PARAMETER;
+
+    if (!(package = msihandle2msiinfo( handle, MSIHANDLETYPE_PACKAGE )))
+        return ERROR_INVALID_HANDLE;
+
+    /* features may not have been loaded yet */
+    msi_load_all_components( package );
+    msi_load_all_features( package );
+
+    r = MSI_GetFeatureInfo( package, feature, attrs, title, title_len, help, help_len );
+    msiobj_release( &package->hdr );
+    return r;
+}
+
 /***********************************************************************
  * MsiSetComponentStateA (MSI.@)
  */
