@@ -82,11 +82,6 @@ struct tagMSITABLE
     WCHAR name[1];
 };
 
-static const WCHAR szStringData[] = {
-    '_','S','t','r','i','n','g','D','a','t','a',0 };
-static const WCHAR szStringPool[] = {
-    '_','S','t','r','i','n','g','P','o','o','l',0 };
-
 /* information for default tables */
 static WCHAR szTables[]  = { '_','T','a','b','l','e','s',0 };
 static WCHAR szTable[]  = { 'T','a','b','l','e',0 };
@@ -108,14 +103,6 @@ static const MSICOLUMNINFO _Tables_cols[1] = {
 
 #define MAX_STREAM_NAME 0x1f
 
-static UINT table_get_column_info( MSIDATABASE *db, LPCWSTR name,
-       MSICOLUMNINFO **pcols, UINT *pcount );
-static void table_calc_column_offsets( MSIDATABASE *db, MSICOLUMNINFO *colinfo,
-       DWORD count );
-static UINT get_tablecolumns( MSIDATABASE *db,
-       LPCWSTR szTableName, MSICOLUMNINFO *colinfo, UINT *sz);
-static void msi_free_colinfo( MSICOLUMNINFO *colinfo, UINT count );
-
 static inline UINT bytes_per_column( MSIDATABASE *db, const MSICOLUMNINFO *col, UINT bytes_per_strref )
 {
     if( MSITYPE_IS_BINARY(col->type) )
@@ -386,6 +373,18 @@ end:
     return ret;
 }
 
+static void msi_free_colinfo( MSICOLUMNINFO *colinfo, UINT count )
+{
+    UINT i;
+
+    for (i = 0; i < count; i++)
+    {
+        msi_free( colinfo[i].tablename );
+        msi_free( colinfo[i].colname );
+        msi_free( colinfo[i].hash_table );
+    }
+}
+
 static void free_table( MSITABLE *table )
 {
     UINT i;
@@ -518,6 +517,60 @@ static MSITABLE *find_cached_table( MSIDATABASE *db, LPCWSTR name )
     return NULL;
 }
 
+static void table_calc_column_offsets( MSIDATABASE *db, MSICOLUMNINFO *colinfo, DWORD count )
+{
+    DWORD i;
+
+    for (i = 0; colinfo && i < count; i++)
+    {
+         assert( i + 1 == colinfo[i].number );
+         if (i) colinfo[i].offset = colinfo[i - 1].offset +
+                                    bytes_per_column( db, &colinfo[i - 1], LONG_STR_BYTES );
+         else colinfo[i].offset = 0;
+
+         TRACE("column %d is [%s] with type %08x ofs %d\n",
+               colinfo[i].number, debugstr_w(colinfo[i].colname),
+               colinfo[i].type, colinfo[i].offset);
+    }
+}
+
+static UINT get_defaulttablecolumns( MSIDATABASE *db, LPCWSTR name, MSICOLUMNINFO *colinfo, UINT *sz )
+{
+    const MSICOLUMNINFO *p;
+    DWORD i, n;
+
+    TRACE("%s\n", debugstr_w(name));
+
+    if (!strcmpW( name, szTables ))
+    {
+        p = _Tables_cols;
+        n = 1;
+    }
+    else if (!strcmpW( name, szColumns ))
+    {
+        p = _Columns_cols;
+        n = 4;
+    }
+    else return ERROR_FUNCTION_FAILED;
+
+    /* duplicate the string data so we can free it in msi_free_colinfo */
+    for (i = 0; i < n; i++)
+    {
+        if (colinfo && i < *sz)
+        {
+            colinfo[i] = p[i];
+            colinfo[i].tablename = strdupW( p[i].tablename );
+            colinfo[i].colname = strdupW( p[i].colname );
+        }
+        if (colinfo && i >= *sz) break;
+    }
+    table_calc_column_offsets( db, colinfo, n );
+    *sz = n;
+    return ERROR_SUCCESS;
+}
+
+static UINT get_tablecolumns( MSIDATABASE *db, LPCWSTR szTableName, MSICOLUMNINFO *colinfo, UINT *sz );
+
 static UINT table_get_column_info( MSIDATABASE *db, LPCWSTR name, MSICOLUMNINFO **pcols, UINT *pcount )
 {
     UINT r, column_count = 0;
@@ -526,33 +579,169 @@ static UINT table_get_column_info( MSIDATABASE *db, LPCWSTR name, MSICOLUMNINFO
     /* get the number of columns in this table */
     column_count = 0;
     r = get_tablecolumns( db, name, NULL, &column_count );
-    if( r != ERROR_SUCCESS )
+    if (r != ERROR_SUCCESS)
         return r;
 
     *pcount = column_count;
 
     /* if there's no columns, there's no table */
-    if( column_count == 0 )
+    if (!column_count)
         return ERROR_INVALID_PARAMETER;
 
-    TRACE("Table %s found\n", debugstr_w(name) );
+    TRACE("table %s found\n", debugstr_w(name));
 
-    columns = msi_alloc( column_count*sizeof (MSICOLUMNINFO) );
-    if( !columns )
+    columns = msi_alloc( column_count * sizeof(MSICOLUMNINFO) );
+    if (!columns)
         return ERROR_FUNCTION_FAILED;
 
     r = get_tablecolumns( db, name, columns, &column_count );
-    if( r != ERROR_SUCCESS )
+    if (r != ERROR_SUCCESS)
     {
         msi_free( columns );
         return ERROR_FUNCTION_FAILED;
     }
-
     *pcols = columns;
-
     return r;
 }
 
+static UINT get_table( MSIDATABASE *db, LPCWSTR name, MSITABLE **table_ret )
+{
+    MSITABLE *table;
+    UINT r;
+
+    /* first, see if the table is cached */
+    table = find_cached_table( db, name );
+    if (table)
+    {
+        *table_ret = table;
+        return ERROR_SUCCESS;
+    }
+
+    /* nonexistent tables should be interpreted as empty tables */
+    table = msi_alloc( sizeof(MSITABLE) + lstrlenW( name ) * sizeof(WCHAR) );
+    if (!table)
+        return ERROR_FUNCTION_FAILED;
+
+    table->row_count = 0;
+    table->data = NULL;
+    table->data_persistent = NULL;
+    table->colinfo = NULL;
+    table->col_count = 0;
+    table->persistent = MSICONDITION_TRUE;
+    lstrcpyW( table->name, name );
+
+    if (!strcmpW( name, szTables ) || !strcmpW( name, szColumns ))
+        table->persistent = MSICONDITION_NONE;
+
+    r = table_get_column_info( db, name, &table->colinfo, &table->col_count );
+    if (r != ERROR_SUCCESS)
+    {
+        free_table( table );
+        return r;
+    }
+    r = read_table_from_storage( db, table, db->storage );
+    if (r != ERROR_SUCCESS)
+    {
+        free_table( table );
+        return r;
+    }
+    list_add_head( &db->tables, &table->entry );
+    *table_ret = table;
+    return ERROR_SUCCESS;
+}
+
+static UINT read_table_int( BYTE *const *data, UINT row, UINT col, UINT bytes )
+{
+    UINT ret = 0, i;
+
+    for (i = 0; i < bytes; i++)
+        ret += data[row][col + i] << i * 8;
+
+    return ret;
+}
+
+static LPWSTR msi_makestring( const MSIDATABASE *db, UINT string_id )
+{
+    return strdupW( msi_string_lookup_id( db->strings, string_id ) );
+}
+
+static UINT get_tablecolumns( MSIDATABASE *db, LPCWSTR szTableName, MSICOLUMNINFO *colinfo, UINT *sz )
+{
+    UINT r, i, n = 0, table_id, count, maxcount = *sz;
+    MSITABLE *table = NULL;
+
+    TRACE("%s\n", debugstr_w(szTableName));
+
+    /* first check if there is a default table with that name */
+    r = get_defaulttablecolumns( db, szTableName, colinfo, sz );
+    if (r == ERROR_SUCCESS && *sz)
+        return r;
+
+    r = get_table( db, szColumns, &table );
+    if (r != ERROR_SUCCESS)
+    {
+        ERR("couldn't load _Columns table\n");
+        return ERROR_FUNCTION_FAILED;
+    }
+
+    /* convert table and column names to IDs from the string table */
+    r = msi_string2idW( db->strings, szTableName, &table_id );
+    if (r != ERROR_SUCCESS)
+    {
+        WARN("Couldn't find id for %s\n", debugstr_w(szTableName));
+        return r;
+    }
+    TRACE("Table id is %d, row count is %d\n", table_id, table->row_count);
+
+    /* Note: _Columns table doesn't have non-persistent data */
+
+    /* if maxcount is non-zero, assume it's exactly right for this table */
+    memset( colinfo, 0, maxcount * sizeof(*colinfo) );
+    count = table->row_count;
+    for (i = 0; i < count; i++)
+    {
+        if (read_table_int( table->data, i, 0, LONG_STR_BYTES) != table_id) continue;
+        if (colinfo)
+        {
+            UINT id = read_table_int( table->data, i, table->colinfo[2].offset, LONG_STR_BYTES );
+            UINT col = read_table_int( table->data, i, table->colinfo[1].offset, sizeof(USHORT) ) - (1 << 15);
+
+            /* check the column number is in range */
+            if (col < 1 || col > maxcount)
+            {
+                ERR("column %d out of range\n", col);
+                continue;
+            }
+            /* check if this column was already set */
+            if (colinfo[col - 1].number)
+            {
+                ERR("duplicate column %d\n", col);
+                continue;
+            }
+            colinfo[col - 1].tablename = msi_makestring( db, table_id );
+            colinfo[col - 1].number = col;
+            colinfo[col - 1].colname = msi_makestring( db, id );
+            colinfo[col - 1].type = read_table_int( table->data, i, table->colinfo[3].offset,
+                                                    sizeof(USHORT) ) - (1 << 15);
+            colinfo[col - 1].offset = 0;
+            colinfo[col - 1].ref_count = 0;
+            colinfo[col - 1].hash_table = NULL;
+        }
+        n++;
+    }
+    TRACE("%s has %d columns\n", debugstr_w(szTableName), n);
+
+    if (colinfo && n != maxcount)
+    {
+        ERR("missing column in table %s\n", debugstr_w(szTableName));
+        msi_free_colinfo( colinfo, maxcount );
+        return ERROR_FUNCTION_FAILED;
+    }
+    table_calc_column_offsets( db, colinfo, n );
+    *sz = n;
+    return ERROR_SUCCESS;
+}
+
 UINT msi_create_table( MSIDATABASE *db, LPCWSTR name, column_info *col_info,
                        MSICONDITION persistent, MSITABLE **table_ret)
 {
@@ -705,64 +894,6 @@ err:
     return r;
 }
 
-static UINT get_table( MSIDATABASE *db, LPCWSTR name, MSITABLE **table_ret )
-{
-    MSITABLE *table;
-    UINT r;
-
-    /* first, see if the table is cached */
-    table = find_cached_table( db, name );
-    if( table )
-    {
-        *table_ret = table;
-        return ERROR_SUCCESS;
-    }
-
-    /* nonexistent tables should be interpreted as empty tables */
-    table = msi_alloc( sizeof (MSITABLE) + lstrlenW(name)*sizeof (WCHAR) );
-    if( !table )
-        return ERROR_FUNCTION_FAILED;
-
-    table->row_count = 0;
-    table->data = NULL;
-    table->data_persistent = NULL;
-    table->colinfo = NULL;
-    table->col_count = 0;
-    table->persistent = MSICONDITION_TRUE;
-    lstrcpyW( table->name, name );
-
-    if ( !strcmpW( name, szTables ) || !strcmpW( name, szColumns ) )
-        table->persistent = MSICONDITION_NONE;
-
-    r = table_get_column_info( db, name, &table->colinfo, &table->col_count);
-    if (r != ERROR_SUCCESS)
-    {
-        free_table ( table );
-        return r;
-    }
-
-    r = read_table_from_storage( db, table, db->storage );
-    if( r != ERROR_SUCCESS )
-    {
-        free_table( table );
-        return r;
-    }
-
-    list_add_head( &db->tables, &table->entry );
-    *table_ret = table;
-    return ERROR_SUCCESS;
-}
-
-static UINT read_table_int(BYTE *const *data, UINT row, UINT col, UINT bytes)
-{
-    UINT ret = 0, i;
-
-    for (i = 0; i < bytes; i++)
-        ret += data[row][col + i] << i * 8;
-
-    return ret;
-}
-
 static UINT save_table( MSIDATABASE *db, const MSITABLE *t, UINT bytes_per_strref )
 {
     BYTE *rawdata = NULL;
@@ -838,165 +969,6 @@ err:
     return r;
 }
 
-static void table_calc_column_offsets( MSIDATABASE *db, MSICOLUMNINFO *colinfo, DWORD count )
-{
-    DWORD i;
-
-    for( i=0; colinfo && (i<count); i++ )
-    {
-         assert( (i+1) == colinfo[ i ].number );
-         if (i)
-             colinfo[i].offset = colinfo[ i - 1 ].offset
-                               + bytes_per_column( db, &colinfo[ i - 1 ], LONG_STR_BYTES );
-         else
-             colinfo[i].offset = 0;
-         TRACE("column %d is [%s] with type %08x ofs %d\n",
-               colinfo[i].number, debugstr_w(colinfo[i].colname),
-               colinfo[i].type, colinfo[i].offset);
-    }
-}
-
-static UINT get_defaulttablecolumns( MSIDATABASE *db, LPCWSTR name,
-                                     MSICOLUMNINFO *colinfo, UINT *sz)
-{
-    const MSICOLUMNINFO *p;
-    DWORD i, n;
-
-    TRACE("%s\n", debugstr_w(name));
-
-    if (!strcmpW( name, szTables ))
-    {
-        p = _Tables_cols;
-        n = 1;
-    }
-    else if (!strcmpW( name, szColumns ))
-    {
-        p = _Columns_cols;
-        n = 4;
-    }
-    else
-        return ERROR_FUNCTION_FAILED;
-
-    /* duplicate the string data so we can free it in msi_free_colinfo */
-    for (i=0; i<n; i++)
-    {
-        if (colinfo && (i < *sz) )
-        {
-            colinfo[i] = p[i];
-            colinfo[i].tablename = strdupW( p[i].tablename );
-            colinfo[i].colname = strdupW( p[i].colname );
-        }
-        if( colinfo && (i >= *sz) )
-            break;
-    }
-    table_calc_column_offsets( db, colinfo, n );
-    *sz = n;
-    return ERROR_SUCCESS;
-}
-
-static void msi_free_colinfo( MSICOLUMNINFO *colinfo, UINT count )
-{
-    UINT i;
-
-    for( i=0; i<count; i++ )
-    {
-        msi_free( colinfo[i].tablename );
-        msi_free( colinfo[i].colname );
-        msi_free( colinfo[i].hash_table );
-    }
-}
-
-static LPWSTR msi_makestring( const MSIDATABASE *db, UINT stringid)
-{
-    return strdupW(msi_string_lookup_id( db->strings, stringid ));
-}
-
-static UINT get_tablecolumns( MSIDATABASE *db,
-       LPCWSTR szTableName, MSICOLUMNINFO *colinfo, UINT *sz)
-{
-    UINT r, i, n=0, table_id, count, maxcount = *sz;
-    MSITABLE *table = NULL;
-
-    TRACE("%s\n", debugstr_w(szTableName));
-
-    /* first check if there is a default table with that name */
-    r = get_defaulttablecolumns( db, szTableName, colinfo, sz );
-    if( ( r == ERROR_SUCCESS ) && *sz )
-        return r;
-
-    r = get_table( db, szColumns, &table );
-    if( r != ERROR_SUCCESS )
-    {
-        ERR("couldn't load _Columns table\n");
-        return ERROR_FUNCTION_FAILED;
-    }
-
-    /* convert table and column names to IDs from the string table */
-    r = msi_string2idW( db->strings, szTableName, &table_id );
-    if( r != ERROR_SUCCESS )
-    {
-        WARN("Couldn't find id for %s\n", debugstr_w(szTableName));
-        return r;
-    }
-
-    TRACE("Table id is %d, row count is %d\n", table_id, table->row_count);
-
-    /* Note: _Columns table doesn't have non-persistent data */
-
-    /* if maxcount is non-zero, assume it's exactly right for this table */
-    memset( colinfo, 0, maxcount*sizeof(*colinfo) );
-    count = table->row_count;
-    for( i=0; i<count; i++ )
-    {
-        if( read_table_int(table->data, i, 0, LONG_STR_BYTES) != table_id )
-            continue;
-        if( colinfo )
-        {
-            UINT id = read_table_int(table->data, i, table->colinfo[2].offset, LONG_STR_BYTES);
-            UINT col = read_table_int(table->data, i, table->colinfo[1].offset, sizeof(USHORT)) - (1<<15);
-
-            /* check the column number is in range */
-            if (col<1 || col>maxcount)
-            {
-                ERR("column %d out of range\n", col);
-                continue;
-            }
-
-            /* check if this column was already set */
-            if (colinfo[ col - 1 ].number)
-            {
-                ERR("duplicate column %d\n", col);
-                continue;
-            }
-
-            colinfo[ col - 1 ].tablename = msi_makestring( db, table_id );
-            colinfo[ col - 1 ].number = col;
-            colinfo[ col - 1 ].colname = msi_makestring( db, id );
-            colinfo[ col - 1 ].type = read_table_int(table->data, i,
-                                                     table->colinfo[3].offset,
-                                                     sizeof(USHORT)) - (1<<15);
-            colinfo[ col - 1 ].offset = 0;
-            colinfo[ col - 1 ].ref_count = 0;
-            colinfo[ col - 1 ].hash_table = NULL;
-        }
-        n++;
-    }
-
-    TRACE("%s has %d columns\n", debugstr_w(szTableName), n);
-
-    if (colinfo && n != maxcount)
-    {
-        ERR("missing column in table %s\n", debugstr_w(szTableName));
-        msi_free_colinfo(colinfo, maxcount );
-        return ERROR_FUNCTION_FAILED;
-    }
-
-    table_calc_column_offsets( db, colinfo, n );
-    *sz = n;
-
-    return ERROR_SUCCESS;
-}
-
 static void msi_update_table_columns( MSIDATABASE *db, LPCWSTR name )
 {
     MSITABLE *table;
@@ -1226,7 +1198,7 @@ static UINT TABLE_fetch_stream( struct tagMSIVIEW *view, UINT row, UINT col, ISt
     }
 
     encname = encode_streamname( FALSE, full_name );
-    r = db_get_raw_stream( tv->db, encname, stm );
+    r = msi_get_raw_stream( tv->db, encname, stm );
     if( r )
         ERR("fetching stream %s, error = %d\n",debugstr_w(full_name), r);
 
@@ -1584,9 +1556,9 @@ static UINT TABLE_get_column_info( struct tagMSIVIEW *view,
     return ERROR_SUCCESS;
 }
 
-static UINT msi_table_find_row( MSITABLEVIEW *tv, MSIRECORD *rec, UINT *row );
+static UINT msi_table_find_row( MSITABLEVIEW *tv, MSIRECORD *rec, UINT *row, UINT *column );
 
-static UINT table_validate_new( MSITABLEVIEW *tv, MSIRECORD *rec )
+static UINT table_validate_new( MSITABLEVIEW *tv, MSIRECORD *rec, UINT *column )
 {
     UINT r, row, i;
 
@@ -1604,7 +1576,10 @@ static UINT table_validate_new( MSITABLEVIEW *tv, MSIRECORD *rec )
 
             str = MSI_RecordGetString( rec, i+1 );
             if (str == NULL || str[0] == 0)
+            {
+                if (column) *column = i;
                 return ERROR_INVALID_DATA;
+            }
         }
         else
         {
@@ -1612,12 +1587,15 @@ static UINT table_validate_new( MSITABLEVIEW *tv, MSIRECORD *rec )
 
             n = MSI_RecordGetInteger( rec, i+1 );
             if (n == MSI_NULL_INTEGER)
+            {
+                if (column) *column = i;
                 return ERROR_INVALID_DATA;
+            }
         }
     }
 
     /* check there's no duplicate keys */
-    r = msi_table_find_row( tv, rec, &row );
+    r = msi_table_find_row( tv, rec, &row, column );
     if (r == ERROR_SUCCESS)
         return ERROR_FUNCTION_FAILED;
 
@@ -1690,7 +1668,7 @@ static UINT TABLE_insert_row( struct tagMSIVIEW *view, MSIRECORD *rec, UINT row,
     TRACE("%p %p %s\n", tv, rec, temporary ? "TRUE" : "FALSE" );
 
     /* check that the key is unique - can we find a matching row? */
-    r = table_validate_new( tv, rec );
+    r = table_validate_new( tv, rec, NULL );
     if( r != ERROR_SUCCESS )
         return ERROR_FUNCTION_FAILED;
 
@@ -1765,7 +1743,7 @@ static UINT msi_table_update(struct tagMSIVIEW *view, MSIRECORD *rec, UINT row)
     if (!tv->table)
         return ERROR_INVALID_PARAMETER;
 
-    r = msi_table_find_row(tv, rec, &new_row);
+    r = msi_table_find_row(tv, rec, &new_row, NULL);
     if (r != ERROR_SUCCESS)
     {
         ERR("can't find row to modify\n");
@@ -1790,7 +1768,7 @@ static UINT msi_table_assign(struct tagMSIVIEW *view, MSIRECORD *rec)
     if (!tv->table)
         return ERROR_INVALID_PARAMETER;
 
-    r = msi_table_find_row(tv, rec, &row);
+    r = msi_table_find_row(tv, rec, &row, NULL);
     if (r == ERROR_SUCCESS)
         return TABLE_set_row(view, row, rec, (1 << tv->num_cols) - 1);
     else
@@ -1802,7 +1780,7 @@ static UINT modify_delete_row( struct tagMSIVIEW *view, MSIRECORD *rec )
     MSITABLEVIEW *tv = (MSITABLEVIEW *)view;
     UINT row, r;
 
-    r = msi_table_find_row(tv, rec, &row);
+    r = msi_table_find_row(tv, rec, &row, NULL);
     if (r != ERROR_SUCCESS)
         return r;
 
@@ -1833,7 +1811,7 @@ static UINT TABLE_modify( struct tagMSIVIEW *view, MSIMODIFY eModifyMode,
                           MSIRECORD *rec, UINT row)
 {
     MSITABLEVIEW *tv = (MSITABLEVIEW*)view;
-    UINT r;
+    UINT r, column;
 
     TRACE("%p %d %p\n", view, eModifyMode, rec );
 
@@ -1843,18 +1821,24 @@ static UINT TABLE_modify( struct tagMSIVIEW *view, MSIMODIFY eModifyMode,
         r = modify_delete_row( view, rec );
         break;
     case MSIMODIFY_VALIDATE_NEW:
-        r = table_validate_new( tv, rec );
+        r = table_validate_new( tv, rec, &column );
+        if (r != ERROR_SUCCESS)
+        {
+            tv->view.error = MSIDBERROR_DUPLICATEKEY;
+            tv->view.error_column = tv->columns[column].colname;
+            r = ERROR_INVALID_DATA;
+        }
         break;
 
     case MSIMODIFY_INSERT:
-        r = table_validate_new( tv, rec );
+        r = table_validate_new( tv, rec, NULL );
         if (r != ERROR_SUCCESS)
             break;
         r = TABLE_insert_row( view, rec, -1, FALSE );
         break;
 
     case MSIMODIFY_INSERT_TEMPORARY:
-        r = table_validate_new( tv, rec );
+        r = table_validate_new( tv, rec, NULL );
         if (r != ERROR_SUCCESS)
             break;
         r = TABLE_insert_row( view, rec, -1, TRUE );
@@ -2022,7 +2006,7 @@ static UINT TABLE_remove_column(struct tagMSIVIEW *view, LPCWSTR table, UINT num
     if (r != ERROR_SUCCESS)
         return r;
 
-    r = msi_table_find_row((MSITABLEVIEW *)columns, rec, &row);
+    r = msi_table_find_row((MSITABLEVIEW *)columns, rec, &row, NULL);
     if (r != ERROR_SUCCESS)
         goto done;
 
@@ -2299,7 +2283,7 @@ static UINT TABLE_drop(struct tagMSIVIEW *view)
     if (r != ERROR_SUCCESS)
         return r;
 
-    r = msi_table_find_row((MSITABLEVIEW *)tables, rec, &row);
+    r = msi_table_find_row((MSITABLEVIEW *)tables, rec, &row, NULL);
     if (r != ERROR_SUCCESS)
         goto done;
 
@@ -2665,7 +2649,7 @@ static UINT* msi_record_to_row( const MSITABLEVIEW *tv, MSIRECORD *rec )
     return data;
 }
 
-static UINT msi_row_matches( MSITABLEVIEW *tv, UINT row, const UINT *data )
+static UINT msi_row_matches( MSITABLEVIEW *tv, UINT row, const UINT *data, UINT *column )
 {
     UINT i, r, x, ret = ERROR_FUNCTION_FAILED;
 
@@ -2688,14 +2672,13 @@ static UINT msi_row_matches( MSITABLEVIEW *tv, UINT row, const UINT *data )
             ret = ERROR_FUNCTION_FAILED;
             break;
         }
-
+        if (column) *column = i;
         ret = ERROR_SUCCESS;
     }
-
     return ret;
 }
 
-static UINT msi_table_find_row( MSITABLEVIEW *tv, MSIRECORD *rec, UINT *row )
+static UINT msi_table_find_row( MSITABLEVIEW *tv, MSIRECORD *rec, UINT *row, UINT *column )
 {
     UINT i, r = ERROR_FUNCTION_FAILED, *data;
 
@@ -2704,7 +2687,7 @@ static UINT msi_table_find_row( MSITABLEVIEW *tv, MSIRECORD *rec, UINT *row )
         return r;
     for( i = 0; i < tv->table->row_count; i++ )
     {
-        r = msi_row_matches( tv, i, data );
+        r = msi_row_matches( tv, i, data, column );
         if( r == ERROR_SUCCESS )
         {
             *row = i;
@@ -2763,7 +2746,6 @@ static UINT msi_table_load_transform( MSIDATABASE *db, IStorage *stg,
           debugstr_w(name), tv->num_cols, tv->row_size, rawsize );
 
     /* interpret the data */
-    r = ERROR_SUCCESS;
     for( n=0; n < rawsize;  )
     {
         mask = rawdata[n] | (rawdata[n+1] << 8);
@@ -2849,7 +2831,7 @@ static UINT msi_table_load_transform( MSIDATABASE *db, IStorage *stg,
 
             if (TRACE_ON(msidb)) dump_record( rec );
 
-            r = msi_table_find_row( tv, rec, &row );
+            r = msi_table_find_row( tv, rec, &row, NULL );
             if (r == ERROR_SUCCESS)
             {
                 if (!mask)
