@@ -20,9 +20,8 @@
 #include "windef.h"
 
 typedef unsigned char MSVCP_bool;
+typedef int MSVCP_long;
 typedef SIZE_T MSVCP_size_t;
-typedef SIZE_T streamoff;
-typedef SIZE_T streamsize;
 
 void __cdecl _invalid_parameter(const wchar_t*, const wchar_t*,
         const wchar_t*, unsigned int, uintptr_t);
@@ -134,7 +133,7 @@ typedef struct _rtti_base_descriptor
 
 typedef struct _rtti_base_array
 {
-    const rtti_base_descriptor *bases[4]; /* First element is the class itself */
+    const rtti_base_descriptor *bases[3]; /* First element is the class itself */
 } rtti_base_array;
 
 typedef struct _rtti_object_hierarchy
@@ -195,31 +194,10 @@ typedef struct
     struct locale__Locimp *ptr;
 } locale;
 
-locale* __thiscall locale_ctor(locale*);
-void __thiscall locale_dtor(locale*);
-
 /* class _Lockit */
 typedef struct {
     int locktype;
 } _Lockit;
 
-#define _LOCK_LOCALE 0
-#define _LOCK_MALLOC 1
-#define _LOCK_STREAM 2
-#define _LOCK_DEBUG 3
-#define _MAX_LOCK 4
-
 void init_lockit(void);
 void free_lockit(void);
-_Lockit* __thiscall _Lockit_ctor_locktype(_Lockit*, int);
-void __thiscall _Lockit_dtor(_Lockit*);
-
-/* class mutex */
-typedef struct {
-        void *mutex;
-} mutex;
-
-mutex* __thiscall mutex_ctor(mutex*);
-void __thiscall mutex_dtor(mutex*);
-void __thiscall mutex_lock(mutex*);
-void __thiscall mutex_unlock(mutex*);
