@@ -40,7 +40,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(msvcrt);
  *		_mbsdup (MSVCRT.@)
  *		_strdup (MSVCRT.@)
  */
-char* CDECL _strdup(const char* str)
+char* CDECL MSVCRT__strdup(const char* str)
 {
     if(str)
     {
@@ -58,9 +58,6 @@ int CDECL _strlwr_s_l(char *str, MSVCRT_size_t len, MSVCRT__locale_t locale)
 {
     char *ptr = str;
 
-    if(!locale)
-        locale = get_locale();
-
     if (!str || !len)
     {
         *MSVCRT__errno() = MSVCRT_EINVAL;
@@ -82,7 +79,7 @@ int CDECL _strlwr_s_l(char *str, MSVCRT_size_t len, MSVCRT__locale_t locale)
 
     while (*str)
     {
-        *str = MSVCRT__tolower_l(*str, locale);
+        *str = MSVCRT__tolower_l((unsigned char)*str, locale);
         str++;
     }
 
@@ -100,17 +97,19 @@ int CDECL _strlwr_s(char *str, MSVCRT_size_t len)
 /*********************************************************************
  *		_strlwr_l (MSVCRT.@)
  */
-int CDECL _strlwr_l(char *str, MSVCRT__locale_t locale)
+char* CDECL _strlwr_l(char *str, MSVCRT__locale_t locale)
 {
-    return _strlwr_s_l(str, -1, locale);
+    _strlwr_s_l(str, -1, locale);
+    return str;
 }
 
 /*********************************************************************
  *		_strlwr (MSVCRT.@)
  */
-int CDECL _strlwr(char *str)
+char* CDECL MSVCRT__strlwr(char *str)
 {
-    return _strlwr_s_l(str, -1, NULL);
+    _strlwr_s_l(str, -1, NULL);
+    return str;
 }
 
 /*********************************************************************
@@ -120,9 +119,6 @@ int CDECL _strupr_s_l(char *str, MSVCRT_size_t len, MSVCRT__locale_t locale)
 {
     char *ptr = str;
 
-    if(!locale)
-        locale = get_locale();
-
     if (!str || !len)
     {
         *MSVCRT__errno() = MSVCRT_EINVAL;
@@ -144,7 +140,7 @@ int CDECL _strupr_s_l(char *str, MSVCRT_size_t len, MSVCRT__locale_t locale)
 
     while (*str)
     {
-        *str = MSVCRT__toupper_l(*str, locale);
+        *str = MSVCRT__toupper_l((unsigned char)*str, locale);
         str++;
     }
 
@@ -162,17 +158,19 @@ int CDECL _strupr_s(char *str, MSVCRT_size_t len)
 /*********************************************************************
  *              _strupr_l (MSVCRT.@)
  */
-int CDECL _strupr_l(char *str, MSVCRT__locale_t locale)
+char* CDECL MSVCRT__strupr_l(char *str, MSVCRT__locale_t locale)
 {
-    return _strupr_s_l(str, -1, locale);
+    _strupr_s_l(str, -1, locale);
+    return str;
 }
 
 /*********************************************************************
  *              _strupr (MSVCRT.@)
  */
-int CDECL _strupr(char *str)
+char* CDECL MSVCRT__strupr(char *str)
 {
-    return _strupr_s_l(str, -1, NULL);
+    _strupr_s_l(str, -1, NULL);
+    return str;
 }
 
 /*********************************************************************
@@ -189,7 +187,7 @@ char* CDECL MSVCRT__strnset(char* str, int value, MSVCRT_size_t len)
 /*********************************************************************
  *		_strrev (MSVCRT.@)
  */
-char* CDECL _strrev(char* str)
+char* CDECL MSVCRT__strrev(char* str)
 {
   char * p1;
   char * p2;
@@ -289,6 +287,7 @@ void CDECL MSVCRT__swab(char* src, char* dst, int len)
  */
 double CDECL MSVCRT_strtod_l( const char *str, char **end, MSVCRT__locale_t locale)
 {
+    MSVCRT_pthreadlocinfo locinfo;
     unsigned __int64 d=0, hlp;
     unsigned fpcontrol;
     int exp=0, sign=1;
@@ -302,7 +301,9 @@ double CDECL MSVCRT_strtod_l( const char *str, char **end, MSVCRT__locale_t loca
     }
 
     if(!locale)
-        locale = get_locale();
+        locinfo = get_locinfo();
+    else
+        locinfo = locale->locinfo;
 
     /* FIXME: use *_l functions */
     p = str;
@@ -329,7 +330,7 @@ double CDECL MSVCRT_strtod_l( const char *str, char **end, MSVCRT__locale_t loca
         p++;
     }
 
-    if(*p == *locale->locinfo->lconv->decimal_point)
+    if(*p == *locinfo->lconv->decimal_point)
         p++;
 
     while(isdigit(*p)) {
@@ -427,6 +428,7 @@ double CDECL MSVCRT__atof_l( const char *str, MSVCRT__locale_t locale)
  */
 int CDECL MSVCRT__atoflt_l( MSVCRT__CRT_FLOAT *value, char *str, MSVCRT__locale_t locale)
 {
+    MSVCRT_pthreadlocinfo locinfo;
     unsigned __int64 d=0, hlp;
     unsigned fpcontrol;
     int exp=0, sign=1;
@@ -435,7 +437,9 @@ int CDECL MSVCRT__atoflt_l( MSVCRT__CRT_FLOAT *value, char *str, MSVCRT__locale_
     BOOL found_digit = FALSE;
 
     if(!locale)
-        locale = get_locale();
+        locinfo = get_locinfo();
+    else
+        locinfo = locale->locinfo;
 
     /* FIXME: use *_l functions */
     p = str;
@@ -462,7 +466,7 @@ int CDECL MSVCRT__atoflt_l( MSVCRT__CRT_FLOAT *value, char *str, MSVCRT__locale_
         p++;
     }
 
-    if(*p == *locale->locinfo->lconv->decimal_point)
+    if(*p == *locinfo->lconv->decimal_point)
         p++;
 
     while(isdigit(*p)) {
@@ -532,10 +536,14 @@ int CDECL MSVCRT__atoflt_l( MSVCRT__CRT_FLOAT *value, char *str, MSVCRT__locale_
  */
 int CDECL MSVCRT_strcoll_l( const char* str1, const char* str2, MSVCRT__locale_t locale )
 {
-    if(!locale)
-        locale = get_locale();
+    MSVCRT_pthreadlocinfo locinfo;
 
-    return CompareStringA(locale->locinfo->lc_handle[MSVCRT_LC_CTYPE], 0, str1, -1, str2, -1)-2;
+    if(!locale)
+        locinfo = get_locinfo();
+    else
+        locinfo = locale->locinfo;
+
+    return CompareStringA(locinfo->lc_handle[MSVCRT_LC_CTYPE], 0, str1, -1, str2, -1)-2;
 }
 
 /*********************************************************************
@@ -551,10 +559,14 @@ int CDECL MSVCRT_strcoll( const char* str1, const char* str2 )
  */
 int CDECL MSVCRT__stricoll_l( const char* str1, const char* str2, MSVCRT__locale_t locale )
 {
-    if(!locale)
-        locale = get_locale();
+    MSVCRT_pthreadlocinfo locinfo;
 
-    return CompareStringA(locale->locinfo->lc_handle[MSVCRT_LC_CTYPE], NORM_IGNORECASE,
+    if(!locale)
+        locinfo = get_locinfo();
+    else
+        locinfo = locale->locinfo;
+
+    return CompareStringA(locinfo->lc_handle[MSVCRT_LC_CTYPE], NORM_IGNORECASE,
             str1, -1, str2, -1)-2;
 }
 
@@ -571,10 +583,14 @@ int CDECL MSVCRT__stricoll( const char* str1, const char* str2 )
  */
 int CDECL MSVCRT_strncoll_l( const char* str1, const char* str2, MSVCRT_size_t count, MSVCRT__locale_t locale )
 {
-    if(!locale)
-        locale = get_locale();
+    MSVCRT_pthreadlocinfo locinfo;
 
-    return CompareStringA(locale->locinfo->lc_handle[MSVCRT_LC_CTYPE], 0, str1, count, str2, count)-2;
+    if(!locale)
+        locinfo = get_locinfo();
+    else
+        locinfo = locale->locinfo;
+
+    return CompareStringA(locinfo->lc_handle[MSVCRT_LC_CTYPE], 0, str1, count, str2, count)-2;
 }
 
 /*********************************************************************
@@ -590,10 +606,14 @@ int CDECL MSVCRT_strncoll( const char* str1, const char* str2, MSVCRT_size_t cou
  */
 int CDECL MSVCRT__strnicoll_l( const char* str1, const char* str2, MSVCRT_size_t count, MSVCRT__locale_t locale )
 {
-    if(!locale)
-        locale = get_locale();
+    MSVCRT_pthreadlocinfo locinfo;
 
-    return CompareStringA(locale->locinfo->lc_handle[MSVCRT_LC_CTYPE], NORM_IGNORECASE,
+    if(!locale)
+        locinfo = get_locinfo();
+    else
+        locinfo = locale->locinfo;
+
+    return CompareStringA(locinfo->lc_handle[MSVCRT_LC_CTYPE], NORM_IGNORECASE,
             str1, count, str2, count)-2;
 }
 
