@@ -41,6 +41,8 @@
 WINE_DEFAULT_DEBUG_CHANNEL(ntoskrnl);
 WINE_DECLARE_DEBUG_CHANNEL(relay);
 
+BOOLEAN KdDebuggerEnabled = FALSE;
+
 extern LONG CALLBACK vectored_handler( EXCEPTION_POINTERS *ptrs );
 
 KSYSTEM_TIME KeTickCount = { 0, 0, 0 };
@@ -1414,6 +1416,21 @@ NTSTATUS WINAPI ObReferenceObjectByHandle( HANDLE obj, ACCESS_MASK access,
     return STATUS_NOT_IMPLEMENTED;
 }
 
+ /***********************************************************************
+ *           ObReferenceObjectByName    (NTOSKRNL.EXE.@)
+ */
+NTSTATUS WINAPI ObReferenceObjectByName( UNICODE_STRING *ObjectName,
+                                         ULONG Attributes,
+                                         ACCESS_STATE *AccessState,
+                                         ACCESS_MASK DesiredAccess,
+                                         POBJECT_TYPE ObjectType,
+                                         KPROCESSOR_MODE AccessMode,
+                                         void *ParseContext,
+                                         void **Object)
+{
+    FIXME("stub\n");
+    return STATUS_NOT_IMPLEMENTED;
+}
 
 /***********************************************************************
  *           ObfDereferenceObject   (NTOSKRNL.EXE.@)
@@ -1613,6 +1630,16 @@ NTSTATUS WINAPI PsLookupProcessByProcessId(HANDLE processid, PEPROCESS *process)
     return STATUS_NOT_IMPLEMENTED;
 }
 
+
+/*****************************************************
+ *           IoSetThreadHardErrorMode  (NTOSKRNL.EXE.@)
+ */
+BOOLEAN WINAPI IoSetThreadHardErrorMode(BOOLEAN EnableHardErrors)
+{
+    FIXME("stub\n");
+    return FALSE;
+}
+
 /*****************************************************
  *           DllMain
  */
