@@ -155,7 +155,7 @@ static inline IShellViewImpl *impl_from_IShellFolderView(IShellFolderView *iface
     return CONTAINING_RECORD(iface, IShellViewImpl, IShellFolderView_iface);
 }
 
-/* ListView Header IDs */
+/* ListView Header ID's */
 #define LISTVIEW_COLUMN_NAME 0
 #define LISTVIEW_COLUMN_SIZE 1
 #define LISTVIEW_COLUMN_TYPE 2
@@ -728,28 +728,13 @@ static LRESULT ShellView_OnCreate(IShellViewImpl *This)
     hr = IShellFolder_QueryInterface(This->pSFParent, &IID_IPersistFolder2, (LPVOID*)&ppf2);
     if (hr == S_OK)
     {
-        LPITEMIDLIST raw_pidl;
         SHChangeNotifyEntry ntreg;
 
-        hr = IPersistFolder2_GetCurFolder(ppf2, (LPITEMIDLIST*)&raw_pidl);
-        if(SUCCEEDED(hr))
+        hr = IPersistFolder2_GetCurFolder(ppf2, (LPITEMIDLIST*)&ntreg.pidl);
+        if (hr == S_OK)
         {
-            LPITEMIDLIST computer_pidl;
-            SHGetFolderLocation(NULL,CSIDL_DRIVES,NULL,0,&computer_pidl);
-            if(ILIsParent(computer_pidl,raw_pidl,FALSE))
-            {
-                /* Normalize the pidl to unixfs to workaround an issue with
-                 * sending notifications on dos paths
-                 */
-                WCHAR path[MAX_PATH];
-                SHGetPathFromIDListW(raw_pidl,path);
-                SHParseDisplayName(path,NULL,(LPITEMIDLIST*)&ntreg.pidl,0,NULL);
-                SHFree(raw_pidl);
-            }
-            else
-                ntreg.pidl = raw_pidl;
             ntreg.fRecursive = TRUE;
-            This->hNotify = SHChangeNotifyRegister(This->hWnd, SHCNRF_InterruptLevel, SHCNE_ALLEVENTS,
+            This->hNotify = SHChangeNotifyRegister(This->hWnd, SHCNF_IDLIST, SHCNE_ALLEVENTS,
                                                    SHV_CHANGE_NOTIFY, 1, &ntreg);
             SHFree((LPITEMIDLIST)ntreg.pidl);
         }
@@ -1272,7 +1257,7 @@ static LRESULT ShellView_OnKillFocus(IShellViewImpl * This)
 * ShellView_OnCommand()
 *
 * NOTES
-*	the CmdIDs are the ones from the context menu
+*	the CmdID's are the ones from the context menu
 */
 static LRESULT ShellView_OnCommand(IShellViewImpl * This,DWORD dwCmdID, DWORD dwCmd, HWND hwndCmd)
 {
@@ -1304,7 +1289,7 @@ static LRESULT ShellView_OnCommand(IShellViewImpl * This,DWORD dwCmdID, DWORD dw
 	    CheckToolbar(This);
 	    break;
 
-	  /* the menu IDs for sorting are 0x30... see shell32.rc */
+	  /* the menu-ID's for sorting are 0x30... see shell32.rc */
 	  case 0x30:
 	  case 0x31:
 	  case 0x32:
