@@ -77,11 +77,12 @@ typedef struct _AudioSession {
     GUID guid;
     struct list clients;
 
-    EDataFlow dataflow;
+    IMMDevice *device;
 
     float master_vol;
     UINT32 channel_count;
     float *channel_vols;
+    BOOL mute;
 
     CRITICAL_SECTION lock;
 
@@ -165,7 +166,7 @@ typedef struct _SessionMgr {
 
     LONG ref;
 
-    EDataFlow flow;
+    IMMDevice *device;
 } SessionMgr;
 
 static HANDLE g_timer_q;
@@ -604,6 +605,8 @@ static HRESULT ca_get_audiodesc(AudioStreamBasicDescription *desc,
 {
     const WAVEFORMATEXTENSIBLE *fmtex = (const WAVEFORMATEXTENSIBLE *)fmt;
 
+    desc->mFormatFlags = 0;
+
     if(fmt->wFormatTag == WAVE_FORMAT_PCM ||
             (fmt->wFormatTag == WAVE_FORMAT_EXTENSIBLE &&
              IsEqualGUID(&fmtex->SubFormat, &KSDATAFORMAT_SUBTYPE_PCM))){
@@ -719,18 +722,26 @@ static HRESULT ca_setup_aqueue(AudioDeviceID did, EDataFlow flow,
 
 static void session_init_vols(AudioSession *session, UINT channels)
 {
-    session->channel_vols = HeapAlloc(GetProcessHeap(), 0,
-            sizeof(float) * channels);
-    if(!session->channel_vols)
-        return;
+    if(session->channel_count < channels){
+        UINT i;
 
-    session->channel_count = channels;
+        if(session->channel_vols)
+            session->channel_vols = HeapReAlloc(GetProcessHeap(), 0,
+                    session->channel_vols, sizeof(float) * channels);
+        else
+            session->channel_vols = HeapAlloc(GetProcessHeap(), 0,
+                    sizeof(float) * channels);
+        if(!session->channel_vols)
+            return;
 
-    for(; channels > 0; --channels)
-        session->channel_vols[channels - 1] = 1.f;
+        for(i = session->channel_count; i < channels; ++i)
+            session->channel_vols[i] = 1.f;
+
+        session->channel_count = channels;
+    }
 }
 
-static AudioSession *create_session(const GUID *guid, EDataFlow flow,
+static AudioSession *create_session(const GUID *guid, IMMDevice *device,
         UINT num_channels)
 {
     AudioSession *ret;
@@ -741,7 +752,7 @@ static AudioSession *create_session(const GUID *guid, EDataFlow flow,
 
     memcpy(&ret->guid, guid, sizeof(GUID));
 
-    ret->dataflow = flow;
+    ret->device = device;
 
     list_init(&ret->clients);
 
@@ -749,8 +760,7 @@ static AudioSession *create_session(const GUID *guid, EDataFlow flow,
 
     InitializeCriticalSection(&ret->lock);
 
-    if(num_channels > 0)
-        session_init_vols(ret, num_channels);
+    session_init_vols(ret, num_channels);
 
     ret->master_vol = 1.f;
 
@@ -760,12 +770,12 @@ static AudioSession *create_session(const GUID *guid, EDataFlow flow,
 /* if channels == 0, then this will return or create a session with
  * matching dataflow and GUID. otherwise, channels must also match */
 static HRESULT get_audio_session(const GUID *sessionguid,
-        EDataFlow flow, UINT channels, AudioSession **out)
+        IMMDevice *device, UINT channels, AudioSession **out)
 {
     AudioSession *session;
 
     if(!sessionguid || IsEqualGUID(sessionguid, &GUID_NULL)){
-        *out = create_session(&GUID_NULL, flow, channels);
+        *out = create_session(&GUID_NULL, device, channels);
         if(!*out)
             return E_OUTOFMEMORY;
 
@@ -774,20 +784,16 @@ static HRESULT get_audio_session(const GUID *sessionguid,
 
     *out = NULL;
     LIST_FOR_EACH_ENTRY(session, &g_sessions, AudioSession, entry){
-        if(IsEqualGUID(sessionguid, &session->guid) &&
-                session->dataflow == flow){
-            if(session->channel_count > 0){
-                if(channels > 0 && session->channel_count != channels)
-                    return E_INVALIDARG;
-            }else if(channels > 0)
-                session_init_vols(session, channels);
+        if(session->device == device &&
+                IsEqualGUID(sessionguid, &session->guid)){
+            session_init_vols(session, channels);
             *out = session;
             break;
         }
     }
 
     if(!*out){
-        *out = create_session(sessionguid, flow, channels);
+        *out = create_session(sessionguid, device, channels);
         if(!*out)
             return E_OUTOFMEMORY;
     }
@@ -914,7 +920,7 @@ static HRESULT WINAPI AudioClient_Initialize(IAudioClient *iface,
 
     EnterCriticalSection(&g_sessions_lock);
 
-    hr = get_audio_session(sessionguid, This->dataflow, fmt->nChannels,
+    hr = get_audio_session(sessionguid, This->parent, fmt->nChannels,
             &This->session);
     if(FAILED(hr)){
         LeaveCriticalSection(&g_sessions_lock);
@@ -1454,16 +1460,20 @@ static HRESULT WINAPI AudioClient_GetService(IAudioClient *iface, REFIID riid,
             OSSpinLockUnlock(&This->lock);
             return AUDCLNT_E_WRONG_ENDPOINT_TYPE;
         }
+        IAudioRenderClient_AddRef(&This->IAudioRenderClient_iface);
         *ppv = &This->IAudioRenderClient_iface;
     }else if(IsEqualIID(riid, &IID_IAudioCaptureClient)){
         if(This->dataflow != eCapture){
             OSSpinLockUnlock(&This->lock);
             return AUDCLNT_E_WRONG_ENDPOINT_TYPE;
         }
+        IAudioCaptureClient_AddRef(&This->IAudioCaptureClient_iface);
         *ppv = &This->IAudioCaptureClient_iface;
     }else if(IsEqualIID(riid, &IID_IAudioClock)){
+        IAudioClock_AddRef(&This->IAudioClock_iface);
         *ppv = &This->IAudioClock_iface;
     }else if(IsEqualIID(riid, &IID_IAudioStreamVolume)){
+        IAudioStreamVolume_AddRef(&This->IAudioStreamVolume_iface);
         *ppv = &This->IAudioStreamVolume_iface;
     }else if(IsEqualIID(riid, &IID_IAudioSessionControl)){
         if(!This->session_wrapper){
@@ -1472,7 +1482,8 @@ static HRESULT WINAPI AudioClient_GetService(IAudioClient *iface, REFIID riid,
                 OSSpinLockUnlock(&This->lock);
                 return E_OUTOFMEMORY;
             }
-        }
+        }else
+            IAudioSessionControl2_AddRef(&This->session_wrapper->IAudioSessionControl2_iface);
 
         *ppv = &This->session_wrapper->IAudioSessionControl2_iface;
     }else if(IsEqualIID(riid, &IID_IChannelAudioVolume)){
@@ -1482,7 +1493,8 @@ static HRESULT WINAPI AudioClient_GetService(IAudioClient *iface, REFIID riid,
                 OSSpinLockUnlock(&This->lock);
                 return E_OUTOFMEMORY;
             }
-        }
+        }else
+            IChannelAudioVolume_AddRef(&This->session_wrapper->IChannelAudioVolume_iface);
 
         *ppv = &This->session_wrapper->IChannelAudioVolume_iface;
     }else if(IsEqualIID(riid, &IID_ISimpleAudioVolume)){
@@ -1492,13 +1504,13 @@ static HRESULT WINAPI AudioClient_GetService(IAudioClient *iface, REFIID riid,
                 OSSpinLockUnlock(&This->lock);
                 return E_OUTOFMEMORY;
             }
-        }
+        }else
+            ISimpleAudioVolume_AddRef(&This->session_wrapper->ISimpleAudioVolume_iface);
 
         *ppv = &This->session_wrapper->ISimpleAudioVolume_iface;
     }
 
     if(*ppv){
-        IUnknown_AddRef((IUnknown*)*ppv);
         OSSpinLockUnlock(&This->lock);
         return S_OK;
     }
@@ -2030,6 +2042,8 @@ static AudioSessionWrapper *AudioSessionWrapper_Create(ACImpl *client)
     ret->ISimpleAudioVolume_iface.lpVtbl = &SimpleAudioVolume_Vtbl;
     ret->IChannelAudioVolume_iface.lpVtbl = &ChannelAudioVolume_Vtbl;
 
+    ret->ref = 1;
+
     ret->client = client;
     if(client){
         ret->session = client->session;
@@ -2077,10 +2091,12 @@ static ULONG WINAPI AudioSessionControl_Release(IAudioSessionControl2 *iface)
     ref = InterlockedDecrement(&This->ref);
     TRACE("(%p) Refcount now %u\n", This, ref);
     if(!ref){
-        OSSpinLockLock(&This->client->lock);
-        This->client->session_wrapper = NULL;
-        OSSpinLockUnlock(&This->client->lock);
-        AudioClient_Release(&This->client->IAudioClient_iface);
+        if(This->client){
+            OSSpinLockLock(&This->client->lock);
+            This->client->session_wrapper = NULL;
+            OSSpinLockUnlock(&This->client->lock);
+            AudioClient_Release(&This->client->IAudioClient_iface);
+        }
         HeapFree(GetProcessHeap(), 0, This);
     }
     return ref;
@@ -2175,7 +2191,7 @@ static HRESULT WINAPI AudioSessionControl_GetGroupingParam(
 }
 
 static HRESULT WINAPI AudioSessionControl_SetGroupingParam(
-        IAudioSessionControl2 *iface, GUID *group, const GUID *session)
+        IAudioSessionControl2 *iface, const GUID *group, const GUID *session)
 {
     AudioSessionWrapper *This = impl_from_IAudioSessionControl2(iface);
 
@@ -2284,7 +2300,6 @@ static const IAudioSessionControl2Vtbl AudioSessionControl2_Vtbl =
 /* index == -1 means set all channels, otherwise sets only the given channel */
 static HRESULT ca_setvol(ACImpl *This, UINT32 index)
 {
-    AudioObjectPropertyAddress addr;
     float level;
     OSStatus sc;
 
@@ -2300,17 +2315,15 @@ static HRESULT ca_setvol(ACImpl *This, UINT32 index)
         return ret;
     }
 
-    level = This->session->master_vol * This->session->channel_vols[index] *
-        This->vols[index];
+    if(This->session->mute)
+        level = 0;
+    else
+        level = This->session->master_vol *
+            This->session->channel_vols[index] * This->vols[index];
 
-    addr.mScope = This->scope;
-    addr.mSelector = kAudioDevicePropertyVolumeScalar;
-    addr.mElement = index + 1;
-
-    sc = AudioObjectSetPropertyData(This->adevid, &addr, 0, NULL,
-            sizeof(float), &level);
+    sc = AudioQueueSetParameter(This->aqueue, kAudioQueueParam_Volume, level);
     if(sc != noErr){
-        WARN("Setting _VolumeScalar property failed: %lx\n", sc);
+        WARN("Setting _Volume property failed: %lx\n", sc);
         return E_FAIL;
     }
 
@@ -2413,9 +2426,20 @@ static HRESULT WINAPI SimpleAudioVolume_SetMute(ISimpleAudioVolume *iface,
     AudioSessionWrapper *This = impl_from_ISimpleAudioVolume(iface);
     AudioSession *session = This->session;
 
-    FIXME("(%p)->(%u, %p) - stub\n", session, mute, context);
+    TRACE("(%p)->(%u, %p)\n", session, mute, context);
 
-    return E_NOTIMPL;
+    if(context)
+        FIXME("Notifications not supported yet\n");
+
+    EnterCriticalSection(&session->lock);
+
+    session->mute = mute;
+
+    ca_session_setvol(session, -1);
+
+    LeaveCriticalSection(&session->lock);
+
+    return S_OK;
 }
 
 static HRESULT WINAPI SimpleAudioVolume_GetMute(ISimpleAudioVolume *iface,
@@ -2424,12 +2448,14 @@ static HRESULT WINAPI SimpleAudioVolume_GetMute(ISimpleAudioVolume *iface,
     AudioSessionWrapper *This = impl_from_ISimpleAudioVolume(iface);
     AudioSession *session = This->session;
 
-    FIXME("(%p)->(%p) - stub\n", session, mute);
+    TRACE("(%p)->(%p)\n", session, mute);
 
     if(!mute)
         return NULL_PTR_ERR;
 
-    return E_NOTIMPL;
+    *mute = session->mute;
+
+    return S_OK;
 }
 
 static const ISimpleAudioVolumeVtbl SimpleAudioVolume_Vtbl  =
@@ -2509,6 +2535,7 @@ static HRESULT WINAPI AudioStreamVolume_SetChannelVolume(
 
     This->vols[index] = level;
 
+    WARN("AudioQueue doesn't support per-channel volume control\n");
     ret = ca_setvol(This, index);
 
     OSSpinLockUnlock(&This->lock);
@@ -2670,6 +2697,7 @@ static HRESULT WINAPI ChannelAudioVolume_SetChannelVolume(
 
     session->channel_vols[index] = level;
 
+    WARN("AudioQueue doesn't support per-channel volume control\n");
     ret = ca_session_setvol(session, index);
 
     LeaveCriticalSection(&session->lock);
@@ -2816,7 +2844,7 @@ HRESULT WINAPI AudioSessionManager_GetAudioSessionControl(
     TRACE("(%p)->(%s, %x, %p)\n", This, debugstr_guid(session_guid),
             flags, out);
 
-    hr = get_audio_session(session_guid, This->flow, 0, &session);
+    hr = get_audio_session(session_guid, This->device, 0, &session);
     if(FAILED(hr))
         return hr;
 
@@ -2843,7 +2871,7 @@ HRESULT WINAPI AudioSessionManager_GetSimpleAudioVolume(
     TRACE("(%p)->(%s, %x, %p)\n", This, debugstr_guid(session_guid),
             flags, out);
 
-    hr = get_audio_session(session_guid, This->flow, 0, &session);
+    hr = get_audio_session(session_guid, This->device, 0, &session);
     if(FAILED(hr))
         return hr;
 
@@ -2914,17 +2942,17 @@ static const IAudioSessionManager2Vtbl AudioSessionManager2_Vtbl =
     AudioSessionManager_UnregisterDuckNotification
 };
 
-HRESULT WINAPI AUDDRV_GetAudioSessionManager(EDataFlow dataflow,
+HRESULT WINAPI AUDDRV_GetAudioSessionManager(IMMDevice *device,
         IAudioSessionManager2 **out)
 {
     SessionMgr *This;
 
     This = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(SessionMgr));
-    if(!*out)
+    if(!This)
         return E_OUTOFMEMORY;
 
     This->IAudioSessionManager2_iface.lpVtbl = &AudioSessionManager2_Vtbl;
-    This->flow = dataflow;
+    This->device = device;
     This->ref = 1;
 
     *out = &This->IAudioSessionManager2_iface;
