@@ -112,85 +112,7 @@ static void context_destroy_fbo(struct wined3d_context *context, GLuint *fbo)
 }
 
 /* GL locking is done by the caller */
-static void context_apply_attachment_filter_states(const struct wined3d_context *context,
-        struct wined3d_surface *surface, DWORD location)
-{
-    /* Update base texture states array */
-    if (surface->container.type == WINED3D_CONTAINER_TEXTURE)
-    {
-        struct wined3d_texture *texture = surface->container.u.texture;
-        struct wined3d_device *device = surface->resource.device;
-        BOOL update_minfilter = FALSE;
-        BOOL update_magfilter = FALSE;
-        struct gl_texture *gl_tex;
-
-        switch (location)
-        {
-            case SFLAG_INTEXTURE:
-            case SFLAG_INSRGBTEX:
-                gl_tex = wined3d_texture_get_gl_texture(texture,
-                        context->gl_info, location == SFLAG_INSRGBTEX);
-                break;
-
-            default:
-                ERR("Unsupported location %s (%#x).\n", debug_surflocation(location), location);
-                return;
-        }
-
-        if (gl_tex->states[WINED3DTEXSTA_MINFILTER] != WINED3DTEXF_POINT
-            || gl_tex->states[WINED3DTEXSTA_MIPFILTER] != WINED3DTEXF_NONE)
-        {
-            gl_tex->states[WINED3DTEXSTA_MINFILTER] = WINED3DTEXF_POINT;
-            gl_tex->states[WINED3DTEXSTA_MIPFILTER] = WINED3DTEXF_NONE;
-            update_minfilter = TRUE;
-        }
-
-        if (gl_tex->states[WINED3DTEXSTA_MAGFILTER] != WINED3DTEXF_POINT)
-        {
-            gl_tex->states[WINED3DTEXSTA_MAGFILTER] = WINED3DTEXF_POINT;
-            update_magfilter = TRUE;
-        }
-
-        if (texture->bind_count)
-        {
-            WARN("Render targets should not be bound to a sampler\n");
-            device_invalidate_state(device, STATE_SAMPLER(texture->sampler));
-        }
-
-        if (update_minfilter || update_magfilter)
-        {
-            GLenum target, bind_target;
-            GLint old_binding;
-
-            target = surface->texture_target;
-            if (target == GL_TEXTURE_2D)
-            {
-                bind_target = GL_TEXTURE_2D;
-                glGetIntegerv(GL_TEXTURE_BINDING_2D, &old_binding);
-            }
-            else if (target == GL_TEXTURE_RECTANGLE_ARB)
-            {
-                bind_target = GL_TEXTURE_RECTANGLE_ARB;
-                glGetIntegerv(GL_TEXTURE_BINDING_RECTANGLE_ARB, &old_binding);
-            }
-            else
-            {
-                bind_target = GL_TEXTURE_CUBE_MAP_ARB;
-                glGetIntegerv(GL_TEXTURE_BINDING_CUBE_MAP_ARB, &old_binding);
-            }
-
-            glBindTexture(bind_target, gl_tex->name);
-            if (update_minfilter) glTexParameteri(bind_target, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-            if (update_magfilter) glTexParameteri(bind_target, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-            glBindTexture(bind_target, old_binding);
-        }
-
-        checkGLcall("apply_attachment_filter_states()");
-    }
-}
-
-/* GL locking is done by the caller */
-void context_attach_depth_stencil_fbo(struct wined3d_context *context,
+void context_attach_depth_stencil_fbo(const struct wined3d_context *context,
         GLenum fbo_target, struct wined3d_surface *depth_stencil, BOOL use_render_buffer)
 {
     const struct wined3d_gl_info *gl_info = context->gl_info;
@@ -220,7 +142,6 @@ void context_attach_depth_stencil_fbo(struct wined3d_context *context,
         else
         {
             surface_prepare_texture(depth_stencil, gl_info, FALSE);
-            context_apply_attachment_filter_states(context, depth_stencil, SFLAG_INTEXTURE);
 
             if (format_flags & WINED3DFMT_FLAG_DEPTH)
             {
@@ -279,7 +200,6 @@ static void context_attach_surface_fbo(const struct wined3d_context *context,
             case SFLAG_INSRGBTEX:
                 srgb = location == SFLAG_INSRGBTEX;
                 surface_prepare_texture(surface, gl_info, srgb);
-                context_apply_attachment_filter_states(context, surface, location);
                 gl_info->fbo_ops.glFramebufferTexture2D(fbo_target, GL_COLOR_ATTACHMENT0 + idx,
                         surface->texture_target, surface_get_texture_name(surface, gl_info, srgb),
                         surface->texture_level);
@@ -299,7 +219,7 @@ static void context_attach_surface_fbo(const struct wined3d_context *context,
 }
 
 /* GL locking is done by the caller */
-void context_check_fbo_status(struct wined3d_context *context, GLenum target)
+void context_check_fbo_status(const struct wined3d_context *context, GLenum target)
 {
     const struct wined3d_gl_info *gl_info = context->gl_info;
     GLenum status;
@@ -313,7 +233,7 @@ void context_check_fbo_status(struct wined3d_context *context, GLenum target)
     }
     else
     {
-        struct wined3d_surface *attachment;
+        const struct wined3d_surface *attachment;
         unsigned int i;
 
         FIXME("FBO status %s (%#x)\n", debug_fbostatus(status), status);
@@ -348,7 +268,7 @@ void context_check_fbo_status(struct wined3d_context *context, GLenum target)
     }
 }
 
-static struct fbo_entry *context_create_fbo_entry(struct wined3d_context *context,
+static struct fbo_entry *context_create_fbo_entry(const struct wined3d_context *context,
         struct wined3d_surface **render_targets, struct wined3d_surface *depth_stencil, DWORD location)
 {
     const struct wined3d_gl_info *gl_info = context->gl_info;
@@ -450,31 +370,20 @@ static void context_apply_fbo_entry(struct wined3d_context *context, GLenum targ
 
     context_bind_fbo(context, target, &entry->id);
 
-    if (!entry->attached)
-    {
-        /* Apply render targets */
-        for (i = 0; i < gl_info->limits.buffers; ++i)
-        {
-            context_attach_surface_fbo(context, target, i, entry->render_targets[i], entry->location);
-        }
+    if (entry->attached) return;
 
-        /* Apply depth targets */
-        if (entry->depth_stencil)
-            surface_set_compatible_renderbuffer(entry->depth_stencil, entry->render_targets[0]);
-        context_attach_depth_stencil_fbo(context, target, entry->depth_stencil, TRUE);
-
-        entry->attached = TRUE;
-    }
-    else
+    /* Apply render targets */
+    for (i = 0; i < gl_info->limits.buffers; ++i)
     {
-        for (i = 0; i < gl_info->limits.buffers; ++i)
-        {
-            if (entry->render_targets[i])
-                context_apply_attachment_filter_states(context, entry->render_targets[i], entry->location);
-        }
-        if (entry->depth_stencil)
-            context_apply_attachment_filter_states(context, entry->depth_stencil, SFLAG_INTEXTURE);
+        context_attach_surface_fbo(context, target, i, entry->render_targets[i], entry->location);
     }
+
+    /* Apply depth targets */
+    if (entry->depth_stencil)
+        surface_set_compatible_renderbuffer(entry->depth_stencil, entry->render_targets[0]);
+    context_attach_depth_stencil_fbo(context, target, entry->depth_stencil, TRUE);
+
+    entry->attached = TRUE;
 }
 
 /* GL locking is done by the caller */
@@ -654,8 +563,8 @@ void context_free_event_query(struct wined3d_event_query *query)
 
 typedef void (context_fbo_entry_func_t)(struct wined3d_context *context, struct fbo_entry *entry);
 
-static void context_enum_surface_fbo_entries(struct wined3d_device *device,
-        struct wined3d_surface *surface, context_fbo_entry_func_t *callback)
+static void context_enum_surface_fbo_entries(const struct wined3d_device *device,
+        const struct wined3d_surface *surface, context_fbo_entry_func_t *callback)
 {
     UINT i;
 
@@ -695,7 +604,7 @@ static void context_queue_fbo_entry_destruction(struct wined3d_context *context,
     list_add_head(&context->fbo_destroy_list, &entry->entry);
 }
 
-void context_resource_released(struct wined3d_device *device,
+void context_resource_released(const struct wined3d_device *device,
         struct wined3d_resource *resource, WINED3DRESOURCETYPE type)
 {
     if (!device->d3d_initialized) return;
@@ -717,7 +626,7 @@ static void context_detach_fbo_entry(struct wined3d_context *context, struct fbo
     entry->attached = FALSE;
 }
 
-void context_resource_unloaded(struct wined3d_device *device,
+void context_resource_unloaded(const struct wined3d_device *device,
         struct wined3d_resource *resource, WINED3DRESOURCETYPE type)
 {
     switch (type)
@@ -732,7 +641,7 @@ void context_resource_unloaded(struct wined3d_device *device,
     }
 }
 
-void context_surface_update(struct wined3d_context *context, struct wined3d_surface *surface)
+void context_surface_update(struct wined3d_context *context, const struct wined3d_surface *surface)
 {
     const struct wined3d_gl_info *gl_info = context->gl_info;
     struct fbo_entry *entry = context->current_fbo;
@@ -804,58 +713,36 @@ static BOOL context_set_gl_context(struct wined3d_context *ctx)
 
     if (!pwglMakeCurrent(ctx->hdc, ctx->glCtx))
     {
+        HDC dc;
+
         WARN("Failed to make GL context %p current on device context %p, last error %#x.\n",
                 ctx->glCtx, ctx->hdc, GetLastError());
         ctx->valid = 0;
         WARN("Trying fallback to the backup window.\n");
 
-        if (!swapchain->backup_dc)
+        if (!(dc = swapchain_get_backup_dc(swapchain)))
         {
-            TRACE("Creating the backup window for swapchain %p.\n", swapchain);
-            swapchain->backup_wnd = CreateWindowA(WINED3D_OPENGL_WINDOW_CLASS_NAME, "WineD3D fake window",
-                    WS_OVERLAPPEDWINDOW, 10, 10, 10, 10, NULL, NULL, NULL, NULL);
-            if (!swapchain->backup_wnd)
-            {
-                ERR("Failed to create a window.\n");
-                goto fail;
-            }
-            swapchain->backup_dc = GetDC(swapchain->backup_wnd);
-            if (!swapchain->backup_dc)
-            {
-                ERR("Failed to get a DC.\n");
-                goto fail;
-            }
-            if (!context_set_pixel_format(ctx->gl_info, swapchain->backup_dc, ctx->pixel_format))
-            {
-                ERR("Failed to set pixel format %d on device context %p.\n",
-                        ctx->pixel_format, swapchain->backup_dc);
-                goto fail;
-            }
+            context_set_current(NULL);
+            return FALSE;
         }
 
-        if (!pwglMakeCurrent(swapchain->backup_dc, ctx->glCtx))
+        if (!context_set_pixel_format(ctx->gl_info, dc, ctx->pixel_format))
+        {
+            ERR("Failed to set pixel format %d on device context %p.\n",
+                    ctx->pixel_format, dc);
+            context_set_current(NULL);
+            return FALSE;
+        }
+
+        if (!pwglMakeCurrent(dc, ctx->glCtx))
         {
             ERR("Fallback to backup window (dc %p) failed too, last error %#x.\n",
-                    swapchain->backup_dc, GetLastError());
+                    dc, GetLastError());
             context_set_current(NULL);
             return FALSE;
         }
     }
     return TRUE;
-
-fail:
-    if (swapchain->backup_dc)
-    {
-        ReleaseDC(swapchain->backup_wnd, swapchain->backup_dc);
-        swapchain->backup_dc = NULL;
-    }
-    if (swapchain->backup_wnd)
-    {
-        DestroyWindow(swapchain->backup_wnd);
-        swapchain->backup_wnd = NULL;
-    }
-    context_set_current(NULL);
-    return FALSE;
 }
 
 static void context_restore_gl_context(HDC dc, HGLRC gl_ctx)
@@ -1323,13 +1210,13 @@ static int WineD3D_ChoosePixelFormat(struct wined3d_device *device, HDC hdc,
     return iPixelFormat;
 }
 
-static inline DWORD generate_rt_mask(GLenum buffer)
+static inline DWORD context_generate_rt_mask(GLenum buffer)
 {
     /* Should take care of all the GL_FRONT/GL_BACK/GL_AUXi/GL_NONE... cases */
     return buffer ? (1 << 31) | buffer : 0;
 }
 
-static inline DWORD generate_rt_mask_from_surface(struct wined3d_surface *target)
+static inline DWORD context_generate_rt_mask_from_surface(const struct wined3d_surface *target)
 {
     return (1 << 31) | surface_get_gl_buffer(target);
 }
@@ -1363,8 +1250,13 @@ struct wined3d_context *context_create(struct wined3d_swapchain *swapchain,
 
     if (!(hdc = GetDC(swapchain->win_handle)))
     {
-        ERR("Failed to retrieve a device context.\n");
-        goto out;
+        WARN("Failed to retireve device context, trying swapchain backup.\n");
+
+        if (!(hdc = swapchain_get_backup_dc(swapchain)))
+        {
+            ERR("Failed to retrieve a device context.\n");
+            goto out;
+        }
     }
 
     color_format = target->resource.format;
@@ -1470,7 +1362,7 @@ struct wined3d_context *context_create(struct wined3d_swapchain *swapchain,
     ret->tid = GetCurrentThreadId();
 
     ret->render_offscreen = surface_is_offscreen(target);
-    ret->draw_buffers_mask = generate_rt_mask(GL_BACK);
+    ret->draw_buffers_mask = context_generate_rt_mask(GL_BACK);
     ret->valid = 1;
 
     ret->glCtx = ctx;
@@ -1923,7 +1815,7 @@ static void SetupForBlit(struct wined3d_device *device, struct wined3d_context *
 }
 
 /* Do not call while under the GL lock. */
-static struct wined3d_context *FindContext(struct wined3d_device *device, struct wined3d_surface *target)
+static struct wined3d_context *FindContext(const struct wined3d_device *device, const struct wined3d_surface *target)
 {
     struct wined3d_context *current_context = context_get_current();
     struct wined3d_context *context;
@@ -1986,7 +1878,7 @@ static inline GLenum draw_buffer_from_rt_mask(DWORD rt_mask)
 }
 
 /* Context activation and GL locking are done by the caller. */
-static void context_apply_draw_buffers(struct wined3d_context *context, DWORD rt_mask, struct wined3d_surface **rts)
+static void context_apply_draw_buffers(struct wined3d_context *context, DWORD rt_mask)
 {
     if (!rt_mask)
     {
@@ -2007,7 +1899,7 @@ static void context_apply_draw_buffers(struct wined3d_context *context, DWORD rt
 
             while (rt_mask)
             {
-                if ((rt_mask & 1) && rts[i] && rts[i]->resource.format->id != WINED3DFMT_NULL)
+                if (rt_mask & 1)
                     context->draw_buffers[i] = GL_COLOR_ATTACHMENT0 + i;
                 else
                     context->draw_buffers[i] = GL_NONE;
@@ -2029,8 +1921,7 @@ static void context_apply_draw_buffers(struct wined3d_context *context, DWORD rt
         }
         else
         {
-            glDrawBuffer(draw_buffer_from_rt_mask(rt_mask));
-            checkGLcall("glDrawBuffer()");
+            ERR("Unexpected draw buffers mask with backbuffer ORM.\n");
         }
     }
 }
@@ -2040,7 +1931,7 @@ void context_set_draw_buffer(struct wined3d_context *context, GLenum buffer)
 {
     glDrawBuffer(buffer);
     checkGLcall("glDrawBuffer()");
-    context->draw_buffers_mask = generate_rt_mask(buffer);
+    context->draw_buffers_mask = context_generate_rt_mask(buffer);
 }
 
 static inline void context_set_render_offscreen(struct wined3d_context *context, const struct StateEntry *StateTable,
@@ -2076,7 +1967,7 @@ static BOOL match_depth_stencil_format(const struct wined3d_format *existing,
 
 /* The caller provides a context */
 static void context_validate_onscreen_formats(struct wined3d_device *device,
-        struct wined3d_context *context, struct wined3d_surface *depth_stencil)
+        struct wined3d_context *context, const struct wined3d_surface *depth_stencil)
 {
     /* Onscreen surfaces are always in a swapchain */
     struct wined3d_swapchain *swapchain = context->current_rt->container.u.swapchain;
@@ -2095,6 +1986,16 @@ static void context_validate_onscreen_formats(struct wined3d_device *device,
     context_set_render_offscreen(context, device->StateTable, TRUE);
 }
 
+static DWORD context_generate_rt_mask_no_fbo(const struct wined3d_device *device, const struct wined3d_surface *rt)
+{
+    if (!rt || rt->resource.format->id == WINED3DFMT_NULL)
+        return 0;
+    else if (rt->container.type == WINED3D_CONTAINER_SWAPCHAIN)
+        return context_generate_rt_mask_from_surface(rt);
+    else
+        return context_generate_rt_mask(device->offscreenBuffer);
+}
+
 /* Context activation is done by the caller. */
 void context_apply_blit_state(struct wined3d_context *context, struct wined3d_device *device)
 {
@@ -2111,25 +2012,28 @@ void context_apply_blit_state(struct wined3d_context *context, struct wined3d_de
             ENTER_GL();
             context_apply_fbo_state_blit(context, GL_FRAMEBUFFER, context->current_rt, NULL, SFLAG_INTEXTURE);
             LEAVE_GL();
-            rt_mask = 1;
+            if (context->current_rt && context->current_rt->resource.format->id != WINED3DFMT_NULL)
+                rt_mask = 1;
+            else
+                rt_mask = 0;
         }
         else
         {
             ENTER_GL();
             context_bind_fbo(context, GL_FRAMEBUFFER, NULL);
             LEAVE_GL();
-            rt_mask = generate_rt_mask_from_surface(context->current_rt);
+            rt_mask = context_generate_rt_mask_from_surface(context->current_rt);
         }
     }
     else
     {
-        rt_mask = generate_rt_mask_from_surface(context->current_rt);
+        rt_mask = context_generate_rt_mask_no_fbo(device, context->current_rt);
     }
 
     ENTER_GL();
     if (rt_mask != context->draw_buffers_mask)
     {
-        context_apply_draw_buffers(context, 1, &context->current_rt);
+        context_apply_draw_buffers(context, rt_mask);
         context->draw_buffers_mask = rt_mask;
     }
 
@@ -2140,10 +2044,11 @@ void context_apply_blit_state(struct wined3d_context *context, struct wined3d_de
     LEAVE_GL();
 
     SetupForBlit(device, context);
+    context_invalidate_state(context, STATE_FRAMEBUFFER, device->StateTable);
 }
 
 static BOOL context_validate_rt_config(UINT rt_count,
-        struct wined3d_surface **rts, struct wined3d_surface *ds)
+        struct wined3d_surface * const *rts, const struct wined3d_surface *ds)
 {
     unsigned int i;
 
@@ -2161,19 +2066,20 @@ static BOOL context_validate_rt_config(UINT rt_count,
 
 /* Context activation is done by the caller. */
 BOOL context_apply_clear_state(struct wined3d_context *context, struct wined3d_device *device,
-        UINT rt_count, struct wined3d_surface **rts, struct wined3d_surface *depth_stencil)
+        UINT rt_count, const struct wined3d_fb_state *fb)
 {
     const struct StateEntry *state_table = device->StateTable;
     DWORD rt_mask = 0;
     UINT i;
+    struct wined3d_surface **rts = fb->render_targets;
 
-    if (!context_validate_rt_config(rt_count, rts, depth_stencil))
+    if (!context_validate_rt_config(rt_count, rts, fb->depth_stencil))
         return FALSE;
 
 
     if (wined3d_settings.offscreen_rendering_mode == ORM_FBO)
     {
-        context_validate_onscreen_formats(device, context, depth_stencil);
+        context_validate_onscreen_formats(device, context, fb->depth_stencil);
 
         ENTER_GL();
 
@@ -2182,34 +2088,38 @@ BOOL context_apply_clear_state(struct wined3d_context *context, struct wined3d_d
             for (i = 0; i < rt_count; ++i)
             {
                 context->blit_targets[i] = rts[i];
-                rt_mask |= (1 << i);
+                if (rts[i] && rts[i]->resource.format->id != WINED3DFMT_NULL)
+                    rt_mask |= (1 << i);
             }
             while (i < context->gl_info->limits.buffers)
             {
                 context->blit_targets[i] = NULL;
                 ++i;
             }
-            context_apply_fbo_state(context, GL_FRAMEBUFFER, context->blit_targets, depth_stencil, SFLAG_INTEXTURE);
+            context_apply_fbo_state(context, GL_FRAMEBUFFER, context->blit_targets, fb->depth_stencil, SFLAG_INTEXTURE);
             glReadBuffer(GL_NONE);
             checkGLcall("glReadBuffer");
         }
         else
         {
             context_apply_fbo_state(context, GL_FRAMEBUFFER, NULL, NULL, SFLAG_INDRAWABLE);
-            rt_mask = generate_rt_mask_from_surface(rts[0]);
+            rt_mask = context_generate_rt_mask_from_surface(rts[0]);
         }
 
         LEAVE_GL();
+
+        /* TODO: This is not necessary if the framebuffer is the device's current framebuffer */
+        context_invalidate_state(context, STATE_FRAMEBUFFER, device->StateTable);
     }
     else
     {
-        rt_mask = generate_rt_mask_from_surface(rts[0]);
+        rt_mask = context_generate_rt_mask_no_fbo(device, rts[0]);
     }
 
     ENTER_GL();
     if (rt_mask != context->draw_buffers_mask)
     {
-        context_apply_draw_buffers(context, rt_mask, rts);
+        context_apply_draw_buffers(context, rt_mask);
         context->draw_buffers_mask = rt_mask;
     }
 
@@ -2239,13 +2149,82 @@ BOOL context_apply_clear_state(struct wined3d_context *context, struct wined3d_d
     return TRUE;
 }
 
+static inline DWORD find_draw_buffers_mask(struct wined3d_context *context, struct wined3d_device *device)
+{
+    struct wined3d_shader *ps = device->stateBlock->state.pixel_shader;
+    struct wined3d_surface **rts = device->fb.render_targets;
+    DWORD rt_mask, rt_mask_bits;
+    unsigned int i;
+
+    if (wined3d_settings.offscreen_rendering_mode != ORM_FBO) return context_generate_rt_mask_no_fbo(device, rts[0]);
+    else if (!context->render_offscreen) return context_generate_rt_mask_from_surface(rts[0]);
+
+    rt_mask = ps ? ps->reg_maps.rt_mask : 1;
+    rt_mask &= device->valid_rt_mask;
+    rt_mask_bits = rt_mask;
+    i = 0;
+    while (rt_mask_bits)
+    {
+        rt_mask_bits &= ~(1 << i);
+        if (!rts[i] || rts[i]->resource.format->id == WINED3DFMT_NULL)
+            rt_mask &= ~(1 << i);
+
+        i++;
+    }
+
+    return rt_mask;
+}
+
+/* GL locking and context activation are done by the caller */
+void context_state_fb(DWORD state, struct wined3d_stateblock *stateblock, struct wined3d_context *context)
+{
+    struct wined3d_device *device = stateblock->device;
+    const struct wined3d_fb_state *fb = &device->fb;
+    DWORD rt_mask = find_draw_buffers_mask(context, device);
+
+    if (wined3d_settings.offscreen_rendering_mode == ORM_FBO)
+    {
+        if (!context->render_offscreen)
+        {
+            context_apply_fbo_state(context, GL_FRAMEBUFFER, NULL, NULL, SFLAG_INDRAWABLE);
+        }
+        else
+        {
+            context_apply_fbo_state(context, GL_FRAMEBUFFER, fb->render_targets, fb->depth_stencil, SFLAG_INTEXTURE);
+            glReadBuffer(GL_NONE);
+            checkGLcall("glReadBuffer");
+        }
+    }
+
+    if (context->draw_buffers_mask != rt_mask)
+    {
+        context_apply_draw_buffers(context, rt_mask);
+        context->draw_buffers_mask = rt_mask;
+    }
+}
+
+/* GL locking and context activation are done by the caller */
+void context_state_drawbuf(DWORD state, struct wined3d_stateblock *stateblock, struct wined3d_context *context)
+{
+    DWORD rt_mask;
+    struct wined3d_device *device = stateblock->device;
+
+    if (isStateDirty(context, STATE_FRAMEBUFFER)) return;
+
+    rt_mask = find_draw_buffers_mask(context, device);
+    if (context->draw_buffers_mask != rt_mask)
+    {
+        context_apply_draw_buffers(context, rt_mask);
+        context->draw_buffers_mask = rt_mask;
+    }
+}
+
 /* Context activation is done by the caller. */
 BOOL context_apply_draw_state(struct wined3d_context *context, struct wined3d_device *device)
 {
     const struct StateEntry *state_table = device->StateTable;
     const struct wined3d_fb_state *fb = &device->fb;
     unsigned int i;
-    DWORD rt_mask;
 
     if (!context_validate_rt_config(context->gl_info->limits.buffers,
             fb->render_targets, fb->depth_stencil))
@@ -2256,50 +2235,10 @@ BOOL context_apply_draw_state(struct wined3d_context *context, struct wined3d_de
      * updating a resource location. */
     device_update_tex_unit_map(device);
     device_preload_textures(device);
-    if (isStateDirty(context, STATE_VDECL))
+    if (isStateDirty(context, STATE_VDECL) || isStateDirty(context, STATE_STREAMSRC))
         device_update_stream_info(device, context->gl_info);
 
-    if (wined3d_settings.offscreen_rendering_mode == ORM_FBO)
-    {
-        context_validate_onscreen_formats(device, context, fb->depth_stencil);
-
-        if (!context->render_offscreen)
-        {
-            ENTER_GL();
-            context_apply_fbo_state(context, GL_FRAMEBUFFER, NULL, NULL, SFLAG_INDRAWABLE);
-            LEAVE_GL();
-            rt_mask = generate_rt_mask_from_surface(fb->render_targets[0]);
-        }
-        else
-        {
-            const struct wined3d_shader *ps = device->stateBlock->state.pixel_shader;
-
-            ENTER_GL();
-            context_apply_fbo_state(context, GL_FRAMEBUFFER, fb->render_targets, fb->depth_stencil, SFLAG_INTEXTURE);
-            glReadBuffer(GL_NONE);
-            checkGLcall("glReadBuffer");
-            LEAVE_GL();
-            rt_mask = ps ? ps->reg_maps.rt_mask : 1;
-            rt_mask &= device->valid_rt_mask;
-        }
-    }
-    else
-    {
-        rt_mask = generate_rt_mask_from_surface(fb->render_targets[0]);
-    }
-
     ENTER_GL();
-    if (context->draw_buffers_mask != rt_mask)
-    {
-        context_apply_draw_buffers(context, rt_mask, fb->render_targets);
-        context->draw_buffers_mask = rt_mask;
-    }
-
-    if (wined3d_settings.offscreen_rendering_mode == ORM_FBO)
-    {
-        context_check_fbo_status(context, GL_FRAMEBUFFER);
-    }
-
     if (context->last_was_blit)
     {
         device->frag_pipe->enable_extension(TRUE);
@@ -2313,6 +2252,12 @@ BOOL context_apply_draw_state(struct wined3d_context *context, struct wined3d_de
         context->isStateDirty[idx] &= ~(1 << shift);
         state_table[rep].apply(rep, device->stateBlock, context);
     }
+
+    if (wined3d_settings.offscreen_rendering_mode == ORM_FBO)
+    {
+        context_check_fbo_status(context, GL_FRAMEBUFFER);
+    }
+
     LEAVE_GL();
     context->numDirtyEntries = 0; /* This makes the whole list clean */
     context->last_was_blit = FALSE;
