@@ -74,11 +74,12 @@ typedef struct _AudioSession {
     GUID guid;
     struct list clients;
 
-    EDataFlow dataflow;
+    IMMDevice *device;
 
     float master_vol;
     UINT32 channel_count;
     float *channel_vols;
+    BOOL mute;
 
     CRITICAL_SECTION lock;
 
@@ -147,7 +148,7 @@ typedef struct _SessionMgr {
 
     LONG ref;
 
-    EDataFlow flow;
+    IMMDevice *device;
 } SessionMgr;
 
 static HANDLE g_timer_q;
@@ -656,18 +657,26 @@ static HRESULT setup_oss_device(ACImpl *This, const WAVEFORMATEX *fmt,
 
 static void session_init_vols(AudioSession *session, UINT channels)
 {
-    session->channel_vols = HeapAlloc(GetProcessHeap(), 0,
-            sizeof(float) * channels);
-    if(!session->channel_vols)
-        return;
+    if(session->channel_count < channels){
+        UINT i;
 
-    session->channel_count = channels;
+        if(session->channel_vols)
+            session->channel_vols = HeapReAlloc(GetProcessHeap(), 0,
+                    session->channel_vols, sizeof(float) * channels);
+        else
+            session->channel_vols = HeapAlloc(GetProcessHeap(), 0,
+                    sizeof(float) * channels);
+        if(!session->channel_vols)
+            return;
 
-    for(; channels > 0; --channels)
-        session->channel_vols[channels - 1] = 1.f;
+        for(i = session->channel_count; i < channels; ++i)
+            session->channel_vols[i] = 1.f;
+
+        session->channel_count = channels;
+    }
 }
 
-static AudioSession *create_session(const GUID *guid, EDataFlow flow,
+static AudioSession *create_session(const GUID *guid, IMMDevice *device,
         UINT num_channels)
 {
     AudioSession *ret;
@@ -678,7 +687,7 @@ static AudioSession *create_session(const GUID *guid, EDataFlow flow,
 
     memcpy(&ret->guid, guid, sizeof(GUID));
 
-    ret->dataflow = flow;
+    ret->device = device;
 
     list_init(&ret->clients);
 
@@ -686,8 +695,7 @@ static AudioSession *create_session(const GUID *guid, EDataFlow flow,
 
     InitializeCriticalSection(&ret->lock);
 
-    if(num_channels > 0)
-        session_init_vols(ret, num_channels);
+    session_init_vols(ret, num_channels);
 
     ret->master_vol = 1.f;
 
@@ -697,12 +705,12 @@ static AudioSession *create_session(const GUID *guid, EDataFlow flow,
 /* if channels == 0, then this will return or create a session with
  * matching dataflow and GUID. otherwise, channels must also match */
 static HRESULT get_audio_session(const GUID *sessionguid,
-        EDataFlow flow, UINT channels, AudioSession **out)
+        IMMDevice *device, UINT channels, AudioSession **out)
 {
     AudioSession *session;
 
     if(!sessionguid || IsEqualGUID(sessionguid, &GUID_NULL)){
-        *out = create_session(&GUID_NULL, flow, channels);
+        *out = create_session(&GUID_NULL, device, channels);
         if(!*out)
             return E_OUTOFMEMORY;
 
@@ -711,20 +719,16 @@ static HRESULT get_audio_session(const GUID *sessionguid,
 
     *out = NULL;
     LIST_FOR_EACH_ENTRY(session, &g_sessions, AudioSession, entry){
-        if(IsEqualGUID(sessionguid, &session->guid) &&
-                session->dataflow == flow){
-            if(session->channel_count > 0){
-                if(channels > 0 && session->channel_count != channels)
-                    return E_INVALIDARG;
-            }else if(channels > 0)
-                session_init_vols(session, channels);
+        if(session->device == device &&
+                IsEqualGUID(sessionguid, &session->guid)){
+            session_init_vols(session, channels);
             *out = session;
             break;
         }
     }
 
     if(!*out){
-        *out = create_session(sessionguid, flow, channels);
+        *out = create_session(sessionguid, device, channels);
         if(!*out)
             return E_OUTOFMEMORY;
     }
@@ -829,7 +833,7 @@ static HRESULT WINAPI AudioClient_Initialize(IAudioClient *iface,
 
     EnterCriticalSection(&g_sessions_lock);
 
-    hr = get_audio_session(sessionguid, This->dataflow, fmt->nChannels,
+    hr = get_audio_session(sessionguid, This->parent, fmt->nChannels,
             &This->session);
     if(FAILED(hr)){
         LeaveCriticalSection(&g_sessions_lock);
@@ -1102,6 +1106,18 @@ static HRESULT WINAPI AudioClient_GetDevicePeriod(IAudioClient *iface,
     return S_OK;
 }
 
+static void oss_silence_buffer(ACImpl *This, BYTE *buf, UINT32 frames)
+{
+    WAVEFORMATEXTENSIBLE *fmtex = (WAVEFORMATEXTENSIBLE*)This->fmt;
+    if((This->fmt->wFormatTag == WAVE_FORMAT_PCM ||
+            (This->fmt->wFormatTag == WAVE_FORMAT_EXTENSIBLE &&
+             IsEqualGUID(&fmtex->SubFormat, &KSDATAFORMAT_SUBTYPE_PCM))) &&
+            This->fmt->wBitsPerSample == 8)
+        memset(buf, 128, frames * This->fmt->nBlockAlign);
+    else
+        memset(buf, 0, frames * This->fmt->nBlockAlign);
+}
+
 static void oss_write_data(ACImpl *This)
 {
     ssize_t written;
@@ -1115,6 +1131,9 @@ static void oss_write_data(ACImpl *This)
     else
         to_write = This->held_frames;
 
+    if(This->session->mute)
+        oss_silence_buffer(This, buf, to_write);
+
     written = write(This->fd, buf, to_write * This->fmt->nBlockAlign);
     if(written < 0){
         WARN("write failed: %d (%s)\n", errno, strerror(errno));
@@ -1134,6 +1153,10 @@ static void oss_write_data(ACImpl *This)
 
     if(This->held_frames){
         /* wrapped and have some data back at the start to write */
+
+        if(This->session->mute)
+            oss_silence_buffer(This, This->local_buffer, This->held_frames);
+
         written = write(This->fd, This->local_buffer,
                 This->held_frames * This->fmt->nBlockAlign);
         if(written < 0){
@@ -1189,7 +1212,7 @@ static void CALLBACK oss_period_callback(void *user, BOOLEAN timer)
 
     EnterCriticalSection(&This->lock);
 
-    if(This->dataflow == eRender)
+    if(This->dataflow == eRender && This->held_frames)
         oss_write_data(This);
     else if(This->dataflow == eCapture)
         oss_read_data(This);
@@ -1378,16 +1401,20 @@ static HRESULT WINAPI AudioClient_GetService(IAudioClient *iface, REFIID riid,
             LeaveCriticalSection(&This->lock);
             return AUDCLNT_E_WRONG_ENDPOINT_TYPE;
         }
+        IAudioRenderClient_AddRef(&This->IAudioRenderClient_iface);
         *ppv = &This->IAudioRenderClient_iface;
     }else if(IsEqualIID(riid, &IID_IAudioCaptureClient)){
         if(This->dataflow != eCapture){
             LeaveCriticalSection(&This->lock);
             return AUDCLNT_E_WRONG_ENDPOINT_TYPE;
         }
+        IAudioCaptureClient_AddRef(&This->IAudioCaptureClient_iface);
         *ppv = &This->IAudioCaptureClient_iface;
     }else if(IsEqualIID(riid, &IID_IAudioClock)){
+        IAudioClock_AddRef(&This->IAudioClock_iface);
         *ppv = &This->IAudioClock_iface;
     }else if(IsEqualIID(riid, &IID_IAudioStreamVolume)){
+        IAudioStreamVolume_AddRef(&This->IAudioStreamVolume_iface);
         *ppv = &This->IAudioStreamVolume_iface;
     }else if(IsEqualIID(riid, &IID_IAudioSessionControl)){
         if(!This->session_wrapper){
@@ -1396,7 +1423,8 @@ static HRESULT WINAPI AudioClient_GetService(IAudioClient *iface, REFIID riid,
                 LeaveCriticalSection(&This->lock);
                 return E_OUTOFMEMORY;
             }
-        }
+        }else
+            IAudioSessionControl2_AddRef(&This->session_wrapper->IAudioSessionControl2_iface);
 
         *ppv = &This->session_wrapper->IAudioSessionControl2_iface;
     }else if(IsEqualIID(riid, &IID_IChannelAudioVolume)){
@@ -1406,7 +1434,8 @@ static HRESULT WINAPI AudioClient_GetService(IAudioClient *iface, REFIID riid,
                 LeaveCriticalSection(&This->lock);
                 return E_OUTOFMEMORY;
             }
-        }
+        }else
+            IChannelAudioVolume_AddRef(&This->session_wrapper->IChannelAudioVolume_iface);
 
         *ppv = &This->session_wrapper->IChannelAudioVolume_iface;
     }else if(IsEqualIID(riid, &IID_ISimpleAudioVolume)){
@@ -1416,13 +1445,13 @@ static HRESULT WINAPI AudioClient_GetService(IAudioClient *iface, REFIID riid,
                 LeaveCriticalSection(&This->lock);
                 return E_OUTOFMEMORY;
             }
-        }
+        }else
+            ISimpleAudioVolume_AddRef(&This->session_wrapper->ISimpleAudioVolume_iface);
 
         *ppv = &This->session_wrapper->ISimpleAudioVolume_iface;
     }
 
     if(*ppv){
-        IUnknown_AddRef((IUnknown*)*ppv);
         LeaveCriticalSection(&This->lock);
         return S_OK;
     }
@@ -1590,16 +1619,8 @@ static HRESULT WINAPI AudioRenderClient_ReleaseBuffer(
     else
         buffer = This->tmp_buffer;
 
-    if(flags & AUDCLNT_BUFFERFLAGS_SILENT){
-        WAVEFORMATEXTENSIBLE *fmtex = (WAVEFORMATEXTENSIBLE*)This->fmt;
-        if((This->fmt->wFormatTag == WAVE_FORMAT_PCM ||
-                (This->fmt->wFormatTag == WAVE_FORMAT_EXTENSIBLE &&
-                 IsEqualGUID(&fmtex->SubFormat, &KSDATAFORMAT_SUBTYPE_PCM))) &&
-                This->fmt->wBitsPerSample == 8)
-            memset(buffer, 128, written_frames * This->fmt->nBlockAlign);
-        else
-            memset(buffer, 0, written_frames * This->fmt->nBlockAlign);
-    }
+    if(flags & AUDCLNT_BUFFERFLAGS_SILENT)
+        oss_silence_buffer(This, buffer, written_frames);
 
     if(This->held_frames){
         if(This->buf_state == LOCKED_WRAPPED)
@@ -1610,6 +1631,9 @@ static HRESULT WINAPI AudioRenderClient_ReleaseBuffer(
         ssize_t w_bytes;
         UINT32 w_frames;
 
+        if(This->session->mute)
+            oss_silence_buffer(This, buffer, written_frames);
+
         w_bytes = write(This->fd, buffer,
                 written_frames * This->fmt->nBlockAlign);
         if(w_bytes < 0){
@@ -1948,6 +1972,8 @@ static AudioSessionWrapper *AudioSessionWrapper_Create(ACImpl *client)
     ret->ISimpleAudioVolume_iface.lpVtbl = &SimpleAudioVolume_Vtbl;
     ret->IChannelAudioVolume_iface.lpVtbl = &ChannelAudioVolume_Vtbl;
 
+    ret->ref = 1;
+
     ret->client = client;
     if(client){
         ret->session = client->session;
@@ -2094,7 +2120,7 @@ static HRESULT WINAPI AudioSessionControl_GetGroupingParam(
 }
 
 static HRESULT WINAPI AudioSessionControl_SetGroupingParam(
-        IAudioSessionControl2 *iface, GUID *group, const GUID *session)
+        IAudioSessionControl2 *iface, const GUID *group, const GUID *session)
 {
     AudioSessionWrapper *This = impl_from_IAudioSessionControl2(iface);
 
@@ -2358,9 +2384,29 @@ static HRESULT WINAPI SimpleAudioVolume_SetMute(ISimpleAudioVolume *iface,
     AudioSessionWrapper *This = impl_from_ISimpleAudioVolume(iface);
     AudioSession *session = This->session;
 
-    FIXME("(%p)->(%u, %p) - stub\n", session, mute, context);
+    TRACE("(%p)->(%u, %p)\n", session, mute, context);
 
-    return E_NOTIMPL;
+    EnterCriticalSection(&session->lock);
+
+    if(!mute && session->mute){
+        ACImpl *client;
+
+        session->mute = mute;
+
+        LIST_FOR_EACH_ENTRY(client, &session->clients, ACImpl, entry){
+            EnterCriticalSection(&client->lock);
+            if(ioctl(client->fd, SNDCTL_DSP_SKIP) < 0)
+                WARN("Error calling DSP_SKIP: %d (%s)\n", errno,
+                        strerror(errno));
+            oss_write_data(client);
+            LeaveCriticalSection(&client->lock);
+        }
+    }else
+        session->mute = mute;
+
+    LeaveCriticalSection(&session->lock);
+
+    return S_OK;
 }
 
 static HRESULT WINAPI SimpleAudioVolume_GetMute(ISimpleAudioVolume *iface,
@@ -2369,12 +2415,14 @@ static HRESULT WINAPI SimpleAudioVolume_GetMute(ISimpleAudioVolume *iface,
     AudioSessionWrapper *This = impl_from_ISimpleAudioVolume(iface);
     AudioSession *session = This->session;
 
-    FIXME("(%p)->(%p) - stub\n", session, mute);
+    TRACE("(%p)->(%p)\n", session, mute);
 
     if(!mute)
         return NULL_PTR_ERR;
 
-    return E_NOTIMPL;
+    *mute = This->session->mute;
+
+    return S_OK;
 }
 
 static const ISimpleAudioVolumeVtbl SimpleAudioVolume_Vtbl  =
@@ -2761,7 +2809,7 @@ HRESULT WINAPI AudioSessionManager_GetAudioSessionControl(
     TRACE("(%p)->(%s, %x, %p)\n", This, debugstr_guid(session_guid),
             flags, out);
 
-    hr = get_audio_session(session_guid, This->flow, 0, &session);
+    hr = get_audio_session(session_guid, This->device, 0, &session);
     if(FAILED(hr))
         return hr;
 
@@ -2788,7 +2836,7 @@ HRESULT WINAPI AudioSessionManager_GetSimpleAudioVolume(
     TRACE("(%p)->(%s, %x, %p)\n", This, debugstr_guid(session_guid),
             flags, out);
 
-    hr = get_audio_session(session_guid, This->flow, 0, &session);
+    hr = get_audio_session(session_guid, This->device, 0, &session);
     if(FAILED(hr))
         return hr;
 
@@ -2859,17 +2907,17 @@ static const IAudioSessionManager2Vtbl AudioSessionManager2_Vtbl =
     AudioSessionManager_UnregisterDuckNotification
 };
 
-HRESULT WINAPI AUDDRV_GetAudioSessionManager(EDataFlow dataflow,
+HRESULT WINAPI AUDDRV_GetAudioSessionManager(IMMDevice *device,
         IAudioSessionManager2 **out)
 {
     SessionMgr *This;
 
     This = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(SessionMgr));
-    if(!*out)
+    if(!This)
         return E_OUTOFMEMORY;
 
     This->IAudioSessionManager2_iface.lpVtbl = &AudioSessionManager2_Vtbl;
-    This->flow = dataflow;
+    This->device = device;
     This->ref = 1;
 
     *out = &This->IAudioSessionManager2_iface;
