@@ -61,8 +61,8 @@ DWORD write_spool( PHYSDEV dev, const void *data, DWORD num )
 /**********************************************************************
  *           ExtEscape  (WINEPS.@)
  */
-INT PSDRV_ExtEscape( PHYSDEV dev, INT nEscape, INT cbInput, LPCVOID in_data,
-                     INT cbOutput, LPVOID out_data )
+INT CDECL PSDRV_ExtEscape( PHYSDEV dev, INT nEscape, INT cbInput, LPCVOID in_data,
+                           INT cbOutput, LPVOID out_data )
 {
     PSDRV_PDEVICE *physDev = get_psdrv_dev( dev );
 
@@ -368,7 +368,7 @@ INT PSDRV_ExtEscape( PHYSDEV dev, INT nEscape, INT cbInput, LPCVOID in_data,
 /************************************************************************
  *           PSDRV_StartPage
  */
-INT PSDRV_StartPage( PHYSDEV dev )
+INT CDECL PSDRV_StartPage( PHYSDEV dev )
 {
     PSDRV_PDEVICE *physDev = get_psdrv_dev( dev );
 
@@ -392,7 +392,7 @@ INT PSDRV_StartPage( PHYSDEV dev )
 /************************************************************************
  *           PSDRV_EndPage
  */
-INT PSDRV_EndPage( PHYSDEV dev )
+INT CDECL PSDRV_EndPage( PHYSDEV dev )
 {
     PSDRV_PDEVICE *physDev = get_psdrv_dev( dev );
 
@@ -469,7 +469,7 @@ static INT PSDRV_StartDocA( PHYSDEV dev, const DOCINFOA *doc )
 /************************************************************************
  *           PSDRV_StartDoc
  */
-INT PSDRV_StartDoc( PHYSDEV dev, const DOCINFOW *doc )
+INT CDECL PSDRV_StartDoc( PHYSDEV dev, const DOCINFOW *doc )
 {
     DOCINFOA docA;
     INT ret, len;
@@ -514,7 +514,7 @@ INT PSDRV_StartDoc( PHYSDEV dev, const DOCINFOW *doc )
 /************************************************************************
  *           PSDRV_EndDoc
  */
-INT PSDRV_EndDoc( PHYSDEV dev )
+INT CDECL PSDRV_EndDoc( PHYSDEV dev )
 {
     PSDRV_PDEVICE *physDev = get_psdrv_dev( dev );
     INT ret = 1;
@@ -528,9 +528,7 @@ INT PSDRV_EndDoc( PHYSDEV dev )
         WARN("Somebody forgot an EndPage\n");
 	PSDRV_EndPage( dev );
     }
-
-    if (physDev->job.PageNo)
-        PSDRV_WriteFooter( dev );
+    PSDRV_WriteFooter( dev );
 
     ret = EndDocPrinter(physDev->job.hprinter);
     ClosePrinter(physDev->job.hprinter);
