@@ -342,7 +342,6 @@ BOOL CDECL PSDRV_CreateDC( HDC hdc, PSDRV_PDEVICE **pdev, LPCWSTR driver, LPCWST
     physDev = HeapAlloc( PSDRV_Heap, HEAP_ZERO_MEMORY, sizeof(*physDev) );
     if (!physDev) return FALSE;
     *pdev = physDev;
-    physDev->hdc = hdc;
 
     physDev->pi = pi;
 
@@ -381,8 +380,10 @@ BOOL CDECL PSDRV_CreateDC( HDC hdc, PSDRV_PDEVICE **pdev, LPCWSTR driver, LPCWST
 /**********************************************************************
  *	     PSDRV_DeleteDC
  */
-BOOL CDECL PSDRV_DeleteDC( PSDRV_PDEVICE *physDev )
+BOOL CDECL PSDRV_DeleteDC( PHYSDEV dev )
 {
+    PSDRV_PDEVICE *physDev = get_psdrv_dev( dev );
+
     TRACE("\n");
 
     HeapFree( PSDRV_Heap, 0, physDev->Devmode );
@@ -396,22 +397,26 @@ BOOL CDECL PSDRV_DeleteDC( PSDRV_PDEVICE *physDev )
 /**********************************************************************
  *	     ResetDC   (WINEPS.@)
  */
-HDC CDECL PSDRV_ResetDC( PSDRV_PDEVICE *physDev, const DEVMODEW *lpInitData )
+HDC CDECL PSDRV_ResetDC( PHYSDEV dev, const DEVMODEW *lpInitData )
 {
+    PSDRV_PDEVICE *physDev = get_psdrv_dev( dev );
+
     if(lpInitData) {
         DEVMODEA *devmodeA = DEVMODEdupWtoA(PSDRV_Heap, lpInitData);
         PSDRV_MergeDevmodes(physDev->Devmode, (PSDRV_DEVMODEA *)devmodeA, physDev->pi);
         HeapFree(PSDRV_Heap, 0, devmodeA);
         PSDRV_UpdateDevCaps(physDev);
     }
-    return physDev->hdc;
+    return dev->hdc;
 }
 
 /***********************************************************************
  *           GetDeviceCaps    (WINEPS.@)
  */
-INT CDECL PSDRV_GetDeviceCaps( PSDRV_PDEVICE *physDev, INT cap )
+INT CDECL PSDRV_GetDeviceCaps( PHYSDEV dev, INT cap )
 {
+    PSDRV_PDEVICE *physDev = get_psdrv_dev( dev );
+
     switch(cap)
     {
     case DRIVERVERSION:
@@ -514,7 +519,7 @@ INT CDECL PSDRV_GetDeviceCaps( PSDRV_PDEVICE *physDev, INT cap )
     case SHADEBLENDCAPS:
         return SB_NONE;
     default:
-        FIXME("(%p): unsupported capability %d, will return 0\n", physDev->hdc, cap );
+        FIXME("(%p): unsupported capability %d, will return 0\n", dev->hdc, cap );
         return 0;
     }
 }
