@@ -188,8 +188,9 @@ RGNDATA *X11DRV_GetRegionData( HRGN hrgn, HDC hdc_lptodp )
 /***********************************************************************
  *           X11DRV_SetDeviceClipping
  */
-void CDECL X11DRV_SetDeviceClipping( X11DRV_PDEVICE *physDev, HRGN vis_rgn, HRGN clip_rgn )
+void CDECL X11DRV_SetDeviceClipping( PHYSDEV dev, HRGN vis_rgn, HRGN clip_rgn )
 {
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
     RGNDATA *data;
 
     CombineRgn( physDev->region, vis_rgn, clip_rgn, clip_rgn ? RGN_AND : RGN_COPY );
@@ -241,7 +242,7 @@ BOOL X11DRV_SetupGCForPatBlt( X11DRV_PDEVICE *physDev, GC gc, BOOL fMapColors )
         val.background = X11DRV_PALETTE_XPixelToPalette[val.background];
     }
 
-    val.function = X11DRV_XROPfunction[GetROP2(physDev->hdc)-1];
+    val.function = X11DRV_XROPfunction[GetROP2(physDev->dev.hdc)-1];
     /*
     ** Let's replace GXinvert by GXxor with (black xor white)
     ** This solves the selection color and leak problems in excel
@@ -258,7 +259,7 @@ BOOL X11DRV_SetupGCForPatBlt( X11DRV_PDEVICE *physDev, GC gc, BOOL fMapColors )
     {
     case FillStippled:
     case FillOpaqueStippled:
-	if (GetBkMode(physDev->hdc)==OPAQUE) val.fill_style = FillOpaqueStippled;
+	if (GetBkMode(physDev->dev.hdc)==OPAQUE) val.fill_style = FillOpaqueStippled;
 	val.stipple = physDev->brush.pixmap;
 	mask = GCStipple;
         break;
@@ -289,10 +290,10 @@ BOOL X11DRV_SetupGCForPatBlt( X11DRV_PDEVICE *physDev, GC gc, BOOL fMapColors )
         mask = 0;
         break;
     }
-    GetBrushOrgEx( physDev->hdc, &pt );
+    GetBrushOrgEx( physDev->dev.hdc, &pt );
     val.ts_x_origin = physDev->dc_rect.left + pt.x;
     val.ts_y_origin = physDev->dc_rect.top + pt.y;
-    val.fill_rule = (GetPolyFillMode(physDev->hdc) == WINDING) ? WindingRule : EvenOddRule;
+    val.fill_rule = (GetPolyFillMode(physDev->dev.hdc) == WINDING) ? WindingRule : EvenOddRule;
     wine_tsx11_lock();
     XChangeGC( gdi_display, gc,
 	       GCFunction | GCForeground | GCBackground | GCFillStyle |
@@ -325,7 +326,7 @@ BOOL X11DRV_SetupGCForBrush( X11DRV_PDEVICE *physDev )
 static BOOL X11DRV_SetupGCForPen( X11DRV_PDEVICE *physDev )
 {
     XGCValues val;
-    UINT rop2 = GetROP2(physDev->hdc);
+    UINT rop2 = GetROP2(physDev->dev.hdc);
 
     if (physDev->pen.style == PS_NULL) return FALSE;
 
@@ -385,7 +386,7 @@ static BOOL X11DRV_SetupGCForPen( X11DRV_PDEVICE *physDev )
     }
 
     if (physDev->pen.dash_len)
-        val.line_style = ((GetBkMode(physDev->hdc) == OPAQUE) && (!physDev->pen.ext))
+        val.line_style = ((GetBkMode(physDev->dev.hdc) == OPAQUE) && (!physDev->pen.ext))
                          ? LineDoubleDash : LineOnOffDash;
     else
         val.line_style = LineSolid;
@@ -445,7 +446,7 @@ INT X11DRV_XWStoDS( X11DRV_PDEVICE *physDev, INT width )
     pt[0].y = 0;
     pt[1].x = width;
     pt[1].y = 0;
-    LPtoDP( physDev->hdc, pt, 2 );
+    LPtoDP( physDev->dev.hdc, pt, 2 );
     return pt[1].x - pt[0].x;
 }
 
@@ -462,26 +463,26 @@ INT X11DRV_YWStoDS( X11DRV_PDEVICE *physDev, INT height )
     pt[0].y = 0;
     pt[1].x = 0;
     pt[1].y = height;
-    LPtoDP( physDev->hdc, pt, 2 );
+    LPtoDP( physDev->dev.hdc, pt, 2 );
     return pt[1].y - pt[0].y;
 }
 
 /***********************************************************************
  *           X11DRV_LineTo
  */
-BOOL CDECL
-X11DRV_LineTo( X11DRV_PDEVICE *physDev, INT x, INT y )
+BOOL CDECL X11DRV_LineTo( PHYSDEV dev, INT x, INT y )
 {
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
     POINT pt[2];
 
     if (X11DRV_SetupGCForPen( physDev )) {
 	/* Update the pixmap from the DIB section */
 	X11DRV_LockDIBSection(physDev, DIB_Status_GdiMod);
 
-        GetCurrentPositionEx( physDev->hdc, &pt[0] );
+        GetCurrentPositionEx( dev->hdc, &pt[0] );
         pt[1].x = x;
         pt[1].y = y;
-        LPtoDP( physDev->hdc, pt, 2 );
+        LPtoDP( dev->hdc, pt, 2 );
 
         wine_tsx11_lock();
         XDrawLine(gdi_display, physDev->drawable, physDev->gc,
@@ -504,30 +505,29 @@ X11DRV_LineTo( X11DRV_PDEVICE *physDev, INT x, INT y )
  * 'lines' is the number of lines to draw: 0 for Arc, 1 for Chord, 2 for Pie.
  *
  */
-static BOOL
-X11DRV_DrawArc( X11DRV_PDEVICE *physDev, INT left, INT top, INT right,
-                INT bottom, INT xstart, INT ystart,
-                INT xend, INT yend, INT lines )
+static BOOL X11DRV_DrawArc( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                            INT xstart, INT ystart, INT xend, INT yend, INT lines )
 {
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
     INT xcenter, ycenter, istart_angle, idiff_angle;
     INT width, oldwidth;
     double start_angle, end_angle;
     XPoint points[4];
     BOOL update = FALSE;
     POINT start, end;
-    RECT rc = get_device_rect( physDev->hdc, left, top, right, bottom );
+    RECT rc = get_device_rect( dev->hdc, left, top, right, bottom );
 
     start.x = xstart;
     start.y = ystart;
     end.x = xend;
     end.y = yend;
-    LPtoDP(physDev->hdc, &start, 1);
-    LPtoDP(physDev->hdc, &end, 1);
+    LPtoDP(dev->hdc, &start, 1);
+    LPtoDP(dev->hdc, &end, 1);
 
     if ((rc.left == rc.right) || (rc.top == rc.bottom)
             ||(lines && ((rc.right-rc.left==1)||(rc.bottom-rc.top==1)))) return TRUE;
 
-    if (GetArcDirection( physDev->hdc ) == AD_CLOCKWISE)
+    if (GetArcDirection( dev->hdc ) == AD_CLOCKWISE)
       { POINT tmp = start; start = end; end = tmp; }
 
     oldwidth = width = physDev->pen.width;
@@ -661,47 +661,41 @@ X11DRV_DrawArc( X11DRV_PDEVICE *physDev, INT left, INT top, INT right,
 /***********************************************************************
  *           X11DRV_Arc
  */
-BOOL CDECL
-X11DRV_Arc( X11DRV_PDEVICE *physDev, INT left, INT top, INT right, INT bottom,
-            INT xstart, INT ystart, INT xend, INT yend )
+BOOL CDECL X11DRV_Arc( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                       INT xstart, INT ystart, INT xend, INT yend )
 {
-    return X11DRV_DrawArc( physDev, left, top, right, bottom,
-			   xstart, ystart, xend, yend, 0 );
+    return X11DRV_DrawArc( dev, left, top, right, bottom, xstart, ystart, xend, yend, 0 );
 }
 
 
 /***********************************************************************
  *           X11DRV_Pie
  */
-BOOL CDECL
-X11DRV_Pie( X11DRV_PDEVICE *physDev, INT left, INT top, INT right, INT bottom,
-            INT xstart, INT ystart, INT xend, INT yend )
+BOOL CDECL X11DRV_Pie( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                       INT xstart, INT ystart, INT xend, INT yend )
 {
-    return X11DRV_DrawArc( physDev, left, top, right, bottom,
-			   xstart, ystart, xend, yend, 2 );
+    return X11DRV_DrawArc( dev, left, top, right, bottom, xstart, ystart, xend, yend, 2 );
 }
 
 /***********************************************************************
  *           X11DRV_Chord
  */
-BOOL CDECL
-X11DRV_Chord( X11DRV_PDEVICE *physDev, INT left, INT top, INT right, INT bottom,
-              INT xstart, INT ystart, INT xend, INT yend )
+BOOL CDECL X11DRV_Chord( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                         INT xstart, INT ystart, INT xend, INT yend )
 {
-    return X11DRV_DrawArc( physDev, left, top, right, bottom,
-		  	   xstart, ystart, xend, yend, 1 );
+    return X11DRV_DrawArc( dev, left, top, right, bottom, xstart, ystart, xend, yend, 1 );
 }
 
 
 /***********************************************************************
  *           X11DRV_Ellipse
  */
-BOOL CDECL
-X11DRV_Ellipse( X11DRV_PDEVICE *physDev, INT left, INT top, INT right, INT bottom )
+BOOL CDECL X11DRV_Ellipse( PHYSDEV dev, INT left, INT top, INT right, INT bottom )
 {
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
     INT width, oldwidth;
     BOOL update = FALSE;
-    RECT rc = get_device_rect( physDev->hdc, left, top, right, bottom );
+    RECT rc = get_device_rect( dev->hdc, left, top, right, bottom );
 
     if ((rc.left == rc.right) || (rc.top == rc.bottom)) return TRUE;
 
@@ -754,12 +748,12 @@ X11DRV_Ellipse( X11DRV_PDEVICE *physDev, INT left, INT top, INT right, INT botto
 /***********************************************************************
  *           X11DRV_Rectangle
  */
-BOOL CDECL
-X11DRV_Rectangle(X11DRV_PDEVICE *physDev, INT left, INT top, INT right, INT bottom)
+BOOL CDECL X11DRV_Rectangle(PHYSDEV dev, INT left, INT top, INT right, INT bottom)
 {
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
     INT width, oldwidth, oldjoinstyle;
     BOOL update = FALSE;
-    RECT rc = get_device_rect( physDev->hdc, left, top, right, bottom );
+    RECT rc = get_device_rect( dev->hdc, left, top, right, bottom );
 
     TRACE("(%d %d %d %d)\n", left, top, right, bottom);
 
@@ -821,14 +815,14 @@ X11DRV_Rectangle(X11DRV_PDEVICE *physDev, INT left, INT top, INT right, INT bott
 /***********************************************************************
  *           X11DRV_RoundRect
  */
-BOOL CDECL
-X11DRV_RoundRect( X11DRV_PDEVICE *physDev, INT left, INT top, INT right,
-                  INT bottom, INT ell_width, INT ell_height )
+BOOL CDECL X11DRV_RoundRect( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                             INT ell_width, INT ell_height )
 {
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
     INT width, oldwidth, oldendcap;
     BOOL update = FALSE;
     POINT pts[2];
-    RECT rc = get_device_rect( physDev->hdc, left, top, right, bottom );
+    RECT rc = get_device_rect( dev->hdc, left, top, right, bottom );
 
     TRACE("(%d %d %d %d  %d %d\n",
     	left, top, right, bottom, ell_width, ell_height);
@@ -841,7 +835,7 @@ X11DRV_RoundRect( X11DRV_PDEVICE *physDev, INT left, INT top, INT right,
     pts[0].x = pts[0].y = 0;
     pts[1].x = ell_width;
     pts[1].y = ell_height;
-    LPtoDP(physDev->hdc, pts, 2);
+    LPtoDP(dev->hdc, pts, 2);
     ell_width  = max(abs( pts[1].x - pts[0].x ), 1);
     ell_height = max(abs( pts[1].y - pts[0].y ), 1);
 
@@ -1023,15 +1017,15 @@ X11DRV_RoundRect( X11DRV_PDEVICE *physDev, INT left, INT top, INT right,
 /***********************************************************************
  *           X11DRV_SetPixel
  */
-COLORREF CDECL
-X11DRV_SetPixel( X11DRV_PDEVICE *physDev, INT x, INT y, COLORREF color )
+COLORREF CDECL X11DRV_SetPixel( PHYSDEV dev, INT x, INT y, COLORREF color )
 {
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
     unsigned long pixel;
     POINT pt;
 
     pt.x = x;
     pt.y = y;
-    LPtoDP( physDev->hdc, &pt, 1 );
+    LPtoDP( dev->hdc, &pt, 1 );
     pixel = X11DRV_PALETTE_ToPhysical( physDev, color );
 
     /* Update the pixmap from the DIB section */
@@ -1055,18 +1049,18 @@ X11DRV_SetPixel( X11DRV_PDEVICE *physDev, INT x, INT y, COLORREF color )
 /***********************************************************************
  *           X11DRV_GetPixel
  */
-COLORREF CDECL
-X11DRV_GetPixel( X11DRV_PDEVICE *physDev, INT x, INT y )
+COLORREF CDECL X11DRV_GetPixel( PHYSDEV dev, INT x, INT y )
 {
     static Pixmap pixmap = 0;
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
     XImage * image;
     int pixel;
     POINT pt;
-    BOOL memdc = (GetObjectType(physDev->hdc) == OBJ_MEMDC);
+    BOOL memdc = (GetObjectType(dev->hdc) == OBJ_MEMDC);
 
     pt.x = x;
     pt.y = y;
-    LPtoDP( physDev->hdc, &pt, 1 );
+    LPtoDP( dev->hdc, &pt, 1 );
 
     /* Update the pixmap from the DIB section */
     X11DRV_LockDIBSection(physDev, DIB_Status_GdiMod);
@@ -1107,14 +1101,15 @@ X11DRV_GetPixel( X11DRV_PDEVICE *physDev, INT x, INT y )
 /***********************************************************************
  *           X11DRV_PaintRgn
  */
-BOOL CDECL
-X11DRV_PaintRgn( X11DRV_PDEVICE *physDev, HRGN hrgn )
+BOOL CDECL X11DRV_PaintRgn( PHYSDEV dev, HRGN hrgn )
 {
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
+
     if (X11DRV_SetupGCForBrush( physDev ))
     {
         unsigned int i;
         XRectangle *rect;
-        RGNDATA *data = X11DRV_GetRegionData( hrgn, physDev->hdc );
+        RGNDATA *data = X11DRV_GetRegionData( hrgn, dev->hdc );
 
         if (!data) return FALSE;
         rect = (XRectangle *)data->Buffer;
@@ -1137,9 +1132,9 @@ X11DRV_PaintRgn( X11DRV_PDEVICE *physDev, HRGN hrgn )
 /**********************************************************************
  *          X11DRV_Polyline
  */
-BOOL CDECL
-X11DRV_Polyline( X11DRV_PDEVICE *physDev, const POINT* pt, INT count )
+BOOL CDECL X11DRV_Polyline( PHYSDEV dev, const POINT* pt, INT count )
 {
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
     int i;
     XPoint *points;
 
@@ -1151,7 +1146,7 @@ X11DRV_Polyline( X11DRV_PDEVICE *physDev, const POINT* pt, INT count )
     for (i = 0; i < count; i++)
     {
         POINT tmp = pt[i];
-        LPtoDP(physDev->hdc, &tmp, 1);
+        LPtoDP(dev->hdc, &tmp, 1);
         points[i].x = physDev->dc_rect.left + tmp.x;
         points[i].y = physDev->dc_rect.top + tmp.y;
     }
@@ -1174,10 +1169,10 @@ X11DRV_Polyline( X11DRV_PDEVICE *physDev, const POINT* pt, INT count )
 /**********************************************************************
  *          X11DRV_Polygon
  */
-BOOL CDECL
-X11DRV_Polygon( X11DRV_PDEVICE *physDev, const POINT* pt, INT count )
+BOOL CDECL X11DRV_Polygon( PHYSDEV dev, const POINT* pt, INT count )
 {
-    register int i;
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
+    int i;
     XPoint *points;
     BOOL update = FALSE;
 
@@ -1189,7 +1184,7 @@ X11DRV_Polygon( X11DRV_PDEVICE *physDev, const POINT* pt, INT count )
     for (i = 0; i < count; i++)
     {
         POINT tmp = pt[i];
-        LPtoDP(physDev->hdc, &tmp, 1);
+        LPtoDP(dev->hdc, &tmp, 1);
         points[i].x = physDev->dc_rect.left + tmp.x;
         points[i].y = physDev->dc_rect.top + tmp.y;
     }
@@ -1226,16 +1221,16 @@ X11DRV_Polygon( X11DRV_PDEVICE *physDev, const POINT* pt, INT count )
 /**********************************************************************
  *          X11DRV_PolyPolygon
  */
-BOOL CDECL
-X11DRV_PolyPolygon( X11DRV_PDEVICE *physDev, const POINT* pt, const INT* counts, UINT polygons)
+BOOL CDECL X11DRV_PolyPolygon( PHYSDEV dev, const POINT* pt, const INT* counts, UINT polygons )
 {
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
     HRGN hrgn;
 
     /* FIXME: The points should be converted to device coords before */
     /* creating the region. */
 
-    hrgn = CreatePolyPolygonRgn( pt, counts, polygons, GetPolyFillMode( physDev->hdc ) );
-    X11DRV_PaintRgn( physDev, hrgn );
+    hrgn = CreatePolyPolygonRgn( pt, counts, polygons, GetPolyFillMode( dev->hdc ) );
+    X11DRV_PaintRgn( dev, hrgn );
     DeleteObject( hrgn );
 
       /* Draw the outline of the polygons */
@@ -1260,7 +1255,7 @@ X11DRV_PolyPolygon( X11DRV_PDEVICE *physDev, const POINT* pt, const INT* counts,
 	    for (j = 0; j < counts[i]; j++)
 	    {
                 POINT tmp = *pt;
-                LPtoDP(physDev->hdc, &tmp, 1);
+                LPtoDP(dev->hdc, &tmp, 1);
                 points[j].x = physDev->dc_rect.left + tmp.x;
                 points[j].y = physDev->dc_rect.top + tmp.y;
 		pt++;
@@ -1284,9 +1279,10 @@ X11DRV_PolyPolygon( X11DRV_PDEVICE *physDev, const POINT* pt, const INT* counts,
 /**********************************************************************
  *          X11DRV_PolyPolyline
  */
-BOOL CDECL
-X11DRV_PolyPolyline( X11DRV_PDEVICE *physDev, const POINT* pt, const DWORD* counts, DWORD polylines )
+BOOL CDECL X11DRV_PolyPolyline( PHYSDEV dev, const POINT* pt, const DWORD* counts, DWORD polylines )
 {
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
+
     if (X11DRV_SetupGCForPen ( physDev ))
     {
         unsigned int i, j, max = 0;
@@ -1306,7 +1302,7 @@ X11DRV_PolyPolyline( X11DRV_PDEVICE *physDev, const POINT* pt, const DWORD* coun
             for (j = 0; j < counts[i]; j++)
             {
                 POINT tmp = *pt;
-                LPtoDP(physDev->hdc, &tmp, 1);
+                LPtoDP(dev->hdc, &tmp, 1);
                 points[j].x = physDev->dc_rect.left + tmp.x;
                 points[j].y = physDev->dc_rect.top + tmp.y;
                 pt++;
@@ -1403,10 +1399,9 @@ static int ExtFloodFillXGetImageErrorHandler( Display *dpy, XErrorEvent *event,
 /**********************************************************************
  *          X11DRV_ExtFloodFill
  */
-BOOL CDECL
-X11DRV_ExtFloodFill( X11DRV_PDEVICE *physDev, INT x, INT y, COLORREF color,
-                     UINT fillType )
+BOOL CDECL X11DRV_ExtFloodFill( PHYSDEV dev, INT x, INT y, COLORREF color, UINT fillType )
 {
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
     XImage *image;
     RECT rect;
     POINT pt;
@@ -1415,7 +1410,7 @@ X11DRV_ExtFloodFill( X11DRV_PDEVICE *physDev, INT x, INT y, COLORREF color,
 
     pt.x = x;
     pt.y = y;
-    LPtoDP( physDev->hdc, &pt, 1 );
+    LPtoDP( dev->hdc, &pt, 1 );
     if (!PtInRegion( physDev->region, pt.x, pt.y )) return FALSE;
     GetRgnBox( physDev->region, &rect );
 
@@ -1457,9 +1452,10 @@ X11DRV_ExtFloodFill( X11DRV_PDEVICE *physDev, INT x, INT y, COLORREF color,
 /**********************************************************************
  *          X11DRV_SetBkColor
  */
-COLORREF CDECL
-X11DRV_SetBkColor( X11DRV_PDEVICE *physDev, COLORREF color )
+COLORREF CDECL X11DRV_SetBkColor( PHYSDEV dev, COLORREF color )
 {
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
+
     physDev->backgroundPixel = X11DRV_PALETTE_ToPhysical( physDev, color );
     return color;
 }
@@ -1467,9 +1463,10 @@ X11DRV_SetBkColor( X11DRV_PDEVICE *physDev, COLORREF color )
 /**********************************************************************
  *          X11DRV_SetTextColor
  */
-COLORREF CDECL
-X11DRV_SetTextColor( X11DRV_PDEVICE *physDev, COLORREF color )
+COLORREF CDECL X11DRV_SetTextColor( PHYSDEV dev, COLORREF color )
 {
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
+
     physDev->textPixel = X11DRV_PALETTE_ToPhysical( physDev, color );
     return color;
 }
@@ -1519,7 +1516,7 @@ static const WCHAR color_path[] =
 /***********************************************************************
  *              GetICMProfile (X11DRV.@)
  */
-BOOL CDECL X11DRV_GetICMProfile( X11DRV_PDEVICE *physDev, LPDWORD size, LPWSTR filename )
+BOOL CDECL X11DRV_GetICMProfile( PHYSDEV dev, LPDWORD size, LPWSTR filename )
 {
     static const WCHAR srgb[] =
         {'s','R','G','B',' ','C','o','l','o','r',' ','S','p','a','c','e',' ',
@@ -1593,8 +1590,9 @@ BOOL CDECL X11DRV_GetICMProfile( X11DRV_PDEVICE *physDev, LPDWORD size, LPWSTR f
 /***********************************************************************
  *              EnumICMProfiles (X11DRV.@)
  */
-INT CDECL X11DRV_EnumICMProfiles( X11DRV_PDEVICE *physDev, ICMENUMPROCW proc, LPARAM lparam )
+INT CDECL X11DRV_EnumICMProfiles( PHYSDEV dev, ICMENUMPROCW proc, LPARAM lparam )
 {
+    X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
     HKEY hkey;
     DWORD len_sysdir, len_path, len, index = 0;
     WCHAR sysdir[MAX_PATH], *profile;
