@@ -346,8 +346,6 @@ void X11DRV_XRender_Init(void)
     int event_base, i;
 
     if (client_side_with_render &&
-	wine_dlopen(SONAME_LIBX11, RTLD_NOW|RTLD_GLOBAL, NULL, 0) &&
-	wine_dlopen(SONAME_LIBXEXT, RTLD_NOW|RTLD_GLOBAL, NULL, 0) && 
 	(xrender_handle = wine_dlopen(SONAME_LIBXRENDER, RTLD_NOW, NULL, 0)))
     {
 
@@ -579,7 +577,7 @@ static Picture get_xrender_picture(X11DRV_PDEVICE *physDev)
                                               physDev->dc_rect.left, physDev->dc_rect.top,
                                               (XRectangle *)clip->Buffer, clip->rdh.nCount );
         wine_tsx11_unlock();
-        TRACE("Allocing pict=%lx dc=%p drawable=%08lx\n", info->pict, physDev->hdc, physDev->drawable);
+        TRACE("Allocing pict=%lx dc=%p drawable=%08lx\n", info->pict, physDev->dev.hdc, physDev->drawable);
         HeapFree( GetProcessHeap(), 0, clip );
     }
 
@@ -603,7 +601,7 @@ static Picture get_xrender_picture_source(X11DRV_PDEVICE *physDev, BOOL repeat)
         wine_tsx11_unlock();
 
         TRACE("Allocing pict_src=%lx dc=%p drawable=%08lx repeat=%u\n",
-              info->pict_src, physDev->hdc, physDev->drawable, pa.repeat);
+              info->pict_src, physDev->dev.hdc, physDev->drawable, pa.repeat);
     }
 
     return info->pict_src;
@@ -662,10 +660,7 @@ static int LookupEntry(LFANDSIZE *plfsz)
 
   for(i = mru; i >= 0; i = glyphsetCache[i].next) {
     TRACE("%d\n", i);
-    if(glyphsetCache[i].count == -1) { /* reached free list so stop */
-      i = -1;
-      break;
-    }
+    if(glyphsetCache[i].count == -1) break; /* reached free list so stop */
 
     if(!fontcmp(&glyphsetCache[i].lfsz, plfsz)) {
       glyphsetCache[i].count++;
@@ -797,14 +792,14 @@ static BOOL get_gasp_flags(X11DRV_PDEVICE *physDev, WORD *flags)
 
     *flags = 0;
 
-    size = GetFontData(physDev->hdc, MS_GASP_TAG,  0, NULL, 0);
+    size = GetFontData(physDev->dev.hdc, MS_GASP_TAG,  0, NULL, 0);
     if(size == GDI_ERROR)
         return FALSE;
 
     gasp = buffer = HeapAlloc(GetProcessHeap(), 0, size);
-    GetFontData(physDev->hdc, MS_GASP_TAG,  0, gasp, size);
+    GetFontData(physDev->dev.hdc, MS_GASP_TAG,  0, gasp, size);
 
-    GetTextMetricsW(physDev->hdc, &tm);
+    GetTextMetricsW(physDev->dev.hdc, &tm);
     ppem = abs(X11DRV_YWStoDS(physDev, tm.tmAscent + tm.tmDescent - tm.tmInternalLeading));
 
     gasp++;
@@ -1067,7 +1062,7 @@ BOOL X11DRV_XRender_SelectFont(X11DRV_PDEVICE *physDev, HFONT hfont)
     lfsz.devsize.cx = X11DRV_XWStoDS( physDev, lfsz.lf.lfWidth );
     lfsz.devsize.cy = X11DRV_YWStoDS( physDev, lfsz.lf.lfHeight );
 
-    GetTransform( physDev->hdc, 0x204, &lfsz.xform );
+    GetTransform( physDev->dev.hdc, 0x204, &lfsz.xform );
     TRACE("font transform %f %f %f %f\n", lfsz.xform.eM11, lfsz.xform.eM12,
           lfsz.xform.eM21, lfsz.xform.eM22);
 
@@ -1132,14 +1127,25 @@ BOOL X11DRV_XRender_SetPhysBitmapDepth(X_PHYSBITMAP *physBitmap, int bits_pixel,
 
     if (dib)
     {
-        X11DRV_PALETTE_ComputeColorShifts(&shifts, dib->dsBitfields[0], dib->dsBitfields[1], dib->dsBitfields[2]);
+        const DWORD *bitfields;
+        static const DWORD bitfields_32[3] = {0xff0000, 0x00ff00, 0x0000ff};
+        static const DWORD bitfields_16[3] = {0x7c00, 0x03e0, 0x001f};
+
+        if(dib->dsBmih.biCompression == BI_BITFIELDS)
+            bitfields = dib->dsBitfields;
+        else if(bits_pixel == 24 || bits_pixel == 32)
+            bitfields = bitfields_32;
+        else
+            bitfields = bitfields_16;
+
+        X11DRV_PALETTE_ComputeColorShifts(&shifts, bitfields[0], bitfields[1], bitfields[2]);
         fmt = get_xrender_format_from_color_shifts(dib->dsBm.bmBitsPixel, &shifts);
 
         /* Common formats should be in our picture format table. */
         if (!fmt)
         {
             TRACE("Unhandled dibsection format bpp=%d, redMask=%x, greenMask=%x, blueMask=%x\n",
-                dib->dsBm.bmBitsPixel, dib->dsBitfields[0], dib->dsBitfields[1], dib->dsBitfields[2]);
+                dib->dsBm.bmBitsPixel, bitfields[0], bitfields[1], bitfields[2]);
             return FALSE;
         }
     }
@@ -1196,13 +1202,13 @@ void X11DRV_XRender_UpdateDrawable(X11DRV_PDEVICE *physDev)
         XFlush( gdi_display );
         if (info->pict)
         {
-            TRACE("freeing pict = %lx dc = %p\n", info->pict, physDev->hdc);
+            TRACE("freeing pict = %lx dc = %p\n", info->pict, physDev->dev.hdc);
             pXRenderFreePicture(gdi_display, info->pict);
             info->pict = 0;
         }
         if(info->pict_src)
         {
-            TRACE("freeing pict = %lx dc = %p\n", info->pict_src, physDev->hdc);
+            TRACE("freeing pict = %lx dc = %p\n", info->pict_src, physDev->dev.hdc);
             pXRenderFreePicture(gdi_display, info->pict_src);
             info->pict_src = 0;
         }
@@ -1255,20 +1261,20 @@ static void UploadGlyph(X11DRV_PDEVICE *physDev, int glyph, AA_Type format)
 	break;
     }
 
-    buflen = GetGlyphOutlineW(physDev->hdc, glyph, ggo_format, &gm, 0, NULL, &identity);
+    buflen = GetGlyphOutlineW(physDev->dev.hdc, glyph, ggo_format, &gm, 0, NULL, &identity);
     if(buflen == GDI_ERROR) {
         if(format != AA_None) {
             format = AA_None;
             entry->aa_default = AA_None;
             ggo_format = GGO_GLYPH_INDEX | GGO_BITMAP;
-            buflen = GetGlyphOutlineW(physDev->hdc, glyph, ggo_format, &gm, 0, NULL, &identity);
+            buflen = GetGlyphOutlineW(physDev->dev.hdc, glyph, ggo_format, &gm, 0, NULL, &identity);
         }
         if(buflen == GDI_ERROR) {
             WARN("GetGlyphOutlineW failed using default glyph\n");
-            buflen = GetGlyphOutlineW(physDev->hdc, 0, ggo_format, &gm, 0, NULL, &identity);
+            buflen = GetGlyphOutlineW(physDev->dev.hdc, 0, ggo_format, &gm, 0, NULL, &identity);
             if(buflen == GDI_ERROR) {
                 WARN("GetGlyphOutlineW failed for default glyph trying for space\n");
-                buflen = GetGlyphOutlineW(physDev->hdc, 0x20, ggo_format, &gm, 0, NULL, &identity);
+                buflen = GetGlyphOutlineW(physDev->dev.hdc, 0x20, ggo_format, &gm, 0, NULL, &identity);
                 if(buflen == GDI_ERROR) {
                     ERR("GetGlyphOutlineW for all attempts unable to upload a glyph\n");
                     return;
@@ -1350,7 +1356,7 @@ static void UploadGlyph(X11DRV_PDEVICE *physDev, int glyph, AA_Type format)
 
 
     buf = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, buflen);
-    GetGlyphOutlineW(physDev->hdc, glyph, ggo_format, &gm, buflen, buf, &identity);
+    GetGlyphOutlineW(physDev->dev.hdc, glyph, ggo_format, &gm, buflen, buf, &identity);
     formatEntry->realized[glyph] = TRUE;
 
     TRACE("buflen = %d. Got metrics: %dx%d adv=%d,%d origin=%d,%d\n",
@@ -1779,6 +1785,22 @@ static int XRenderErrorHandler(Display *dpy, XErrorEvent *event, void *arg)
     return 1;
 }
 
+/********************************************************************
+ *                   is_dib_with_colortable
+ *
+ * Return TRUE if physdev is backed by a dibsection with <= 8 bits per pixel
+ */
+static inline BOOL is_dib_with_colortable( X11DRV_PDEVICE *physDev )
+{
+    DIBSECTION dib;
+
+    if( physDev->bitmap && GetObjectW( physDev->bitmap->hbitmap, sizeof(dib), &dib ) == sizeof(dib) &&
+        dib.dsBmih.biBitCount <= 8 )
+        return TRUE;
+
+    return FALSE;
+}
+
 /***********************************************************************
  *   X11DRV_XRender_ExtTextOut
  */
@@ -1794,16 +1816,12 @@ BOOL X11DRV_XRender_ExtTextOut( X11DRV_PDEVICE *physDev, INT x, INT y, UINT flag
     HRGN saved_region = 0;
     BOOL disable_antialias = FALSE;
     AA_Type aa_type = AA_None;
-    DIBSECTION bmp;
     unsigned int idx;
     const WineXRenderFormat *dst_format = get_xrender_format_from_color_shifts(physDev->depth, physDev->color_shifts);
     Picture tile_pict = 0;
 
-    /* Do we need to disable antialiasing because of palette mode? */
-    if( !physDev->bitmap || GetObjectW( physDev->bitmap->hbitmap, sizeof(bmp), &bmp ) != sizeof(bmp) ) {
-        TRACE("bitmap is not a DIB\n");
-    }
-    else if (bmp.dsBmih.biBitCount <= 8) {
+    if(is_dib_with_colortable( physDev ))
+    {
         TRACE("Disabling antialiasing\n");
         disable_antialias = TRUE;
     }
@@ -1854,7 +1872,7 @@ BOOL X11DRV_XRender_ExtTextOut( X11DRV_PDEVICE *physDev, INT x, INT y, UINT flag
         /* make a copy of the current device region */
         saved_region = CreateRectRgn( 0, 0, 0, 0 );
         CombineRgn( saved_region, physDev->region, 0, RGN_COPY );
-        X11DRV_SetDeviceClipping( physDev, saved_region, clip_region );
+        X11DRV_SetDeviceClipping( &physDev->dev, saved_region, clip_region );
         DeleteObject( clip_region );
     }
 
@@ -2116,7 +2134,7 @@ BOOL X11DRV_XRender_ExtTextOut( X11DRV_PDEVICE *physDev, INT x, INT y, UINT flag
     if (flags & ETO_CLIPPED)
     {
         /* restore the device region */
-        X11DRV_SetDeviceClipping( physDev, saved_region, 0 );
+        X11DRV_SetDeviceClipping( &physDev->dev, saved_region, 0 );
         DeleteObject( saved_region );
     }
 
