@@ -30,17 +30,15 @@
 #include <ctype.h>
 #include <wine/unicode.h>
 
-/* Data structure to hold commands delimitors/separators */
+/*	Data structure to hold commands to be processed */
 
 typedef enum _CMDdelimiters {
   CMD_NONE,        /* End of line or single & */
   CMD_ONFAILURE,   /* ||                      */
   CMD_ONSUCCESS,   /* &&                      */
-  CMD_PIPE         /* Single |                */
+  CMD_PIPE         /* Single                  */
 } CMD_DELIMITERS;
 
-/* Data structure to hold commands to be processed */
-
 typedef struct _CMD_LIST {
   WCHAR              *command;     /* Command string to execute                */
   WCHAR              *redirects;   /* Redirects in place                       */
@@ -50,15 +48,15 @@ typedef struct _CMD_LIST {
   WCHAR               pipeFile[MAX_PATH]; /* Where to get input from for pipes */
 } CMD_LIST;
 
-void WCMD_assoc (const WCHAR *, BOOL);
+void WCMD_assoc (WCHAR *, BOOL);
 void WCMD_batch (WCHAR *, WCHAR *, int, WCHAR *, HANDLE);
 void WCMD_call (WCHAR *command);
 void WCMD_change_tty (void);
-void WCMD_choice (const WCHAR *);
+void WCMD_choice (WCHAR *);
 void WCMD_clear_screen (void);
 void WCMD_color (void);
 void WCMD_copy (void);
-void WCMD_create_dir (WCHAR *);
+void WCMD_create_dir (void);
 BOOL WCMD_delete (WCHAR *);
 void WCMD_directory (WCHAR *);
 void WCMD_echo (const WCHAR *);
@@ -66,7 +64,7 @@ void WCMD_endlocal (void);
 void WCMD_enter_paged_mode(const WCHAR *);
 void WCMD_exit (CMD_LIST **cmdList);
 void WCMD_for (WCHAR *, CMD_LIST **cmdList);
-void WCMD_give_help (const WCHAR *command);
+void WCMD_give_help (WCHAR *command);
 void WCMD_goto (CMD_LIST **cmdList);
 void WCMD_if (WCHAR *, CMD_LIST **cmdList);
 void WCMD_leave_paged_mode(void);
@@ -74,63 +72,61 @@ void WCMD_more (WCHAR *);
 void WCMD_move (void);
 void WCMD_output (const WCHAR *format, ...);
 void WCMD_output_asis (const WCHAR *message);
-void WCMD_output_asis_stderr (const WCHAR *message);
 void WCMD_pause (void);
 void WCMD_popd (void);
 void WCMD_print_error (void);
 void WCMD_pushd (WCHAR *);
+int  WCMD_read_console (WCHAR *string, int str_len);
 void WCMD_remove_dir (WCHAR *command);
 void WCMD_rename (void);
 void WCMD_run_program (WCHAR *command, int called);
 void WCMD_setlocal (const WCHAR *command);
+void WCMD_setshow_attrib (void);
 void WCMD_setshow_date (void);
-void WCMD_setshow_default (const WCHAR *command);
+void WCMD_setshow_default (WCHAR *command);
 void WCMD_setshow_env (WCHAR *command);
-void WCMD_setshow_path (const WCHAR *command);
+void WCMD_setshow_path (WCHAR *command);
 void WCMD_setshow_prompt (void);
 void WCMD_setshow_time (void);
-void WCMD_shift (const WCHAR *command);
-void WCMD_title (const WCHAR *);
+void WCMD_shift (WCHAR *command);
+void WCMD_title (WCHAR *);
 void WCMD_type (WCHAR *);
-void WCMD_verify (const WCHAR *command);
+void WCMD_verify (WCHAR *command);
 void WCMD_version (void);
-int  WCMD_volume (int mode, const WCHAR *command);
+int  WCMD_volume (int mode, WCHAR *command);
 
 WCHAR *WCMD_fgets (WCHAR *s, int n, HANDLE stream);
-WCHAR *WCMD_parameter (WCHAR *s, int n, WCHAR **where, WCHAR **end);
-WCHAR *WCMD_skip_leading_spaces (WCHAR *string);
-BOOL WCMD_keyword_ws_found(const WCHAR *keyword, int len, const WCHAR *ptr);
-void WCMD_HandleTildaModifiers(WCHAR **start, const WCHAR *forVariable, const WCHAR *forValue, BOOL justFors);
+WCHAR *WCMD_parameter (WCHAR *s, int n, WCHAR **where);
+WCHAR *WCMD_strtrim_leading_spaces (WCHAR *string);
+void WCMD_HandleTildaModifiers(WCHAR **start, WCHAR *forVariable, WCHAR *forValue, BOOL justFors);
 
 void WCMD_splitpath(const WCHAR* path, WCHAR* drv, WCHAR* dir, WCHAR* name, WCHAR* ext);
 void WCMD_opt_s_strip_quotes(WCHAR *cmd);
 WCHAR *WCMD_LoadMessage(UINT id);
-WCHAR *WCMD_strdupW(const WCHAR *input);
-void WCMD_strsubstW(WCHAR *start, const WCHAR* next, const WCHAR* insert, int len);
+WCHAR *WCMD_strdupW(WCHAR *input);
+void WCMD_strsubstW(WCHAR *start, WCHAR* next, WCHAR* insert, int len);
 BOOL WCMD_ReadFile(const HANDLE hIn, WCHAR *intoBuf, const DWORD maxChars,
                    LPDWORD charsRead, const LPOVERLAPPED unused);
 
-WCHAR    *WCMD_ReadAndParseLine(const WCHAR *initialcmd, CMD_LIST **output, HANDLE readFrom);
-CMD_LIST *WCMD_process_commands(CMD_LIST *thisCmd, BOOL oneBracket,
-                                const WCHAR *var, const WCHAR *val);
+WCHAR    *WCMD_ReadAndParseLine(WCHAR *initialcmd, CMD_LIST **output, HANDLE readFrom);
+CMD_LIST *WCMD_process_commands(CMD_LIST *thisCmd, BOOL oneBracket, WCHAR *var, WCHAR *val);
 void      WCMD_free_commands(CMD_LIST *cmds);
-void      WCMD_execute (const WCHAR *orig_command, const WCHAR *redirects,
-                        const WCHAR *parameter, const WCHAR *substitution,
-                        CMD_LIST **cmdList);
+void      WCMD_execute (WCHAR *orig_command, WCHAR *redirects, WCHAR *parameter,
+                        WCHAR *substitution, CMD_LIST **cmdList);
 
-/* Data structure to hold context when executing batch files */
+/*	Data structure to hold context when executing batch files */
 
-typedef struct _BATCH_CONTEXT {
+typedef struct {
   WCHAR *command;	/* The command which invoked the batch file */
   HANDLE h;             /* Handle to the open batch file */
   WCHAR *batchfileW;    /* Name of same */
   int shift_count[10];	/* Offset in terms of shifts for %0 - %9 */
-  struct _BATCH_CONTEXT *prev_context; /* Pointer to the previous context block */
+  void *prev_context;	/* Pointer to the previous context block */
   BOOL  skip_rest;      /* Skip the rest of the batch program and exit */
   CMD_LIST *toExecute;  /* Commands left to be executed */
 } BATCH_CONTEXT;
 
-/* Data structure to handle building lists during recursive calls */
+/* Data structure to save setlocal and pushd information */
 
 struct env_stack
 {
@@ -142,7 +138,7 @@ struct env_stack
   WCHAR *strings;
 };
 
-/* Data structure to save setlocal and pushd information */
+/* Data structure to handle building lists during recursive calls */
 
 typedef struct _DIRECTORY_STACK
 {
@@ -155,61 +151,62 @@ typedef struct _DIRECTORY_STACK
 
 /*
  *	Serial nos of builtin commands. These constants must be in step with
- *	the list of strings defined in wcmd.rc, and WCMD_EXIT *must* always be
+ *	the list of strings defined in WCMD.C, and WCMD_EXIT *must* always be
  *	the last one.
  *
  *	Yes it *would* be nice to use an enumeration here, but the Resource
  *	Compiler won't accept resource IDs from enumerations :-(
  */
 
-#define WCMD_CALL      0
-#define WCMD_CD        1
-#define WCMD_CHDIR     2
-#define WCMD_CLS       3
-#define WCMD_COPY      4
-#define WCMD_CTTY      5
-#define WCMD_DATE      6
-#define WCMD_DEL       7
-#define WCMD_DIR       8
-#define WCMD_ECHO      9
-#define WCMD_ERASE    10
-#define WCMD_FOR      11
-#define WCMD_GOTO     12
-#define WCMD_HELP     13
-#define WCMD_IF       14
-#define WCMD_LABEL    15
-#define WCMD_MD       16
-#define WCMD_MKDIR    17
-#define WCMD_MOVE     18
-#define WCMD_PATH     19
-#define WCMD_PAUSE    20
-#define WCMD_PROMPT   21
-#define WCMD_REM      22
-#define WCMD_REN      23
-#define WCMD_RENAME   24
-#define WCMD_RD       25
-#define WCMD_RMDIR    26
-#define WCMD_SET      27
-#define WCMD_SHIFT    28
-#define WCMD_TIME     29
-#define WCMD_TITLE    30
-#define WCMD_TYPE     31
-#define WCMD_VERIFY   32
-#define WCMD_VER      33
-#define WCMD_VOL      34
+#define WCMD_ATTRIB  0
+#define WCMD_CALL    1
+#define WCMD_CD      2
+#define WCMD_CHDIR   3
+#define WCMD_CLS     4
+#define WCMD_COPY    5
+#define WCMD_CTTY    6
+#define WCMD_DATE    7
+#define WCMD_DEL     8
+#define WCMD_DIR     9
+#define WCMD_ECHO   10
+#define	WCMD_ERASE  11
+#define WCMD_FOR    12
+#define WCMD_GOTO   13
+#define WCMD_HELP   14
+#define WCMD_IF     15
+#define WCMD_LABEL  16
+#define	WCMD_MD     17
+#define WCMD_MKDIR  18
+#define WCMD_MOVE   19
+#define WCMD_PATH   20
+#define WCMD_PAUSE  21
+#define WCMD_PROMPT 22
+#define	WCMD_REM    23
+#define WCMD_REN    24
+#define WCMD_RENAME 25
+#define WCMD_RD     26
+#define WCMD_RMDIR  27
+#define WCMD_SET    28
+#define	WCMD_SHIFT  29
+#define WCMD_TIME   30
+#define WCMD_TITLE  31
+#define WCMD_TYPE   32
+#define WCMD_VERIFY 33
+#define WCMD_VER    34
+#define WCMD_VOL    35
 
-#define WCMD_ENDLOCAL 35
-#define WCMD_SETLOCAL 36
-#define WCMD_PUSHD    37
-#define WCMD_POPD     38
-#define WCMD_ASSOC    39
-#define WCMD_COLOR    40
-#define WCMD_FTYPE    41
-#define WCMD_MORE     42
-#define WCMD_CHOICE   43
+#define WCMD_ENDLOCAL 36
+#define WCMD_SETLOCAL 37
+#define WCMD_PUSHD  38
+#define WCMD_POPD   39
+#define WCMD_ASSOC  40
+#define WCMD_COLOR  41
+#define WCMD_FTYPE  42
+#define WCMD_MORE   43
+#define WCMD_CHOICE 44
 
 /* Must be last in list */
-#define WCMD_EXIT     44
+#define WCMD_EXIT   45
 
 /* Some standard messages */
 extern const WCHAR newline[];
