@@ -21,7 +21,6 @@
  */
 
 #define COBJMACROS
-#define NONAMELESSUNION
 
 #include "wine/unicode.h"
 #include "wine/debug.h"
@@ -32,7 +31,6 @@
 #include <windows.h>
 #include <shobjidl.h>
 #include <shlobj.h>
-#include <shlwapi.h>
 #include <commoncontrols.h>
 #include <commctrl.h>
 
@@ -63,9 +61,8 @@ typedef struct
     IExplorerBrowser *browser;
     HWND main_window,path_box;
     INT rebar_height;
-    LPITEMIDLIST pidl;
+    LPCITEMIDLIST pidl;
     IImageList *icon_list;
-    DWORD advise_cookie;
 } explorer_info;
 
 enum
@@ -125,7 +122,7 @@ static BOOL create_combobox_item(IShellFolder *folder, LPCITEMIDLIST pidl, IImag
     switch(strret.uType)
     {
     case STRRET_WSTR:
-        item->pszText = strret.u.pOleStr;
+        item->pszText = strret.pOleStr;
         break;
     default:
         WINE_FIXME("Unimplemented STRRET type:%u\n",strret.uType);
@@ -141,7 +138,6 @@ static BOOL create_combobox_item(IShellFolder *folder, LPCITEMIDLIST pidl, IImag
                                       &icon_index,&icon_flags);
         IExtractIconW_Extract(extract_icon,icon_file,icon_index,NULL,&icon,20);
         item->iImage = ImageList_AddIcon((HIMAGELIST)icon_list,icon);
-        IExtractIconW_Release(extract_icon);
     }
     else
     {
@@ -182,14 +178,12 @@ static void update_path_box(explorer_info *info)
     if(SUCCEEDED(IShellFolder_EnumObjects(desktop,NULL,SHCONTF_FOLDERS,&ids))
        && ids!=NULL)
     {
-        LPITEMIDLIST curr_pidl=NULL;
+        LPITEMIDLIST curr_pidl;
         HRESULT hres;
 
         item.iIndent = 1;
         while(1)
         {
-            ILFree(curr_pidl);
-            curr_pidl=NULL;
             hres = IEnumIDList_Next(ids,1,&curr_pidl,NULL);
             if(FAILED(hres) || hres == S_FALSE)
                 break;
@@ -250,8 +244,6 @@ static void update_path_box(explorer_info *info)
                     CoTaskMemFree(item.pszText);
             }
         }
-        ILFree(curr_pidl);
-        IEnumIDList_Release(ids);
     }
     else
         WINE_WARN("Could not enumerate the desktop\n");
@@ -262,8 +254,7 @@ static void update_path_box(explorer_info *info)
 static HRESULT WINAPI IExplorerBrowserEventsImpl_fnOnNavigationComplete(IExplorerBrowserEvents *iface, PCIDLIST_ABSOLUTE pidl)
 {
     IExplorerBrowserEventsImpl *This = impl_from_IExplorerBrowserEvents(iface);
-    ILFree(This->info->pidl);
-    This->info->pidl = ILClone(pidl);
+    This->info->pidl = pidl;
     update_path_box(This->info);
     return S_OK;
 }
@@ -300,9 +291,9 @@ static IExplorerBrowserEvents *make_explorer_events(explorer_info *info)
         = HeapAlloc(GetProcessHeap(), 0, sizeof(IExplorerBrowserEventsImpl));
     ret->IExplorerBrowserEvents_iface.lpVtbl = &vt_IExplorerBrowserEvents;
     ret->info = info;
-    ret->ref = 1;
     SHGetImageList(SHIL_SMALL,&IID_IImageList,(void**)&(ret->info->icon_list));
     SendMessageW(info->path_box,CBEM_SETIMAGELIST,0,(LPARAM)ret->info->icon_list);
+    IExplorerBrowserEvents_AddRef(&ret->IExplorerBrowserEvents_iface);
     return &ret->IExplorerBrowserEvents_iface;
 }
 
@@ -314,6 +305,7 @@ static void make_explorer_window(IShellFolder* startFolder)
     IExplorerBrowserEvents *events;
     explorer_info *info;
     HRESULT hres;
+    DWORD cookie;
     WCHAR explorer_title[100];
     WCHAR pathbox_label[50];
     TBADDBITMAP bitmap_info;
@@ -325,7 +317,7 @@ static void make_explorer_window(IShellFolder* startFolder)
                 sizeof(explorer_title)/sizeof(WCHAR));
     LoadStringW(explorer_hInstance,IDS_PATHBOX_LABEL,pathbox_label,
                 sizeof(pathbox_label)/sizeof(WCHAR));
-    info = HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,sizeof(explorer_info));
+    info = HeapAlloc(GetProcessHeap(),0,sizeof(explorer_info));
     if(!info)
     {
         WINE_ERR("Could not allocate a explorer_info struct\n");
@@ -333,7 +325,7 @@ static void make_explorer_window(IShellFolder* startFolder)
     }
     hres = CoCreateInstance(&CLSID_ExplorerBrowser,NULL,CLSCTX_INPROC_SERVER,
                             &IID_IExplorerBrowser,(LPVOID*)&info->browser);
-    if(FAILED(hres))
+    if(!SUCCEEDED(hres))
     {
         WINE_ERR("Could not obtain an instance of IExplorerBrowser\n");
         HeapFree(GetProcessHeap(),0,info);
@@ -409,7 +401,7 @@ static void make_explorer_window(IShellFolder* startFolder)
     band_info.hwndChild=info->path_box;
     SendMessageW(rebar,RB_INSERTBANDW,-1,(LPARAM)&band_info);
     events = make_explorer_events(info);
-    IExplorerBrowser_Advise(info->browser,events,&info->advise_cookie);
+    IExplorerBrowser_Advise(info->browser,events,&cookie);
     IExplorerBrowser_BrowseToObject(info->browser,(IUnknown*)startFolder,
                                     SBSP_ABSOLUTE);
     ShowWindow(info->main_window,SW_SHOWDEFAULT);
@@ -539,13 +531,8 @@ static LRESULT CALLBACK explorer_wnd_proc(HWND hwnd, UINT uMsg, WPARAM wParam, L
     switch(uMsg)
     {
     case WM_DESTROY:
-        IExplorerBrowser_Unadvise(browser,info->advise_cookie);
-        IExplorerBrowser_Destroy(browser);
         IExplorerBrowser_Release(browser);
-        ILFree(info->pidl);
-        IImageList_Release(info->icon_list);
         HeapFree(GetProcessHeap(),0,info);
-        SetWindowLongPtrW(hwnd,EXPLORER_INFO_INDEX,0);
         PostQuitMessage(0);
         break;
     case WM_QUIT:
@@ -751,14 +738,11 @@ int WINAPI wWinMain(HINSTANCE hinstance,
     explorer_hInstance = hinstance;
     parse_command_line(cmdline,&parameters);
     hres = OleInitialize(NULL);
-    if(FAILED(hres))
+    if(!SUCCEEDED(hres))
     {
         WINE_ERR("Could not initialize COM\n");
         ExitProcess(EXIT_FAILURE);
     }
-    if(parameters.root[0] && !PathIsDirectoryW(parameters.root))
-        if(ShellExecuteW(NULL,NULL,parameters.root,NULL,NULL,SW_SHOWDEFAULT) > (HINSTANCE)32)
-            ExitProcess(EXIT_SUCCESS);
     init_info.dwSize = sizeof(INITCOMMONCONTROLSEX);
     init_info.dwICC = ICC_USEREX_CLASSES | ICC_BAR_CLASSES | ICC_COOL_CLASSES;
     if(!InitCommonControlsEx(&init_info))
