@@ -39,7 +39,7 @@
 static int indentation = 0;
 static int is_object_interface = 0;
 user_type_list_t user_type_list = LIST_INIT(user_type_list);
-static context_handle_list_t context_handle_list = LIST_INIT(context_handle_list);
+context_handle_list_t context_handle_list = LIST_INIT(context_handle_list);
 generic_handle_list_t generic_handle_list = LIST_INIT(generic_handle_list);
 
 static void write_type_def_or_decl(FILE *f, type_t *t, int field, const char *name);
@@ -994,10 +994,10 @@ static void write_function_proto(FILE *header, const type_t *iface, const var_t
 {
   const char *callconv = get_attrp(fun->type->attrs, ATTR_CALLCONV);
 
+  if (!callconv) callconv = "__cdecl";
   /* FIXME: do we need to handle call_as? */
   write_type_decl_left(header, type_function_get_rettype(fun->type));
-  fprintf(header, " ");
-  if (callconv) fprintf(header, "%s ", callconv);
+  fprintf(header, " %s ", callconv);
   fprintf(header, "%s%s(\n", prefix, get_name(fun));
   if (type_get_function_args(fun->type))
     write_args(header, type_get_function_args(fun->type), iface->name, 0, TRUE);
@@ -1152,6 +1152,9 @@ static void write_coclass(FILE *header, type_t *cocl)
   fprintf(header, " * %s coclass\n", cocl->name);
   fprintf(header, " */\n\n");
   write_coclass_guid(header, cocl);
+  fprintf(header, "\n#ifdef __cplusplus\n");
+  fprintf(header, "class %s;\n", cocl->name);
+  fprintf(header, "#endif\n");
   fprintf(header, "\n");
 }
 
@@ -1264,7 +1267,7 @@ static void write_header_stmts(FILE *header, const statement_list_t *stmts, cons
             write_header_stmts(header, type_iface_get_stmts(iface), iface, FALSE);
             write_rpc_interface_end(header, iface);
           }
-          if (is_object(iface)) is_object_interface++;
+          if (is_object(iface)) is_object_interface--;
         }
         else if (type_get_type(stmt->u.type) == TYPE_COCLASS)
           write_coclass(header, stmt->u.type);
@@ -1338,6 +1341,11 @@ void write_header(const statement_list_t *stmts)
   fprintf(header, "#include <rpc.h>\n" );
   fprintf(header, "#include <rpcndr.h>\n\n" );
 
+  fprintf(header, "#if !defined(COM_NO_WINDOWS_H) && !defined(__WINESRC__)\n");
+  fprintf(header, "#include <windows.h>\n");
+  fprintf(header, "#include <ole2.h>\n");
+  fprintf(header, "#endif\n\n");
+
   fprintf(header, "#ifndef __WIDL_%s\n", header_token);
   fprintf(header, "#define __WIDL_%s\n\n", header_token);
 
